@@ -19,7 +19,7 @@
 
 #ifndef lint
 static char rcsid[]="$Header: /ufs/repository/magic/calma/CalmaRdpt.c,v 1.5 2001/07/23 17:49:49 tim Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #include <sys/types.h>
@@ -42,8 +42,15 @@ static char rcsid[]="$Header: /ufs/repository/magic/calma/CalmaRdpt.c,v 1.5 2001
 #include "misc/styles.h"
 #include "textio/textio.h"
 #include "calma/calmaInt.h"
+#include "calma/calma.h"
 
 extern int calmaNonManhattan;
+
+#ifdef NONMANHATTAN
+extern int CalmaPolygonCount;
+extern HashTable calmaDefInitHash;
+#endif
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -68,6 +75,9 @@ calmaElementBoundary()
     CIFPath *pathheadp;
     LinkedRect *rp;
     Plane *plane;
+#ifdef NONMANHATTAN
+    CellDef *savedef, *newdef = NULL;
+#endif
 
     /* Skip CALMA_ELFLAGS, CALMA_PLEX */
     calmaSkipSet(calmaElementIgnore);
@@ -97,30 +107,79 @@ calmaElementBoundary()
     }
 
     /* Convert the polygon to rectangles. */
+
 #ifdef NONMANHATTAN
-    rp = CIFPolyToRects(pathheadp, plane);
+    if (CalmaSubcellPolygons && (calmaNonManhattan > 0))
+    {
+	/* Place the polygon in its own subcell */
+	char newname[] = "polygonXXXXX";
+	HashEntry *he;
+
+	savedef = cifReadCellDef;
+
+	/* Make up name for cell */
+	sprintf(newname + 7, "%05d", ++CalmaPolygonCount);
+
+	he = HashFind(&calmaDefInitHash, newname);
+	if (!HashGetValue(he))
+	{
+	    newdef = calmaFindCell(newname);
+	    cifReadCellDef = newdef;
+	    DBCellClearDef(cifReadCellDef);
+	    DBCellSetAvail(cifReadCellDef);
+
+	    /* cifEditCellPlanes is not used by the gds reader, so it's */
+	    /* available to be used to store the polygon.		*/
+
+	    cifCurReadPlanes = cifEditCellPlanes;
+	    if (plane != NULL)
+		plane = cifCurReadPlanes[ciftype];
+	}
+    }
+
+    rp = CIFPolyToRects(pathheadp, plane, CIFPaintTable, (PaintUndoInfo *)NULL);
 #else
     rp = CIFPolyToRects(pathheadp);
 #endif
     CIFFreePath(pathheadp);
-    if (rp == NULL)
-    {
-        /* The non-Manhattan geometry polygon parsing algorithm */
-        /* typically leaves behind degenerate paths, so they    */
-        /* should not be considered erroneous.                  */
+
 #ifndef NONMANHATTAN
+    /* The non-Manhattan geometry polygon parsing algorithm */
+    /* typically leaves behind degenerate paths, so they    */
+    /* should not be considered erroneous.                  */
+    if (rp == NULL)
 	calmaReadError("Can't convert boundary/box into rects; ignored.\n");
 #endif
-	return;
-    }
 
-    /* Paint the rectangles */
+    /* Paint the rectangles (if any) */
     for (; rp != NULL ; rp = rp->r_next)
     {
 	if (plane)
 	    DBPaintPlane(plane, &rp->r_r, CIFPaintTable, (PaintUndoInfo *)NULL);
 	FREE((char *) rp);
     }
+
+#ifdef NONMANHATTAN
+    if (cifCurReadPlanes == cifEditCellPlanes)
+    {
+	CellUse *use;
+
+	CIFPaintCurrent();
+	DBReComputeBbox(cifReadCellDef);
+	DRCCheckThis(cifReadCellDef, TT_CHECKPAINT, &cifReadCellDef->cd_bbox);
+	DBWAreaChanged(cifReadCellDef, &cifReadCellDef->cd_bbox,
+		DBW_ALLWINDOWS, &DBAllButSpaceBits);
+	DBCellSetModified(cifReadCellDef, TRUE);
+	DBGenerateUniqueIds(cifReadCellDef, FALSE);  /* Is this necessary? */
+
+	cifCurReadPlanes = cifSubcellPlanes;
+	cifReadCellDef = savedef;
+
+	use = DBCellNewUse(newdef, (char *)NULL);
+	DBSetTrans(use, &GeoIdentityTransform);
+	DBPlaceCell(use, cifReadCellDef);
+    }
+#endif
 }
 
 /*
@@ -308,7 +367,7 @@ calmaElementPath()
 
 	if (xmin != xmax && ymin != ymax)
 	{
-	     calmaReadError("Non-orthgonal path; ignored.\n");
+	     calmaReadError("Non-orthogonal path; ignored.\n");
 	     return;
 	}
 	if (first == FALSE && last == FALSE)
@@ -493,10 +552,13 @@ calmaElementText()
 	    }
 	}
 	if (changed) {
-	    TxError("Warning:  weird characters fixed in label '%s'\n", textbody);
+	    calmaReadError("Warning:  weird characters fixed in label '%s'\n",
+			textbody);
 	    if (!algmsg) {
 		algmsg = TRUE;
-		TxError("    (algorithm used:  trailing <CR> dropped, <CR> and ' ' changed to '_', \n    other non-printables changed to '?')\n");
+		calmaReadError("    (algorithm used:  trailing <CR> dropped, "
+				"<CR> and ' ' changed to '_', \n"
+				"    other non-printables changed to '?')\n");
 	    }
 	}
     }
@@ -504,7 +566,7 @@ calmaElementText()
     /* Place the label */
     if (type >= 0)
     {
-    	 (void) DBPutLabel(cifReadCellDef, &r, -1, textbody, type);
+    	 (void) DBPutLabel(cifReadCellDef, &r, -1, textbody, type, 0);
     }
 
     /* done with textbody */
@@ -561,7 +623,7 @@ calmaReadPath(pathheadpp)
     {
 	READPOINT(&path.cifp_point);
 	if (ABS(path.cifp_x) > 5000000 || ABS(path.cifp_y) > 5000000) {
-	    TxError("Warning:  Very large point in path:  (%d, %d)\n",
+	    calmaReadError("Warning:  Very large point in path:  (%d, %d)\n",
 		path.cifp_x, path.cifp_y);
 	}
 	if (feof(calmaInputFile))
@@ -587,9 +649,7 @@ calmaReadPath(pathheadpp)
 	    {
 		if (!nonManhattan)
 		{
-#ifndef NONMANHATTAN
 		    calmaNonManhattan++;
-#endif
 		    nonManhattan = TRUE;
 		}
 	    }
