@@ -296,7 +296,7 @@ CIFNameToMask(name, result)
     if (!CIFCurStyle)
     {
 	TxError("No CIF output style set!\n");
-	return(FALSE);
+	return FALSE;
     }
 
     TTMaskZero(result);
