@@ -1042,7 +1042,7 @@ CIFTechInputScale(n, d)
     CIFReadStyle *istyle = cifCurReadStyle;
     CIFReadLayer *cl;
     CIFOp *op;
-    int lmult, i, j, lgcf;
+    int lmult, i, lgcf;
 
     /* fprintf(stderr, "CIF input style %s:\n", istyle->crs_name); */
 
