@@ -502,6 +502,77 @@ CIFParseInteger(valuep)
 	CIFReadError("negative integer not permitted.\n");
     return TRUE;
 }
+
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * CIFInputRescale --
+ *
+ *	Scale all CIF distances by n / d.  Normally, rescaling is done in
+ *	the upward direction (n > 1, d = 1) in response to a value in the
+ *	CIF input that does not divide evenly into the database units of
+ *	the CIF planes.  Currently there is no call to CIFInputRescale
+ *	with d > 1, but it is left for the possibility that scalefactors
+ *	could be restored after finishing a subcell definition, possibly
+ *	preventing integer overflow in large designs.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	Many.  Adjusts the CIF input style scalefactor (CIF vs. magic
+ *	units), multiplier (CIF units vs. centimicrons), distances of all
+ *	CIF boolean operations, and the scale of all existing planes in
+ *	the current CIF database.
+ *
+ * ----------------------------------------------------------------------------
+ */
+ 
+void
+CIFInputRescale(n, d)
+    int n, d;
+{
+    CIFReadStyle *istyle = cifCurReadStyle;
+    CIFReadLayer *cl;
+    CIFOp *op;
+    int i;
+
+    /* 2-step process for efficiency */
+
+    if (n > 1)
+    {
+	istyle->crs_scaleFactor *= n;
+	istyle->crs_multiplier *= n;
+
+	for (i = 0; i < istyle->crs_nLayers; i++)
+	{
+	    cl = istyle->crs_layers[i];
+	    for (op = cl->crl_ops; op != NULL; op = op->co_next)
+		if (op->co_distance)
+		    op->co_distance *= n;
+	}
+    }
+
+    if (d > 1)
+    {
+	istyle->crs_scaleFactor /= d;
+	istyle->crs_multiplier /= d;
+
+	for (i = 0; i < istyle->crs_nLayers; i++)
+	{
+	    cl = istyle->crs_layers[i];
+	    for (op = cl->crl_ops; op != NULL; op = op->co_next)
+		if (op->co_distance)
+		    op->co_distance /= d;
+	}
+    }
+
+    CIFScalePlanes(n, d);
+    CIFReadWarning("CIF style %s: units rescaled by factor of %d / %d\n",
+		istyle->crs_name, n, d);
+}
+
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -558,11 +629,8 @@ CIFParsePoint(pointp, iscale)
 	}
 	else
 	{
-	    cifCurReadStyle->crs_scaleFactor *= rescale;
-	    cifCurReadStyle->crs_multiplier *= rescale;
 	    cifReadScale1 *= rescale;
-	    CIFReadWarning("CIF units rescaled by factor of %d\n", rescale);
-	    CIFScalePlanes(rescale, 1);
+	    CIFInputRescale(rescale, 1);
 	    pointp->p_x *= rescale;
 	}
     }
@@ -584,11 +652,8 @@ CIFParsePoint(pointp, iscale)
 	}
 	else
 	{
-	    cifCurReadStyle->crs_scaleFactor *= rescale;
-	    cifCurReadStyle->crs_multiplier *= rescale;
 	    cifReadScale1 *= rescale;
-	    CIFReadWarning("CIF units rescaled by factor of %d\n", rescale);
-	    CIFScalePlanes(rescale, 1);
+	    CIFInputRescale(rescale, 1);
 	    pointp->p_x *= rescale;
 	    pointp->p_y *= rescale;
 	}
