@@ -188,6 +188,7 @@ cifParseLayers(string, style, paintMask, cifMask,spaceOK)
     char curLayer[40], *p, *cp;
     TileType paintType;
     int i;
+    bool allResidues;
 
     if (paintMask != NULL) TTMaskZero(paintMask);
     if (cifMask != NULL) TTMaskZero(cifMask);
@@ -195,6 +196,15 @@ cifParseLayers(string, style, paintMask, cifMask,spaceOK)
     while (*string != 0)
     {
 	p = curLayer;
+
+	if (*string == '*')
+	{
+	    allResidues = TRUE;
+	    string++;
+	}
+	else
+	    allResidues = FALSE;
+
 	while ((*string != ',') && (*string != 0))
 	    *p++ = *string++;
 	*p = 0;
@@ -246,7 +256,15 @@ okpaint:
 		TechError("\"Space\" layer not permitted in CIF rules.\n");
 	    else
 	    {
-		 TTMaskSetMask(paintMask, &curPaintMask);
+		TileType rtype;
+
+		TTMaskSetMask(paintMask, &curPaintMask);
+
+		/* Add residues from '*' notation */
+		if (allResidues)
+		    for (rtype = TT_TECHDEPBASE; rtype < DBNumUserLayers; rtype++)
+			if (TTMaskHasType(DBResidueMask(rtype), paintType))
+			    TTMaskSetType(paintMask, rtype);
 	    } 
 	}
 	else if (!TTMaskEqual(&curCifMask, &DBZeroTypeBits))
@@ -821,6 +839,7 @@ wrongNumArgs:
 		local_bloats[0] = i/2;
 		local_bloats[1] = i;
 		local_bloats[2] = i;
+		local_bloats[3] = 0;	/* placeholder; not used by SQUARES */
 	    }
 	    else if (argc == 4 || 
 		      (argc==5 && newOp->co_opcode==CIFOP_SQUARES_G))
@@ -1090,6 +1109,7 @@ CIFTechFinal()
      * If cs_expander > 1, then all CIF op values must be scaled accordingly.
      * This routine loops through all CIF output styles.
      */
+
     CIFTechOutputScale(1, 1);
 
     if (style->cs_scaleFactor <= 0)
@@ -1109,7 +1129,7 @@ CIFTechFinal()
     {
 	for (op = style->cs_layers[i]->cl_ops; op != NULL; op = op->co_next)
 	{
-	    int j, c;
+	    int j, c, bvalue;
 	    if (op->co_distance > 0)
 	    {
 		c = FindGCF(style->cs_scaleFactor, op->co_distance);
@@ -1117,15 +1137,34 @@ CIFTechFinal()
 	    }
 	    if (op->co_bloats)
 	    {
-		int nlayers = TT_MAXTYPES;
-		if (op->co_opcode == CIFOP_SQUARES || op->co_opcode ==
-				CIFOP_SQUARES_G) nlayers = 4;
-		for (j = 0; j < nlayers; j++)
+		if (op->co_opcode == CIFOP_SQUARES || op->co_opcode == CIFOP_SQUARES_G)
 		{
-		    if (op->co_bloats[j] != 0)
+		    for (j = 0; j < 4; j++)
 		    {
-			c = FindGCF(style->cs_scaleFactor, op->co_bloats[j]);
-			if (c < minReduce) minReduce = c;
+			bvalue = op->co_bloats[j];
+			if (bvalue != 0)
+			{
+			    if ((j == 1) || (j == 2))
+			    {
+				if (bvalue & 0x1)
+				    TxError("Internal error: contact size/sep %d"
+						" cannot be halved.\n", bvalue);
+				bvalue >>= 1;
+			    }
+			    c = FindGCF(style->cs_scaleFactor, bvalue);
+			    if (c < minReduce) minReduce = c;
+			}
+		    }
+		}
+		else
+		{
+		    for (j = 0; j < TT_MAXTYPES; j++)
+		    {
+			if (op->co_bloats[j] != 0)
+			{
+			    c = FindGCF(style->cs_scaleFactor, op->co_bloats[j]);
+			    if (c < minReduce) minReduce = c;
+			}
 		    }
 		}
 	    }
@@ -1326,6 +1365,12 @@ CIFLoadStyle(stylename)
  *   CIF output as small as possible while making sure all output can be
  *   represented by integers.
  *
+ *   Note that because contacts may be placed at 1/2 grid spacing to
+ *   center them, the size and spacing of contacts as given in the 
+ *   "squares" function must *always* be an even number.  To ensure this,
+ *   we check for odd numbers in these positions.  If there are any, and
+ *   "d" is also an odd number, then we multiply both "n" and "d" by 2.
+ *
  * ----------------------------------------------------------------------------
  */
 
@@ -1333,14 +1378,38 @@ Void
 CIFTechOutputScale(n, d)
     int n, d;
 {
-    int i, j, savescale, lgcf, lexpand;
+    int i, j, lgcf, lexpand;
     CIFStyle *ostyle = CIFCurStyle;
     CIFLayer *cl;
     CIFOp *op;
 
+    /* For contact half-grid centering, check for odd numbers. . . */
+    if (d & 0x1)
+    {
+	bool has_odd_space = FALSE;
+	for (i = 0; i < ostyle->cs_nLayers; i++)
+	{
+	    cl = ostyle->cs_layers[i];
+	    for (op = cl->cl_ops; op != NULL; op = op->co_next)
+		if ((op->co_opcode == CIFOP_SQUARES)
+				|| (op->co_opcode == CIFOP_SQUARES_G))
+		    if ((op->co_bloats[1] & 0x01) || (op->co_bloats[2] & 0x01))
+		    {
+			has_odd_space = TRUE;
+			break;
+		    }
+
+	    if (has_odd_space)
+	    {
+		n *= 2;
+		d *= 2;
+		break;
+	    }
+	}
+    }
+
     /* fprintf(stderr, "CIFTechOutputScale(%d, %d)\n", n, d); */
 
-    savescale = ostyle->cs_scaleFactor;
     ostyle->cs_scaleFactor *= n;
     ostyle->cs_expander *= d;
 
@@ -1357,24 +1426,40 @@ CIFTechOutputScale(n, d)
 		op->co_distance *= d;
 		lgcf = FindGCF(abs(op->co_distance), ostyle->cs_expander);
 		lexpand = FindGCF(lexpand, lgcf);
-		if (lexpand == 1) break;
 	    }
 	    if (op->co_bloats)
 	    {
-		int nlayers = TT_MAXTYPES;
-		if (op->co_opcode == CIFOP_SQUARES || op->co_opcode ==
-				CIFOP_SQUARES_G) nlayers = 4;
-		for (j = 0; j < nlayers; j++)
+		int bvalue;
+		if ((op->co_opcode == CIFOP_SQUARES) || (op->co_opcode ==
+				CIFOP_SQUARES_G))
 		{
-		    if (op->co_bloats[j] != 0)
+		    for (j = 0; j < 4; j++)
 		    {
-			op->co_bloats[j] *= d;
-		        lgcf = FindGCF(abs(op->co_bloats[j]), ostyle->cs_expander);
-			lexpand = FindGCF(lexpand, lgcf);
-		        if (lexpand == 1) break;
+			if (op->co_bloats[j] != 0)
+			{
+			    op->co_bloats[j] *= d;
+			    bvalue = abs(op->co_bloats[j]);
+			    if ((j == 1) || (j == 2)) bvalue >>= 1;	/* half-grid */
+		            lgcf = FindGCF(bvalue, ostyle->cs_expander);
+			    lexpand = FindGCF(lexpand, lgcf);
+			}
+		    }
+		}
+		else
+		{
+		    for (j = 0; j < TT_MAXTYPES; j++)
+		    {
+			if (op->co_bloats[j] != 0)
+			{
+			    op->co_bloats[j] *= d;
+		            lgcf = FindGCF(abs(op->co_bloats[j]), ostyle->cs_expander);
+			    lexpand = FindGCF(lexpand, lgcf);
+			}
 		    }
 		}
 	    }
+	    /* time-saving quick check */
+	    if ((lexpand == 1) && (d == 1)) break;
 	}
     }
 
@@ -1382,16 +1467,16 @@ CIFTechOutputScale(n, d)
     /* factor of everything.						*/
 
     /* fprintf(stderr, "All CIF units divisible by %d\n", lexpand);	*/
+    /* fflush(stderr); */
 
     lgcf = FindGCF(ostyle->cs_scaleFactor, ostyle->cs_expander);
     if (lgcf < lexpand) lexpand = lgcf;
-    if (lexpand == 0) return;
+    if (lexpand <= 1) return;
 
     /* fprintf(stderr, "Expander goes from %d to %d\n", ostyle->cs_expander,
-     *	ostyle->cs_expander / lexpand);
-     * fprintf(stderr, "All CIF op distances are divided by %d\n", lexpand);
-     * fflush(stderr);
-     */
+     	ostyle->cs_expander / lexpand); */
+    /* fprintf(stderr, "All CIF op distances are divided by %d\n", lexpand); */
+    /* fflush(stderr); */
 
     ostyle->cs_scaleFactor /= lexpand;
     ostyle->cs_expander /= lexpand;
