@@ -18,7 +18,7 @@
 
 #ifndef lint
 static char rcsid[] = "$Header: /ufs/repository/magic/cif/CIFwrite.c,v 1.4 2001/03/29 19:50:04 tim Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdlib.h>
 #include <stdio.h>
