@@ -531,7 +531,16 @@ CmdBox(w, cmd)
     else
     {
 	option = Lookup(cmd->tx_argv[1], cmdBoxOption);
-	if (option < 0) option = BOX_DEFAULT;
+
+	/* This hack allows 'h' to be used as a synonym for "height"	*/
+	/* 'w' already works for width because no other options	begin	*/
+	/* with "w".  But the addition of option "help" presents an	*/
+	/* ambiguity.							*/
+
+	if (option == -1 && cmd->tx_argv[1][0] == 'h')
+	    option = BOX_HEIGHT;
+	else if (option < 0)
+	    option = BOX_DEFAULT;
     }
 
     windCheckOnlyWindow(&w, DBWclientID);
