@@ -18,7 +18,7 @@
 
 #ifndef lint
 static char rcsid[] = "$Header: /ufs/repository/magic/commands/CmdFI.c,v 1.4 2001/12/10 16:34:16 tim Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #include <string.h>
@@ -28,11 +28,14 @@ static char rcsid[] = "$Header: /ufs/repository/magic/commands/CmdFI.c,v 1.4 200
 #include <readline/readline.h>
 #include <readline/history.h>
 #else
-#include "readline/readline.h"
-#include "readline/history.h"
+#include "readline/readline/readline.h"
+#include "readline/readline/history.h"
 #endif
 #endif
 
+#ifdef MAGIC_WRAPPER
+#include "magic/tclmagic.h"
+#endif
 #include "misc/magic.h"
 #include "utils/geometry.h"
 #include "utils/utils.h"
@@ -329,7 +332,7 @@ CmdFill(w, cmd)
 	return;
     }
 
-    windCheckOnlyWindow(&w);
+    windCheckOnlyWindow(&w, DBWclientID);
     if ( w == (MagWindow *) NULL )
     {
 	TxError("Point to a window\n");
@@ -675,7 +678,7 @@ CmdFlush(w, cmd)
     if (def->cd_flags & (CDMODIFIED|CDSTAMPSCHANGED|CDBOXESCHANGED))
     {
 	prompt = TxPrintString("Really throw away all changes made"
-			" to cell \"%s\"? ", def->cd_name);
+			" to cell %s? ", def->cd_name);
 	action = TxDialog(prompt, actionNames, 0);
 	if (action == 0)	/* No */
 	    return;
@@ -860,7 +863,7 @@ CmdGetnode(w, cmd)
 	    goto badusage;
     }
 
-    windCheckOnlyWindow(&w);
+    windCheckOnlyWindow(&w, DBWclientID);
     if ((w == (MagWindow *) NULL) || (w->w_client != DBWclientID))
     {
 	TxError("Put the cursor in a layout window\n");
@@ -894,12 +897,12 @@ badusage:
  *
  * CmdGrid --
  *
- * Implement the "grid" command.
+ * Implement the "gridspace" command.
  * Toggle the grid on or off in the selected window.
  *
  * Usage:
- *	grid [spacing [spacing [xorig yorig]]]
- *	grid off
+ *	gridspace [spacing [spacing [xorig yorig]]]
+ *	gridspace off
  *
  * Results:
  *	None.
@@ -918,7 +921,7 @@ CmdGrid(w, cmd)
     int xSpacing, ySpacing, xOrig, yOrig;
     DBWclientRec *crec;
 
-    windCheckOnlyWindow(&w);
+    windCheckOnlyWindow(&w, DBWclientID);
     if (w == (MagWindow *) NULL) return;
     crec = (DBWclientRec *) w->w_clientData;
 
@@ -1138,18 +1141,21 @@ CmdGoto(w, cmd)
     MagWindow *w;
     TxCommand *cmd;
 {
-     char		*nodename = cmd->tx_argv[1];
-     char		*s,*s2;
-     SearchContext	scx,scx2;
-     CellUse		*use;
-     Transform		trans,newtrans,tmp;
-     Label		*label;
-     Rect		localrect,rect;
-     Point		pt;
-     CellDef		*rootdef;
-     int		pnum,xpos,ypos;
+    char		*nodename = cmd->tx_argv[1];
+    char		*s,*s2;
+    SearchContext	scx,scx2;
+    CellUse		*use;
+    Transform		trans, newtrans, tmp;
+    Label		*label;
+    Rect		localrect, rect;
+    Point		pt;
+    CellDef		*rootdef;
+    int			pnum, xpos, ypos;
+    char 		*xstr, *ystr;
+    bool		locvalid;
+    TileType		ttype;
      
-    windCheckOnlyWindow(&w);
+    windCheckOnlyWindow(&w, DBWclientID);
     if ((w == (MagWindow *) NULL) || (w->w_client != DBWclientID))
     {
 	TxError("Put the cursor in a layout window\n");
@@ -1158,63 +1164,168 @@ CmdGoto(w, cmd)
     scx.scx_use = (CellUse  *) w->w_surfaceID;
     rootdef = scx.scx_use->cu_def;
     scx.scx_trans = GeoIdentityTransform;
-     if (cmd->tx_argc != 2)
-     {
-     	  TxError("usage: goto nodename\n");
-	  return;
-     }
-     s = nodename;
-     trans = GeoIdentityTransform;
-     while (s2 = strchr(s,'/'))
-     {
-     	  *s2 = '\0';
-	  DBTreeFindUse(s,scx.scx_use,&scx2);
-	  use = scx2.scx_use;
-	  if (use == NULL)
-	  {
-	       TxError("Couldn't find use %s\n",s);
-	       return;
-	  }
-	  GeoTransTrans(DBGetArrayTransform(use,scx2.scx_x,scx2.scx_y)
+    if (cmd->tx_argc != 2)
+    {
+     	TxError("usage: goto nodename\n");
+	return;
+    }
+    s = nodename;
+    trans = GeoIdentityTransform;
+    use = scx.scx_use;
+    while (s2 = strchr(s,'/'))
+    {
+     	*s2 = '\0';
+	DBTreeFindUse(s,scx.scx_use,&scx2);
+	use = scx2.scx_use;
+	if (use == NULL)
+	{
+	    TxError("Couldn't find use %s\n",s);
+	    return;
+	}
+	GeoTransTrans(DBGetArrayTransform(use,scx2.scx_x,scx2.scx_y)
 	  		,&use->cu_transform,&tmp);
-	  GeoTransTrans(&tmp,&trans,&newtrans);
-	  trans = newtrans;
-	  scx = scx2;
-          *s2 = '/';
-	  s = s2+1;
-     }
-     /* check to see if this name is automatically generated */
-     if (sscanf(s,"%d_%d_%d",&pnum,&xpos,&ypos) == 3)
-     {
-     	  xpos = ((xpos & 0x1)?-1:1)*xpos/2;
-     	  ypos = ((ypos & 0x1)?-1:1)*ypos/2;
-	  localrect.r_xbot = xpos;
-     	  localrect.r_ybot = ypos;
-     	  localrect.r_xtop = xpos;
-     	  localrect.r_ytop = ypos;
-	  TxPrintf("node %s is on the plane \"%s\"\n",
-	  			s,DBPlaneLongNameTbl[pnum]);
-     }
-     else
-     {
-          for (label = scx.scx_use->cu_def->cd_labels;label;label=label->lab_next)
-          {
-     	       if (strcmp(label->lab_text,s) == 0) break;
-          }
-          if (label)
-          {
-	       localrect = label->lab_rect;
-          }
-          else
-          {
-     	       TxError("Couldn't find label %s\n",s);
-	       return;
-          }
-     }
-     GeoTransRect(&trans,&localrect,&rect);
-     ToolMoveBox(TOOL_BL,&rect.r_ll,FALSE,rootdef);
-     ToolMoveCorner(TOOL_TR,&rect.r_ur,FALSE,rootdef);
+	GeoTransTrans(&tmp,&trans,&newtrans);
+	trans = newtrans;
+	scx = scx2;
+        *s2 = '/';
+	s = s2+1;
+    }
+
+    /* If this node name is in the format of automatically-generated */
+    /* node names, then parse the node string for X and Y values and */
+    /* go to that point (transformed to the top level of the design  */
+    /* hierarchy).						     */
+
+    /* see extract/extractInt.h for the format of the node, found in */
+    /* extMakeNodeNumPrint(), which is a macro, not a subroutine.    */
+
+    locvalid = FALSE;
+    if ((xstr = strchr(s, '_')) != NULL)
+    {
+	bool isNeg = FALSE;
+
+        /* The characters up to the leading '_' should match one of the	*/
+	/* "short names" for a plane in this technology.		*/ 
+
+	*xstr = '\0';
+	for (pnum = PL_TECHDEPBASE; pnum < DBNumPlanes; pnum++)
+	    if (!strcmp(s, DBPlaneShortName(pnum)))
+		break;
+	*xstr = '_';
+	if (pnum != DBNumPlanes)
+	{
+	    xstr++;
+	    if (*xstr == 'n')
+	    {
+	        isNeg = TRUE;
+	        xstr++;
+	    }
+	    if (sscanf(xstr, "%d", &xpos) == 1)
+	    {
+	        if (isNeg) xpos = -xpos;
+	        if ((ystr = strchr(xstr, '_')) != NULL)
+	        {
+		    isNeg = FALSE;
+		    ystr++;
+		    if (*ystr == 'n')
+		    {
+		        isNeg = TRUE;
+		        ystr++;
+		    }
+		    if (sscanf(ystr, "%d", &ypos) == 1)
+		    {
+		        if (isNeg) ypos = -ypos;
+		        localrect.r_xbot = xpos;
+		        localrect.r_ybot = ypos;
+		        localrect.r_xtop = xpos + 1;
+		        localrect.r_ytop = ypos + 1;
+			/* TxPrintf("Node is on the plane \"%s\"\n", 
+					DBPlaneLongNameTbl[pnum]); */
+		        locvalid = TRUE;
+		    }
+	        }
+	    }
+	}
+    }
+
+    /* This is the original version, and assumes a format for node	*/
+    /* coordinates that is no longer generated by magic.  It is kept	*/
+    /* here for backward compatibility.					*/
+
+    if ((locvalid == FALSE) && (sscanf(s,"%d_%d_%d",&pnum,&xpos,&ypos) == 3))
+    {
+	xpos = ((xpos & 0x1)?-1:1)*xpos/2;
+	ypos = ((ypos & 0x1)?-1:1)*ypos/2;
+	localrect.r_xbot = xpos;
+	localrect.r_ybot = ypos;
+	localrect.r_xtop = xpos + 1;
+	localrect.r_ytop = ypos + 1;
+	locvalid = TRUE;
+    }
+
+    if (locvalid == TRUE)
+    {
+	int findTile();
+	CellDef	 *targetdef = use->cu_def;
+	Plane *plane = targetdef->cd_planes[pnum];
+
+	ttype = TT_SPACE;	/* revert to space in case of failure */
+	
+	/* Find the tile type of the tile at the specified point which	*/
+	/* exists on the plane pnum.					*/
+
+	(void) TiSrArea(NULL, plane, &localrect, findTile, (ClientData) &ttype);
+    }
+    else
+    {
+	for (label = scx.scx_use->cu_def->cd_labels;label;label=label->lab_next)
+	{
+	    if (strcmp(label->lab_text,s) == 0) break;
+	}
+	if (label)
+	{
+	    localrect = label->lab_rect;
+	    ttype = label->lab_type;
+	}
+	else
+	{
+     	    TxError("Couldn't find label %s\n",s);
+	    return;
+	}
+    }
+    GeoTransRect(&trans, &localrect, &rect);
+    ToolMoveBox(TOOL_BL, &rect.r_ll, FALSE, rootdef);
+    ToolMoveCorner(TOOL_TR, &rect.r_ur, FALSE, rootdef);
+
+    /* Return the tile type so we know what we're looking at if there	*/
+    /* are multiple layers drawn at the indicated point.		*/ 
+
+#ifdef MAGIC_WRAPPER
+    Tcl_SetResult(magicinterp, DBTypeLongName(ttype), NULL);
+#else
+    TxPrintf("node %s is type %s\n", s, DBTypeLongName(ttype));
+#endif
 }
+
+/*
+ * ----------------------------------------------------------------------------
+ * Filter function for finding the tile type of the specified node
+ * ----------------------------------------------------------------------------
+ */
+
+int
+findTile(tile, rtype)
+    Tile *tile;
+    TileType *rtype;
+{
+    TileType ttype;
+    Rect rect;
+
+    ttype = TiGetTypeExact(tile);
+    *rtype = ttype;
+    return 1;			/* stop search */
+}
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -1260,6 +1371,7 @@ CmdFlatten(w, cmd)
     ASSERT(newdef, "CmdFlatten");
     DBCellSetAvail(newdef);
     newuse = DBCellNewUse(newdef, (char *) NULL);
+    (void) StrDup(&(newuse->cu_id), "Flattened cell");
     DBSetTrans(newuse, &GeoIdentityTransform);
     newuse->cu_expandMask = -1;
     UndoDisable();
@@ -1326,7 +1438,8 @@ flatCopyAllLabels(scx, lab, tpath, targetUse)
     	 char *f = tpath->tp_first;
 	 char c = *n;
 	 strcpy(n,lab->lab_text);
-         (void) DBPutLabel(def, &labTargetRect, targetPos,f,lab->lab_type);
+         (void) DBPutLabel(def, &labTargetRect, targetPos,
+			f, lab->lab_type, lab->lab_flags);
 	 *n = c;
     }
     return 0;
