@@ -199,7 +199,7 @@ CmdLabel(w, cmd)
  * Implement the "load" command.
  *
  * Usage:
- *	load [name]
+ *	load [name [scaled n [d]]]
  *
  * If name is supplied, then the window containing the point tool is
  * remapped so as to edit the cell with the given name.
@@ -209,6 +209,9 @@ CmdLabel(w, cmd)
  * that name in existence (eg, in another window), that cell gets loaded
  * rather than a new cell being created.
  *
+ * An input file can be scaled by specifying the "scaled" option, for
+ * which the geometry of the input file is multiplied by n/d.
+ *
  * Results:
  *	None.
  *
@@ -223,6 +226,9 @@ CmdLoad(w, cmd)
     MagWindow *w;
     TxCommand *cmd;
 {
+    int n = 1;
+    int d = 1;
+
     windCheckOnlyWindow(&w, DBWclientID);
     if (w == (MagWindow *) NULL)
     {
@@ -232,15 +238,44 @@ CmdLoad(w, cmd)
 
     if (cmd->tx_argc > 2)
     {
-	TxError("Usage: %s [name]\n", cmd->tx_argv[0]);
-	return;
+	if (cmd->tx_argc >= 4 && !strncmp(cmd->tx_argv[2], "scale", 5) &&
+		StrIsInt(cmd->tx_argv[3]))
+	{
+	    n = atoi(cmd->tx_argv[3]);
+ 	    if (cmd->tx_argc == 5 && StrIsInt(cmd->tx_argv[4]))
+	        d = atoi(cmd->tx_argv[4]);
+	    else
+	    {
+		TxError("Usage: %s name scaled n [d]\n", cmd->tx_argv[0]);
+		return;
+	    }
+	    DBLambda[0] *= d;
+	    DBLambda[1] *= n;
+	    ReduceFraction(&DBLambda[0], &DBLambda[1]);
+	}
+	else
+	{
+	    TxError("Usage: %s [name [scaled n [d]]]\n", cmd->tx_argv[0]);
+	    return;
+	}
     }
 
-    if (cmd->tx_argc == 2)
+    if (cmd->tx_argc > 1)
     {
 	if (CmdIllegalChars(cmd->tx_argv[1], "[],", "Cell name"))
 	    return;
 	DBWloadWindow(w, cmd->tx_argv[1]);
+
+	if ((n > 1) || (d > 1))
+	{
+	    /* We don't want to save and restore DBLambda, because      */
+	    /* loading the file may change their values.  Instead, we   */
+	    /* invert the scaling factor and reapply.                   */
+
+	    DBLambda[0] *= n;
+	    DBLambda[1] *= d;
+	    ReduceFraction(&DBLambda[0], &DBLambda[1]);
+	}
     }
     else DBWloadWindow(w, (char *) NULL);
 }
@@ -1182,7 +1217,7 @@ CmdProperty(w, cmd)
     else if (cmd->tx_argc == 2)
     {
 	/* print the value of the indicated property */
-	value = DBPropGet(editDef, cmd->tx_argv[1], &propfound);
+	value = (char *)DBPropGet(editDef, cmd->tx_argv[1], &propfound);
 	if (propfound)
 #ifdef MAGIC_WRAPPER
 	    Tcl_SetResult(magicinterp, value, NULL);
