@@ -19,7 +19,7 @@
 
 #ifndef lint
 static char rcsid[] = "$Header: /ufs/repository/magic/commands/CmdSubrs.c,v 1.5 2001/10/19 13:41:06 tim Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -52,6 +52,7 @@ TileTypeBitMask CmdYMLabel;
 TileTypeBitMask CmdYMCell;
 TileTypeBitMask CmdYMAllButSpace;
 
+#define round(a) (((a) < 0) ? (int)((a) - 0.5) : (int)((a) + 0.5)) 
 
 /*
  * ----------------------------------------------------------------------------
@@ -101,13 +102,13 @@ cmdParseCoord(arg)
 	/* lambda or default units */
 	dval *= (double)DBLambda[1];
 	dval /= (double)DBLambda[0];
-	return (int)(dval + 0.5);	/* Always round result to nearest integer */
+	return round(dval);
     }
     else if ((*endptr == 'i')
 		|| ((*endptr == '\0') && (DBWSnapToGrid != DBW_SNAP_LAMBDA)))
     {
 	/* internal units */
-        return (int)(dval + 0.5);
+	return round(dval);
     }
     else
     {
@@ -131,7 +132,7 @@ cmdParseCoord(arg)
 		default:
 		    TxError("Unknown metric prefix \"%cm\"; assuming internal units\n",
 				*endptr);
-		    return (int)(dval + 0.5);
+		    return round(dval);
 	    }
 	}
 	else if (!strncmp(endptr, "micron", 5))
@@ -142,10 +143,11 @@ cmdParseCoord(arg)
 	{
 	    TxError("Unknown coordinate type \"%s\"; assuming internal units\n",
 			endptr);
-	    return (int)(dval + 0.5);
+	    return round(dval);
 	}
     }
-    return (int)((dval / CIFGetOutputScale(mscale)) + 0.5);
+    dval /= CIFGetOutputScale(mscale);
+    return round(dval);
 }
 
 
@@ -736,7 +738,7 @@ CmdSetWindCaption(newEditUse, rootDef)
 {
     int cmdWindSet();
 
-    newEditDef = newEditUse->cu_def;
+    newEditDef = (newEditUse) ? newEditUse->cu_def : NULL;
     newRootDef = rootDef;
     (void) WindSearch(DBWclientID, (ClientData) NULL, (Rect *) NULL, 
 	    cmdWindSet, (ClientData) 0);
