@@ -144,6 +144,15 @@ CmdTech(w, cmd)
 	    Tcl_AppendElement(magicinterp, DBTechDescription);
 	    break;
 	case TECH_LAMBDA:
+	    if ((cmd->tx_argc > 2) && (StrIsInt(cmd->tx_argv[2])))
+	    {
+		DBLambda[1] = atoi(cmd->tx_argv[2]);
+		if (cmd->tx_argc == 3)
+		    DBLambda[0] = 1;
+		else if ((cmd->tx_argc > 3) && (StrIsInt(cmd->tx_argv[3])))
+		DBLambda[0] = atoi(cmd->tx_argv[3]);
+		ReduceFraction(&DBLambda[0], &DBLambda[1]);
+	    }
 	    lobj = Tcl_NewListObj(0, NULL);
  	    Tcl_ListObjAppendElement(magicinterp, lobj,
 			Tcl_NewIntObj(DBLambda[0]));
@@ -167,6 +176,15 @@ CmdTech(w, cmd)
 	    TxPrintf("Description: %s\n", DBTechDescription);
 	    break;
 	case TECH_LAMBDA:
+	    if ((cmd->tx_argc > 2) && (StrIsInt(cmd->tx_argv[2])))
+	    {
+		DBLambda[1] = atoi(cmd->tx_argv[2]);
+		if (cmd->tx_argc == 3)
+		    DBLambda[0] = 1;
+		else if ((cmd->tx_argc > 3) && (StrIsInt(cmd->tx_argv[3])))
+		DBLambda[0] = atoi(cmd->tx_argv[3]);
+		ReduceFraction(&DBLambda[0], &DBLambda[1]);
+	    }
 	    TxPrintf("Internal units per Lambda = %d / %d\n",
 		DBLambda[0], DBLambda[1]);
 	    break;
@@ -621,8 +639,6 @@ CmdWhat(w, cmd)
     foundAny = FALSE;
     if (labelBlockTop)
     {
-	int i;
-
 	qsort(labelBlockTop, labelEntryCount, sizeof(LabelStore), orderLabelFunc);
     		/* now print them out */
 	for (labelEntry = labelBlockTop; labelEntryCount-- > 0; labelEntry++)
