@@ -18,7 +18,7 @@
 
 #ifndef	lint
 static char rcsid[] = "$Header: /ufs/repository/magic/database/DBcell.c,v 1.2 2001/01/12 22:12:19 jsolomon Exp $";
-#endif	not lint
+#endif	/* not lint */
 
 #include <sys/types.h>
 #include <stdio.h>
@@ -233,7 +233,7 @@ placeCellFunc (tile, arg)
 
 #ifdef	CELLDEBUG
     if (dbCellDebug) TxPrintf("placeCellFunc called %x\n",tile);
-#endif	CELLDEBUG
+#endif	/* CELLDEBUG */
 
     tp = clipCellTile (tile, arg->plane, arg->rect);
 
@@ -298,7 +298,7 @@ deleteCellFunc (tile, arg)
 
 #ifdef	CELLDEBUG
     if (dbCellDebug) TxPrintf("deleteCellFunc called %x\n",tile);
-#endif	CELLDEBUG
+#endif	/* CELLDEBUG */
 
     celluse = arg->celluse;
 
@@ -370,7 +370,7 @@ clipCellTile (tile, plane, rect)
     {
 #ifdef	CELLDEBUG
 	if (dbCellDebug) TxPrintf("clipCellTile calls TiSplitY TOP\n");
-#endif	CELLDEBUG
+#endif	/* CELLDEBUG */
 
         newtile = TiSplitY (tile, rect->r_ytop);	/* no merge */
 	dupTileBody (tile, newtile);
@@ -379,7 +379,7 @@ clipCellTile (tile, plane, rect)
     {
 #ifdef	CELLDEBUG
 	if (dbCellDebug) TxPrintf("clipCellTile calls TiSplitY BOTTOM\n");
-#endif	CELLDEBUG
+#endif	/* CELLDEBUG */
 
 	newtile = tile;
         tile = TiSplitY (tile, rect->r_ybot);		/* no merge */
@@ -389,7 +389,7 @@ clipCellTile (tile, plane, rect)
     {
 #ifdef	CELLDEBUG
 	if (dbCellDebug) TxPrintf("clipCellTile calls TiSplitX RIGHT\n");
-#endif	CELLDEBUG
+#endif	/* CELLDEBUG */
 
         newtile = TiSplitX (tile, rect->r_xtop);
 	dupTileBody (tile, newtile);
@@ -399,7 +399,7 @@ clipCellTile (tile, plane, rect)
     {
 #ifdef	CELLDEBUG
 	if (dbCellDebug) TxPrintf("clipCellTile calls TiSplitX LEFT\n");
-#endif	CELLDEBUG
+#endif	/* CELLDEBUG */
 
 	newtile = tile;
         tile = TiSplitX (tile, rect->r_xbot);
@@ -479,7 +479,7 @@ cellTileMerge (tile, plane, direction)
 
 #ifdef	CELLDEBUG
     if (dbCellDebug) TxPrintf("cellTileMerge %x\n",tile);
-#endif	CELLDEBUG
+#endif	/* CELLDEBUG */
 
     topleft.p_x = LEFT(tile);
     topleft.p_y = TOP(tile);
@@ -493,7 +493,7 @@ cellTileMerge (tile, plane, direction)
 
 #ifdef	CELLDEBUG
 	if (dbCellDebug) TxPrintf("LEFT %x %x\n",tpleft,tpright);
-#endif	CELLDEBUG
+#endif	/* CELLDEBUG */
 
 	while (BOTTOM(tpleft) < topleft.p_y)	/* go up left edge */
 	{
@@ -546,7 +546,7 @@ cellTileMerge (tile, plane, direction)
    
 #ifdef	CELLDEBUG
 	if (dbCellDebug) TxPrintf("RIGHT %x %x\n",tpleft,tpright);
-#endif	CELLDEBUG
+#endif	/* CELLDEBUG */
 
 	while (BOTTOM(tpright) < topleft.p_y)	/* go up right edge */
 	{
@@ -609,7 +609,7 @@ cellTileMerge (tile, plane, direction)
 
 #ifdef	CELLDEBUG
 	if (dbCellDebug) TxPrintf("TOP %x %x\n",tp1,tp2);
-#endif	CELLDEBUG
+#endif	/* CELLDEBUG */
 
 	if ((LEFT(tp1) == LEFT(tp2)  ) &&
 	    (RIGHT(tp1) == RIGHT(tp2)) &&
@@ -633,7 +633,7 @@ cellTileMerge (tile, plane, direction)
 
 #ifdef	CELLDEBUG
 	if (dbCellDebug) TxPrintf("BOTTOM %x %x\n",tp1,tp2);
-#endif	CELLDEBUG
+#endif	/* CELLDEBUG */
 
 	if ((LEFT(tp1) == LEFT(tp2)  ) &&
 	    (RIGHT(tp1) == RIGHT(tp2)) &&
