@@ -18,7 +18,7 @@
 
 #ifndef lint
 static char rcsid[] = "$Header: /ufs/repository/magic/database/DBcellsrch.c,v 1.3 2001/03/29 19:50:05 tim Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #include "misc/magic.h"
@@ -104,18 +104,9 @@ DBTreeSrTiles(scx, mask, xMask, func, cdarg)
     ClientData cdarg;		/* Client data for above function */
 {
     int dbCellTileSrFunc();
-    TreeContext context;
     TreeFilter filter;
-    CellUse *cellUse = scx->scx_use;
-    CellDef *def = cellUse->cu_def;
-    int pNum;
 
-    ASSERT(def != (CellDef *) NULL, "DBTreeSrTiles");
-    if (!DBIsExpand(cellUse, xMask))
-	return 0;
-
-    if ((def->cd_flags & CDAVAILABLE) == 0)
-	if (!DBCellRead(def, (char *) NULL, TRUE)) return 0;
+    /* Set up the filter and call the recursive filter function */
 
     filter.tf_func = func;
     filter.tf_arg = cdarg;
@@ -123,36 +114,13 @@ DBTreeSrTiles(scx, mask, xMask, func, cdarg)
     filter.tf_xmask = xMask;
     filter.tf_planes = DBTechTypesToPlanes(mask);
 
-    context.tc_scx = scx;
-    context.tc_filter = &filter;
-
-    /*
-     * Apply the function first to any of the tiles in the planes
-     * for this CellUse's CellDef that match the mask.
-     */
-
-    for (pNum = PL_PAINTBASE; pNum < DBNumPlanes; pNum++)
-	if (PlaneMaskHasPlane(filter.tf_planes, pNum))
-	{
-	    if (DBSrPaintArea((Tile *) NULL, def->cd_planes[pNum],
-		    &scx->scx_area, mask, func, (ClientData) &context))
-		return 1;
-	}
-
-    /*
-     * Now apply ourselves recursively to each of the CellUses
-     * in our tile plane.
-     */
-
-    if (DBCellSrArea(scx, dbCellTileSrFunc, (ClientData) &filter))
-	return 1;
-    else return 0;
+    return dbCellTileSrFunc(scx, &filter);
 }
 
 /*
  * dbCellTileSrFunc --
  *
- * Filter procedure applied to subcells by DBTreeSrTiles().
+ * Recursive filter procedure applied to the cell by DBTreeSrTiles().
  */
 
 int
@@ -196,6 +164,119 @@ dbCellTileSrFunc(scx, fp)
 	return 1;
     else return 0;
 }
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * DBTreeSrUniqueTiles --
+ *
+ * Recursively search downward from the supplied CellUse for
+ * all visible paint tiles matching the supplied type mask.
+ * This routine is like DBTreeSrTiles, above, except that it will
+ * only call the specified routine ONCE for each contact type, in
+ * the home plane of that contact type.  Stacked contact types will
+ * only be processed if the current search plane matches the home
+ * plane of one of the stacked contact type's residues.
+ *
+ * Results:
+ *	0 is returned if the search finished normally.  1 is returned
+ *	if the search was aborted.
+ *
+ * Side effects:
+ *	Whatever side effects are brought about by applying the
+ *	procedure supplied.
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+int
+DBTreeSrUniqueTiles(scx, mask, xMask, func, cdarg)
+    SearchContext *scx;		/* Pointer to search context specifying
+				 * a cell use to search, an area in the
+				 * coordinates of the cell's def, and a
+				 * transform back to "root" coordinates.
+				 */
+    TileTypeBitMask *mask;	/* Only tiles with a type for which
+				 * a bit in this mask is on are processed.
+				 */
+    int xMask;			/* All subcells are visited recursively
+				 * until we encounter uses whose flags,
+				 * when anded with xMask, are not
+				 * equal to xMask.
+				 */
+    int (*func)();		/* Function to apply at each qualifying tile */
+    ClientData cdarg;		/* Client data for above function */
+{
+    int dbCellTileSrFunc();
+    TreeFilter filter;
+
+    /* Set up the filter and call the recursive filter function */
+
+    filter.tf_func = func;
+    filter.tf_arg = cdarg;
+    filter.tf_mask = mask;
+    filter.tf_xmask = xMask;
+    filter.tf_planes = DBTechTypesToPlanes(mask);
+
+    return dbCellUniqueTileSrFunc(scx, &filter);
+}
+
+/*
+ * dbCellTileSrFunc --
+ *
+ * Recursive filter procedure applied to the cell by DBTreeSrUniqueTiles().
+ * This is similar to dbCellTileSrFunc, except that for each plane searched,
+ * only the tile types having that plane as their home plane will be passed
+ * to the filter function.  Contacts will therefore be processed only once.
+ */
+
+int
+dbCellUniqueTileSrFunc(scx, fp)
+    register SearchContext *scx;
+    register TreeFilter *fp;
+{
+    TreeContext context;
+    TileTypeBitMask uMask;
+    CellDef *def = scx->scx_use->cu_def;
+    int pNum;
+
+    ASSERT(def != (CellDef *) NULL, "dbCellTileSrFunc");
+    if (!DBIsExpand(scx->scx_use, fp->tf_xmask))
+	return 0;
+    if ((def->cd_flags & CDAVAILABLE) == 0)
+	if (!DBCellRead(def, (char *) NULL, TRUE)) return 0;
+
+    context.tc_scx = scx;
+    context.tc_filter = fp;
+
+    /*
+     * Apply the function first to any of the tiles in the planes
+     * for this CellUse's CellDef that match the mask.
+     */
+
+    for (pNum = PL_PAINTBASE; pNum < DBNumPlanes; pNum++)
+	if (PlaneMaskHasPlane(fp->tf_planes, pNum))
+	{
+	    uMask = DBHomePlaneTypes(pNum);
+	    TTMaskAndMask(&uMask, fp->tf_mask);
+
+	    if (DBSrPaintArea((Tile *) NULL, def->cd_planes[pNum],
+		    &scx->scx_area, &uMask,
+		    fp->tf_func, (ClientData) &context))
+		return 1;
+	}
+
+    /*
+     * Now apply ourselves recursively to each of the CellUses
+     * in our tile plane.
+     */
+
+    if (DBCellSrArea(scx, dbCellUniqueTileSrFunc, (ClientData) fp))
+	return 1;
+    else return 0;
+}
+
 
 /*
  *-----------------------------------------------------------------------------
@@ -1238,17 +1319,24 @@ void
 DBScaleValue(v, n, d)
     int *v, n, d;
 {
-    long lv = (long)(*v);
+    dlong llv = (dlong)(*v);
 
-    if ((lv < INFINITY - 2) && (lv > MINFINITY + 2))
+    if ((llv < (dlong)(INFINITY - 2)) && (llv > (dlong)(MINFINITY + 2)))
     {
-	lv *= n;
+	llv *= (dlong)n;
 
-	if (lv > 0)
-	    lv /= d;
-	else if (lv < 0)
-	    lv = ((lv + 1) / d) - 1;
-	*v = (int)lv;
+	if (llv > 0)
+	    llv /= (dlong)d;
+	else if (llv < 0)
+	    llv = ((llv + 1) / (dlong)d) - 1;
+	*v = (int)llv;
+
+	/* Hopefully we do not reach this error.  If we do, there's	*/
+	/* not much we can do about it except to increase Magic's	*/
+	/* internal Point structure to hold 8-byte integer values.	*/
+
+	if ((dlong)(*v) != llv)
+	    TxError("ERROR: ARITHMETIC OVERFLOW in DBScaleValue()!\n");
     }
 }
 
