@@ -310,7 +310,7 @@ leftside:
     if (IsSplit(tile) && SplitSide(tile)) goto bottomside;
 #endif
 
-    for (t2 = tile->ti_bl; BOTTOM(t2) < tileArea.r_ytop; t2 = t2->ti_rt)
+    for (t2 = BL(tile); BOTTOM(t2) < tileArea.r_ytop; t2 = RT(t2))
     {
 #ifdef NONMANHATTAN
 	if (IsSplit(t2))
@@ -339,7 +339,7 @@ bottomside:
 	goto rightside;
 #endif
 
-    for (t2 = tile->ti_lb; LEFT(t2) < tileArea.r_xtop; t2 = t2->ti_tr)
+    for (t2 = LB(tile); LEFT(t2) < tileArea.r_xtop; t2 = TR(t2))
     {
 #ifdef NONMANHATTAN
 	if (IsSplit(t2))
@@ -367,7 +367,7 @@ rightside:
     if (IsSplit(tile) && !SplitSide(tile)) goto topside;
 #endif
 
-    for (t2 = tile->ti_tr; ; t2 = t2->ti_lb)
+    for (t2 = TR(tile); ; t2 = LB(t2))
     {
 #ifdef NONMANHATTAN
 	if (IsSplit(t2))
@@ -396,7 +396,7 @@ topside:
     if (IsSplit(tile) && (SplitSide(tile) ^ SplitDirection(tile))) goto donesides;
 #endif
 
-    for (t2 = tile->ti_rt; ; t2 = t2->ti_bl)
+    for (t2 = RT(tile); ; t2 = BL(t2))
     {
 #ifdef NONMANHATTAN
 	if (IsSplit(t2))
