@@ -1724,7 +1724,7 @@ DBCellWriteFile(cellDef, f)
     }
 
     /* And any properties */
-    if (cellDef->cd_props != NULL)
+    if (cellDef->cd_props != (ClientData)NULL)
     {
 	FPRINTF(f, "<< properties >>\n");
 	DBPropEnum(cellDef, dbWritePropFunc, (ClientData)f);
