@@ -23,7 +23,7 @@
 
 #ifndef lint
 static char rcsid[] = "$Header: /ufs/repository/magic/database/DBpaint.c,v 1.4 2001/04/13 20:02:58 tim Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <sys/types.h>
 #include <stdio.h>
@@ -59,7 +59,7 @@ bool TiNMSplitY();
 
 #ifdef	PAINTDEBUG
 int dbPaintDebug = 0;
-#endif	PAINTDEBUG
+#endif	/* PAINTDEBUG */
 
 /* ----------------------- Flags to dbPaintMerge ---------------------- */
 
@@ -225,7 +225,7 @@ enumerate:
 #ifdef	PAINTDEBUG
 	if (dbPaintDebug)
 	    dbPaintShowTile(tile, undo, "area enum");
-#endif	PAINTDEBUG
+#endif	/* PAINTDEBUG */
 
 	/***
 	 *** ---------- THE FOLLOWING IS IN-LINE PAINT CODE ----------
@@ -416,7 +416,7 @@ enumerate:
 #ifdef	PAINTDEBUG
 	    if (dbPaintDebug)
 		dbPaintShowTile(tile, undo, "after clip");
-#endif	PAINTDEBUG
+#endif	/* PAINTDEBUG */
 	}
 
 #ifdef  NONMANHATTAN
@@ -431,12 +431,11 @@ enumerate:
 		if (undo && UndoIsEnabled())
 		    DBPAINTUNDO(tile, newType, undo);
 		TiSetBody(tile, newType);
-		mergeFlags = MRG_LEFT | MRG_RIGHT | MRG_TOP | MRG_BOTTOM;
 	    }
 	    else
 		mergeFlags = 0;
 	}
-#endif  NONMANHATTAN
+#endif  /* NONMANHATTAN */
 
 	/*
 	 * Merge the tile back into the parts of the plane that have
@@ -485,7 +484,7 @@ enumerate:
 #ifdef	PAINTDEBUG
 	if (dbPaintDebug)
 	    dbPaintShowTile(tile, undo, "changed type");
-#endif	PAINTDEBUG
+#endif	/* PAINTDEBUG */
 
 	if (mergeFlags & MRG_TOP)
 	{
@@ -494,7 +493,7 @@ enumerate:
 #ifdef	PAINTDEBUG
 	    if (dbPaintDebug)
 		dbPaintShowTile(tile, undo, "merged up (CHEAP)");
-#endif	PAINTDEBUG
+#endif	/* PAINTDEBUG */
 	}
 	if (mergeFlags & MRG_BOTTOM)
 	{
@@ -503,7 +502,7 @@ enumerate:
 #ifdef	PAINTDEBUG
 	    if (dbPaintDebug)
 		dbPaintShowTile(tile, undo, "merged down (CHEAP)");
-#endif	PAINTDEBUG
+#endif	/* PAINTDEBUG */
 	}
 
 
@@ -652,7 +651,7 @@ enumerate:
 #ifdef	PAINTDEBUG
 	if (dbPaintDebug)
 	    dbPaintShowTile(tile, undo, "area enum");
-#endif	PAINTDEBUG
+#endif	/* PAINTDEBUG */
 
 	/***
 	 *** ---------- THE FOLLOWING IS IN-LINE PAINT CODE ----------
@@ -837,7 +836,7 @@ enumerate:
 #ifdef	PAINTDEBUG
 	    if (dbPaintDebug)
 		dbPaintShowTile(tile, undo, "after clip");
-#endif	PAINTDEBUG
+#endif	/* PAINTDEBUG */
 	}
 
 #ifdef  NONMANHATTAN
@@ -852,12 +851,11 @@ enumerate:
 		if (undo && UndoIsEnabled())
 		    DBPAINTUNDO(tile, newType, undo);
 		TiSetBody(tile, newType);
-		mergeFlags = MRG_LEFT | MRG_RIGHT | MRG_TOP | MRG_BOTTOM;
 	    }
 	    else
 		mergeFlags = 0;
 	}
-#endif  NONMANHATTAN
+#endif  /* NONMANHATTAN */
 
 	/*
 	 * Merge the tile back into the parts of the plane that have
@@ -906,7 +904,7 @@ enumerate:
 #ifdef	PAINTDEBUG
 	if (dbPaintDebug)
 	    dbPaintShowTile(tile, undo, "changed type");
-#endif	PAINTDEBUG
+#endif	/* PAINTDEBUG */
 
 	if (mergeFlags & MRG_TOP)
 	{
@@ -915,7 +913,7 @@ enumerate:
 #ifdef	PAINTDEBUG
 	    if (dbPaintDebug)
 		dbPaintShowTile(tile, undo, "merged up (CHEAP)");
-#endif	PAINTDEBUG
+#endif	/* PAINTDEBUG */
 	}
 	if (mergeFlags & MRG_BOTTOM)
 	{
@@ -924,7 +922,7 @@ enumerate:
 #ifdef	PAINTDEBUG
 	    if (dbPaintDebug)
 		dbPaintShowTile(tile, undo, "merged down (CHEAP)");
-#endif	PAINTDEBUG
+#endif	/* PAINTDEBUG */
 	}
 
 
@@ -1043,7 +1041,7 @@ enumerate:
 #ifdef	PAINTDEBUG
 	if (dbPaintDebug)
 	    dbPaintShowTile(tile, undo, "area enum");
-#endif	PAINTDEBUG
+#endif	/* PAINTDEBUG */
 
 	/***
 	 *** ---------- THE FOLLOWING IS IN-LINE PAINT CODE ----------
@@ -1161,7 +1159,7 @@ enumerate:
 #ifdef	PAINTDEBUG
 	if (dbPaintDebug)
 	    dbPaintShowTile(tile, undo, "after clip");
-#endif	PAINTDEBUG
+#endif	/* PAINTDEBUG */
 
 	/*
 	 * Merge the tile back into the parts of the plane that have
@@ -1204,7 +1202,7 @@ enumerate:
 #ifdef	PAINTDEBUG
 	    if (dbPaintDebug)
 		dbPaintShowTile(tile, undo, "merged up (CHEAP)");
-#endif	PAINTDEBUG
+#endif	/* PAINTDEBUG */
 	}
 	if (mergeFlags & MRG_BOTTOM)
 	{
@@ -1213,7 +1211,7 @@ enumerate:
 #ifdef	PAINTDEBUG
 	    if (dbPaintDebug)
 		dbPaintShowTile(tile, undo, "merged down (CHEAP)");
-#endif	PAINTDEBUG
+#endif	/* PAINTDEBUG */
 	}
 
 
@@ -1690,7 +1688,7 @@ enumerate:
 #ifdef	PAINTDEBUG
 	if (dbPaintDebug)
 	    dbPaintShowTile(tile, undo, "first area enum");
-#endif	PAINTDEBUG
+#endif	/* PAINTDEBUG */
 
 	/***
 	 *** ---------- THE FOLLOWING IS IN-LINE PAINT CODE ----------
@@ -1760,7 +1758,7 @@ enumerate:
 #ifdef	PAINTDEBUG
 	    if (dbPaintDebug)
 		dbPaintShowTile(tile, undo, "after clip");
-#endif	PAINTDEBUG
+#endif	/* PAINTDEBUG */
 
 	    /* Record the type of the new tile */
 	    if (undo && UndoIsEnabled())
@@ -1830,7 +1828,7 @@ mergenum:
 #ifdef	PAINTDEBUG
 	if (dbPaintDebug)
 	    dbPaintShowTile(tile, undo, "merge area enum");
-#endif	PAINTDEBUG
+#endif	/* PAINTDEBUG */
 
 	/***
 	 *** ---------- THE FOLLOWING IS IN-LINE MERGE CODE ----------
@@ -1883,7 +1881,7 @@ mergenum:
 #ifdef	PAINTDEBUG
 	    if (dbPaintDebug)
 		dbPaintShowTile(tile, undo, "merged up (CHEAP)");
-#endif	PAINTDEBUG
+#endif	/* PAINTDEBUG */
 	}
 	if (mergeFlags & MRG_BOTTOM)
 	{
@@ -1892,7 +1890,7 @@ mergenum:
 #ifdef	PAINTDEBUG
 	    if (dbPaintDebug)
 		dbPaintShowTile(tile, undo, "merged down (CHEAP)");
-#endif	PAINTDEBUG
+#endif	/* PAINTDEBUG */
 	}
 
 
@@ -2049,7 +2047,7 @@ dbPaintMerge(tile, newType, plane, mergeFlags, undo)
 #ifdef	PAINTDEBUG
 	if (dbPaintDebug)
 	    dbPaintShowTile(tile, undo, "(DBMERGE) after split");
-#endif	PAINTDEBUG
+#endif	/* PAINTDEBUG */
     }
 
     /*
@@ -2064,7 +2062,7 @@ dbPaintMerge(tile, newType, plane, mergeFlags, undo)
 #ifdef	PAINTDEBUG
     if (dbPaintDebug)
 	dbPaintShowTile(tile, undo, "(DBMERGE) changed type");
-#endif	PAINTDEBUG
+#endif	/* PAINTDEBUG */
 
     /*
      * Do the merging.
@@ -2082,7 +2080,7 @@ dbPaintMerge(tile, newType, plane, mergeFlags, undo)
 #ifdef	PAINTDEBUG
 	if (dbPaintDebug)
 	    dbPaintShowTile(tile, undo, "(DBMERGE) merged left");
-#endif	PAINTDEBUG
+#endif	/* PAINTDEBUG */
     }
     if (mergeFlags & MRG_RIGHT)
     {
@@ -2094,7 +2092,7 @@ dbPaintMerge(tile, newType, plane, mergeFlags, undo)
 #ifdef	PAINTDEBUG
 	if (dbPaintDebug)
 	    dbPaintShowTile(tile, undo, "(DBMERGE) merged right");
-#endif	PAINTDEBUG
+#endif	/* PAINTDEBUG */
     }
     if (mergeFlags&MRG_TOP)
     {
@@ -2103,7 +2101,7 @@ dbPaintMerge(tile, newType, plane, mergeFlags, undo)
 #ifdef	PAINTDEBUG
 	if (dbPaintDebug)
 	    dbPaintShowTile(tile, undo, "(DBMERGE) merged up");
-#endif	PAINTDEBUG
+#endif	/* PAINTDEBUG */
     }
     if (mergeFlags&MRG_BOTTOM)
     {
@@ -2112,7 +2110,7 @@ dbPaintMerge(tile, newType, plane, mergeFlags, undo)
 #ifdef	PAINTDEBUG
 	if (dbPaintDebug)
 	    dbPaintShowTile(tile, undo, "(DBMERGE) merged down");
-#endif	PAINTDEBUG
+#endif	/* PAINTDEBUG */
     }
 
     return (tile);
@@ -2197,7 +2195,7 @@ enumerate:
 #ifdef	PAINTDEBUG
 	if (dbPaintDebug)
 	    dbPaintShowTile(tile, undo, "area enum");
-#endif	PAINTDEBUG
+#endif	/* PAINTDEBUG */
 
 	/***
 	 *** ---------- THE FOLLOWING IS IN-LINE PAINT CODE ----------
@@ -2306,7 +2304,7 @@ enumerate:
 #ifdef	PAINTDEBUG
 	    if (dbPaintDebug)
 		dbPaintShowTile(tile, undo, "after clip");
-#endif	PAINTDEBUG
+#endif	/* PAINTDEBUG */
 	}
 
 	/*
@@ -2353,7 +2351,7 @@ enumerate:
 #ifdef	PAINTDEBUG
 	if (dbPaintDebug)
 	    dbPaintShowTile(tile, undo, "changed type");
-#endif	PAINTDEBUG
+#endif	/* PAINTDEBUG */
 
 	if (mergeFlags & MRG_TOP)
 	{
@@ -2363,7 +2361,7 @@ enumerate:
 #ifdef	PAINTDEBUG
 	    if (dbPaintDebug)
 		dbPaintShowTile(tile, undo, "merged up (CHEAP)");
-#endif	PAINTDEBUG
+#endif	/* PAINTDEBUG */
 	}
 	if (mergeFlags & MRG_BOTTOM)
 	{
@@ -2373,7 +2371,7 @@ enumerate:
 #ifdef	PAINTDEBUG
 	    if (dbPaintDebug)
 		dbPaintShowTile(tile, undo, "merged down (CHEAP)");
-#endif	PAINTDEBUG
+#endif	/* PAINTDEBUG */
 	}
 
 
@@ -2532,7 +2530,7 @@ dbMergeType(tile, newType, plane, mergeFlags, undo, client)
 #ifdef	PAINTDEBUG
 	if (dbPaintDebug)
 	    dbPaintShowTile(tile, undo, "(DBMERGE) after split");
-#endif	PAINTDEBUG
+#endif	/* PAINTDEBUG */
     }
 
     /*
@@ -2546,7 +2544,7 @@ dbMergeType(tile, newType, plane, mergeFlags, undo, client)
 #ifdef	PAINTDEBUG
     if (dbPaintDebug)
 	dbPaintShowTile(tile, undo, "(DBMERGE) changed type");
-#endif	PAINTDEBUG
+#endif	/* PAINTDEBUG */
 
     /*
      * Do the merging.
@@ -2568,7 +2566,7 @@ dbMergeType(tile, newType, plane, mergeFlags, undo, client)
 #ifdef	PAINTDEBUG
 	if (dbPaintDebug)
 	    dbPaintShowTile(tile, undo, "(DBMERGE) merged left");
-#endif	PAINTDEBUG
+#endif	/* PAINTDEBUG */
     }
     if (mergeFlags & MRG_RIGHT)
     {
@@ -2584,7 +2582,7 @@ dbMergeType(tile, newType, plane, mergeFlags, undo, client)
 #ifdef	PAINTDEBUG
 	if (dbPaintDebug)
 	    dbPaintShowTile(tile, undo, "(DBMERGE) merged right");
-#endif	PAINTDEBUG
+#endif	/* PAINTDEBUG */
     }
     if (mergeFlags&MRG_TOP)
     {
@@ -2593,7 +2591,7 @@ dbMergeType(tile, newType, plane, mergeFlags, undo, client)
 #ifdef	PAINTDEBUG
 	if (dbPaintDebug)
 	    dbPaintShowTile(tile, undo, "(DBMERGE) merged up");
-#endif	PAINTDEBUG
+#endif	/* PAINTDEBUG */
     }
     if (mergeFlags&MRG_BOTTOM)
     {
@@ -2602,7 +2600,7 @@ dbMergeType(tile, newType, plane, mergeFlags, undo, client)
 #ifdef	PAINTDEBUG
 	if (dbPaintDebug)
 	    dbPaintShowTile(tile, undo, "(DBMERGE) merged down");
-#endif	PAINTDEBUG
+#endif	/* PAINTDEBUG */
     }
 
     return (tile);
@@ -2687,7 +2685,7 @@ enumerate:
 #ifdef	PAINTDEBUG
 	if (dbPaintDebug)
 	    dbPaintShowTile(tile, undo, "area enum");
-#endif	PAINTDEBUG
+#endif	/* PAINTDEBUG */
 
 	/***
 	 *** ---------- THE FOLLOWING IS IN-LINE PAINT CODE ----------
@@ -2771,7 +2769,7 @@ enumerate:
 #ifdef	PAINTDEBUG
 	    if (dbPaintDebug)
 		dbPaintShowTile(tile, undo, "after clip");
-#endif	PAINTDEBUG
+#endif	/* PAINTDEBUG */
 	}
 
 	/*
@@ -2818,7 +2816,7 @@ enumerate:
 #ifdef	PAINTDEBUG
 	if (dbPaintDebug)
 	    dbPaintShowTile(tile, undo, "changed type");
-#endif	PAINTDEBUG
+#endif	/* PAINTDEBUG */
 
 	if (mergeFlags & MRG_LEFT)
 	{
@@ -2827,7 +2825,7 @@ enumerate:
 #ifdef	PAINTDEBUG
 	    if (dbPaintDebug)
 		dbPaintShowTile(tile, undo, "merged left (CHEAP)");
-#endif	PAINTDEBUG
+#endif	/* PAINTDEBUG */
 	}
 	if (mergeFlags & MRG_RIGHT)
 	{
@@ -2836,7 +2834,7 @@ enumerate:
 #ifdef	PAINTDEBUG
 	    if (dbPaintDebug)
 		dbPaintShowTile(tile, undo, "merged right (CHEAP)");
-#endif	PAINTDEBUG
+#endif	/* PAINTDEBUG */
 	}
 
 
@@ -2998,7 +2996,7 @@ dbPaintMergeVert(tile, newType, plane, mergeFlags, undo)
 #ifdef	PAINTDEBUG
 	if (dbPaintDebug)
 	    dbPaintShowTile(tile, undo, "(DBMERGE) after split");
-#endif	PAINTDEBUG
+#endif	/* PAINTDEBUG */
     }
 
     /*
@@ -3011,7 +3009,7 @@ dbPaintMergeVert(tile, newType, plane, mergeFlags, undo)
 #ifdef	PAINTDEBUG
     if (dbPaintDebug)
 	dbPaintShowTile(tile, undo, "(DBMERGE) changed type");
-#endif	PAINTDEBUG
+#endif	/* PAINTDEBUG */
 
     /*
      * Do the merging.
@@ -3029,7 +3027,7 @@ dbPaintMergeVert(tile, newType, plane, mergeFlags, undo)
 #ifdef	PAINTDEBUG
 	if (dbPaintDebug)
 	    dbPaintShowTile(tile, undo, "(DBMERGE) merged up");
-#endif	PAINTDEBUG
+#endif	/* PAINTDEBUG */
     }
 
     if (mergeFlags & MRG_BOTTOM)
@@ -3042,7 +3040,7 @@ dbPaintMergeVert(tile, newType, plane, mergeFlags, undo)
 #ifdef	PAINTDEBUG
 	if (dbPaintDebug)
 	    dbPaintShowTile(tile, undo, "(DBMERGE) merged down");
-#endif	PAINTDEBUG
+#endif	/* PAINTDEBUG */
     }
 
     if (mergeFlags&MRG_LEFT)
@@ -3052,7 +3050,7 @@ dbPaintMergeVert(tile, newType, plane, mergeFlags, undo)
 #ifdef	PAINTDEBUG
 	if (dbPaintDebug)
 	    dbPaintShowTile(tile, undo, "(DBMERGE) merged left");
-#endif	PAINTDEBUG
+#endif	/* PAINTDEBUG */
     }
     if (mergeFlags&MRG_RIGHT)
     {
@@ -3061,7 +3059,7 @@ dbPaintMergeVert(tile, newType, plane, mergeFlags, undo)
 #ifdef	PAINTDEBUG
 	if (dbPaintDebug)
 	    dbPaintShowTile(tile, undo, "(DBMERGE) merged right");
-#endif	PAINTDEBUG
+#endif	/* PAINTDEBUG */
     }
 
     return (tile);
@@ -3109,7 +3107,7 @@ dbPaintShowTile(tile, undo, str)
     (void) TxGetLine(answer, sizeof answer);
     DBWFeedbackClear();
 }
-#endif	PAINTDEBUG
+#endif	/* PAINTDEBUG */
 
 #ifdef NONMANHATTAN
 /*
