@@ -56,7 +56,7 @@ DBPropPut(cellDef, name, value)
     HashTable *htab;
     HashEntry *entry;
 
-    if (cellDef->cd_props == NULL) 
+    if (cellDef->cd_props == (ClientData) NULL) 
     {
 	cellDef->cd_props = (ClientData) mallocMagic(sizeof(HashTable));
 	HashInit( (HashTable *) cellDef->cd_props, 8, 0);
@@ -147,7 +147,7 @@ DBPropEnum(cellDef, func, cdata)
     HashEntry *entry;
     int res;
 
-    if (cellDef->cd_props == NULL) return 0;
+    if (cellDef->cd_props == (ClientData) NULL) return 0;
     htab = (HashTable *) cellDef->cd_props;
 
     HashStartSearch(&hs);
@@ -183,7 +183,7 @@ DBPropClearAll(cellDef)
     HashSearch hs;
     HashEntry *entry;
 
-    if (cellDef->cd_props == NULL) return;
+    if (cellDef->cd_props == (ClientData) NULL) return;
     htab = (HashTable *) cellDef->cd_props;
 
     HashStartSearch(&hs);
