@@ -801,7 +801,7 @@ DBTechGetContact(type1, type2)
     }
 
     TxPrintf("No contact type for %d %d\n", type1, type2);
-    return (TileType) 0;
+    return (TileType) -1;
 }
 
 /*
