@@ -21,7 +21,7 @@
 
 #ifndef lint
 static char rcsid[] = "$Header: /ufs/repository/magic/database/DBtech.c,v 1.2 2001/01/12 22:12:21 jsolomon Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #include <ctype.h>
@@ -36,12 +36,12 @@ static char rcsid[] = "$Header: /ufs/repository/magic/database/DBtech.c,v 1.2 20
 #include "textio/textio.h"
 #include "utils/malloc.h"
 
-    /* Name of this technology */
+/* Name of this technology */
 char *DBTechName = 0;
 char *DBTechVersion = 0;
 char *DBTechDescription = 0;
 
-    /* Connectivity */
+/* Connectivity */
 TileTypeBitMask	 DBConnectTbl[NT];
 TileTypeBitMask	 DBNotConnectTbl[NT];
 unsigned int	 DBConnPlanes[NT];
@@ -67,6 +67,12 @@ void
 DBTechInit()
 {
     /* StrDup() takes care of reallocating DBTechName */
+
+    /* TECH_FORMAT_VERSION is defined in tech/tech.h,	*/
+    /* and should be 27, the last version to be		*/
+    /* included as part of the tech filename suffix.	*/
+
+    TechFormatVersion = TECH_FORMAT_VERSION;
 }
 
 
@@ -96,6 +102,24 @@ DBTechSetTech(sectionName, argc, argv)
 {
     if (argc != 1)
     {
+	if (argc == 2)
+	{
+	    if (strncmp(argv[0], "format", 6) == 0 ||
+			strncmp(argv[0], "version", 7) == 0)
+	    {
+		if (StrIsInt(argv[1]))
+		{
+		    TechFormatVersion = atoi(argv[1]);	
+		    return TRUE;
+		}
+		else
+		{
+		    TechError("Bad format version number. . . assuming %d\n",
+				TECH_FORMAT_VERSION);
+		    return TRUE;
+		}
+	    }
+	}
 	TechError("Badly formed technology name\n");
 	return FALSE;
     }
@@ -279,6 +303,20 @@ DBTechFinalConnect()
     for (s = 0; s < DBNumTypes; s++)
 	DBConnPlanes[s] = 0;
 
+    /*
+     * Each stacked contact type must necessarily connect to its
+     * residual contact types, and the connecting types of those
+     * contacts.
+     */
+    for (base = DBNumUserLayers; base < DBNumTypes; base++)
+    {
+	TileTypeBitMask *rmask = DBResidueMask(base);
+	TTMaskSetMask(&DBConnectTbl[base], rmask);
+	for (s = TT_TECHDEPBASE; s < DBNumUserLayers; s++)
+	    if (TTMaskHasType(rmask, s))
+		TTMaskSetMask(&DBConnectTbl[base], &DBConnectTbl[s]);
+    }
+
     /*
      * Each contact type should connect to everything that each of
      * its connected images connects to, but only stuff that's on
@@ -291,18 +329,9 @@ DBTechFinalConnect()
      * as well as updating it.
      */
     bcopy((char *) DBConnectTbl, (char *) saveConnect, sizeof saveConnect);
-    for (n = 0; n < dbNumImages; n++)
+    for (n = 0; n < dbNumContacts; n++)
     {
 	lp = dbContactInfo[n];
-	TTMaskSetMask(&DBConnectTbl[lp->l_type], &lp->l_tmask);
-	for (s = TT_TECHDEPBASE; s < DBNumTypes; s++)
-	    if (TTMaskHasType(&lp->l_tmask, s))
-	    {
-		tmp = DBPlaneTypes[DBPlane(lp->l_type)];
-		TTMaskSetMask(&tmp, &DBPlaneTypes[DBPlane(s)]);
-		TTMaskAndMask3(&tmp2, &saveConnect[s], &tmp);
-		TTMaskSetMask(&DBConnectTbl[lp->l_type], &tmp2);
-	    }
 
 	/*
 	 * DBConnPlanes[] is nonzero only for contact images, for which
