@@ -512,6 +512,13 @@ DBTechPrintCanonicalType(layername)
  *	a parenthesized group.  For example, ~(x,y)/foo refers to
  *	all layers on plane "foo" except "x" and "y".
  *
+ *	Finally, magic version 7.3 defines *x to mean "x and all
+ *	contact layers containing x as a residue".  Thus, for example,
+ *	*metal1 expands to m1,pc,ndc,pdc,nsc,psc,m2c,...  Because many
+ *	connect, cif, extract, and drc rules list all these layers,
+ *	the asterisk notation makes the techfile shorter and more
+ *	readable.
+ *
  * Results:
  *	Returns a plane mask of all the planes specified.
  *
@@ -528,10 +535,10 @@ DBTechNoisyNameMask(layers, mask)
     TileTypeBitMask *mask;		/* Where to store the layer mask. */
 {
     register char *p, *p2, c;
-    TileTypeBitMask m2;            /* Each time around the loop, we will
-                                         * form the mask for one section of
-                                         * the layer string.
-                                         */
+    TileTypeBitMask m2;        /* Each time around the loop, we will
+				* form the mask for one section of
+				* the layer string.
+				*/
     char save;
     bool allBut;
     int planemask;
@@ -589,7 +596,8 @@ DBTechNoisyNameMask(layers, mask)
 	}
 	else
 	{
-	    TileType t;
+	    TileType t, rtype;
+	    bool allResidues = FALSE;
 
 	    /* No parenthesis, so just parse off a single name.  Layer
 	     * name "0" corresponds to no layers at all.
@@ -608,9 +616,24 @@ DBTechNoisyNameMask(layers, mask)
 	    {
 		save = *p2;
 		*p2 = 0;
+
+		/* Check for asterisk notation, meaning to include	*/
+		/* all types which have this type as a residue.		*/
+		if (*p == '*')
+		{
+		    allResidues = TRUE;
+		    p++;
+		}
+
 		t = DBTechNoisyNameType(p);
 		if (t >= 0) m2 = DBLayerTypeMaskTbl[t];
 		*p2 = save;
+
+		/* Include all types which have t as a residue */
+		if (allResidues)
+		    for (rtype = TT_TECHDEPBASE; rtype < DBNumUserLayers; rtype++)
+			if (TTMaskHasType(DBResidueMask(rtype), t))
+			    TTMaskSetType(&m2, rtype);
 	    }
 	    p = p2;
 	}
@@ -765,7 +788,8 @@ DBTechSubsetLayers(src, mask, result)
 
     TTMaskZero(result);
     TTMaskZero(&clrmask);
-    for (i = 0; i < DBNumUserLayers; i++)
+    /* for (i = 0; i < DBNumUserLayers; i++) */	/* ? */
+    for (i = 0; i < DBNumTypes; i++)
     {
 	TTMaskAndMask3(&tmp, &src, &DBLayerTypeMaskTbl[i]);
 	if (!TTMaskIsZero(&tmp))
