@@ -62,7 +62,6 @@ TileTypeBitMask DBAllButSpaceBits;
 TileTypeBitMask DBAllButSpaceAndDRCBits;
 TileTypeBitMask DBSpaceBits;
 TileTypeBitMask DBUserLayerBits;
-TileTypeBitMask DBNonSpaceUserLayerBits;
 
 /* Table of default, builtin planes */
 DefaultPlane dbTechDefaultPlanes[] =
@@ -378,7 +377,6 @@ dbTechNewStackedType(type1, type2)
  *		DBAllButSpaceBits
  *		DBAllButSpaceAndDRCBits
  *		DBUserLayerBits
- *		DBNonSpaceUserLayerBits
  *
  * ----------------------------------------------------------------------------
  */
@@ -405,7 +403,9 @@ DBTechFinalType()
     TTMaskCom2(&DBAllTypeBits, &DBZeroTypeBits);
     TTMaskSetOnlyType(&DBSpaceBits, TT_SPACE);
     TTMaskCom2(&DBAllButSpaceBits, &DBSpaceBits);
-    TTMaskClearMask3(&DBNonSpaceUserLayerBits, &DBUserLayerBits, &DBSpaceBits);
+
+    /* UserLayerBits includes space */
+    TTMaskSetType(&DBUserLayerBits, TT_SPACE);
 
     /* Space is visible on all planes but the subcell plane */
     TTMaskZero(&DBPlaneTypes[PL_CELL]);
