@@ -497,7 +497,7 @@ DBResetTilePlane(plane, cdata)
     register Rect *rect = &TiPlaneRect;
 
     /* Start with the leftmost non-infinity tile in the plane */
-    tp = plane->pl_left->ti_tr;
+    tp = TR(plane->pl_left);
 
     /* Each iteration visits another tile on the LHS of the search area */
     while (TOP(tp) > rect->r_ybot)
@@ -575,7 +575,7 @@ DBFreePaintPlane(plane)
     register Rect *rect = &TiPlaneRect;
 
     /* Start with the bottom-right non-infinity tile in the plane */
-    tp = plane->pl_right->ti_bl;
+    tp = BL(plane->pl_right);
 
     /* Each iteration visits another tile on the RHS of the search area */
     while (BOTTOM(tp) < rect->r_ytop)
