@@ -536,8 +536,8 @@ dbTechCheckImages(t, r, s)
 	for (pNum = PL_TECHDEPBASE; pNum < DBNumPlanes; pNum++)
 	    if (PlaneMaskHasPlane(pMask, pNum) && (t != r))
 	    {
-		TechError("\
-Result image on plane %s must be the same as image of %s on plane %s\n",
+		TechError("Result image on plane %s must be the same "
+			"as image of %s on plane %s\n",
 			DBPlaneLongName(pNum),
 			DBTypeLongName(r),
 			DBPlaneLongName(pNum));
