@@ -259,13 +259,13 @@ dbComposeResidues()
     register TileType s, res;
     register int n, p;
 
-    /* Painting that doesn't affect the residue doesn't affect the contact */
+    /* Painting that doesn't affect the residue doesn't affect the contact. */
 
     for (n = 0; n < dbNumContacts; n++)
     {
 	lp = dbContactInfo[n];
 
-	for (res = TT_TECHDEPBASE; res < DBNumTypes; res++)
+	for (res = TT_TECHDEPBASE; res < DBNumUserLayers; res++)
 	{
 	    if (TTMaskHasType(&lp->l_residues, res))
 	    {
