@@ -18,7 +18,7 @@
 
 #ifndef lint
 static char rcsid[] = "$Header: /ufs/repository/magic/database/DBundo.c,v 1.3 2001/03/29 19:50:06 tim Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #include "misc/magic.h"
@@ -164,6 +164,28 @@ DBUndoInit()
 
     dbUndoLastCell = (CellDef *) NULL;
 }
+
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * DBUndoReset --
+ *
+ * Set dbUndoLastCell to NULL.  Used by the cell delete function when the
+ * dbUndoLastCell points to the cell to be deleted.
+ * ----------------------------------------------------------------------------
+ */
+
+void
+DBUndoReset(celldef)
+    CellDef *celldef;
+{
+    if (celldef == dbUndoLastCell)
+    {
+	UndoFlush();
+	dbUndoLastCell = (CellDef *) NULL;
+    }
+}
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -349,6 +371,7 @@ dbUndoPaintBack(up)
 				 * is allocated to hold all the bytes
 				 * in the label, plus the null byte.
 				 */
+	int	 lue_flags;	/* Flags for label */
     } labelUE;
 
     /*
@@ -376,7 +399,7 @@ dbUndoPaintBack(up)
  */
 
 void
-DBUndoPutLabel(cellDef, rect, pos, text, type)
+DBUndoPutLabel(cellDef, rect, pos, text, type, flags)
     CellDef *cellDef;	/* CellDef being modified */
     register Rect *rect;/* Box definining label.  The lower left coordinate
 			 * of the box determines the tile to which the label
@@ -386,6 +409,7 @@ DBUndoPutLabel(cellDef, rect, pos, text, type)
     int pos;		/* Relative position to point */
     char *text;		/* Text of label */
     TileType type;	/* Type of tile being labelled */
+    int flags;		/* Flags of label */
 {
     register labelUE *lup;
 
@@ -401,6 +425,7 @@ DBUndoPutLabel(cellDef, rect, pos, text, type)
     lup->lue_rect = *rect;
     lup->lue_pos = pos;
     lup->lue_type = type;
+    lup->lue_flags = flags;
     strcpy(lup->lue_text, text);
 }
 
@@ -421,7 +446,7 @@ DBUndoPutLabel(cellDef, rect, pos, text, type)
  */
 
 void
-DBUndoEraseLabel(cellDef, rect, pos, text, type)
+DBUndoEraseLabel(cellDef, rect, pos, text, type, flags)
     CellDef *cellDef;	/* Cell being modified */
     register Rect *rect;/* Box definining label.  The lower left coordinate
 			 * of the box determines the tile to which the label
@@ -431,6 +456,7 @@ DBUndoEraseLabel(cellDef, rect, pos, text, type)
     int pos;		/* Relative position to point */
     char *text;		/* Text of label */
     TileType type;	/* Type of tile being labelled */
+    int flags;		/* Flags for label */
 {
     register labelUE *lup;
 
@@ -446,6 +472,7 @@ DBUndoEraseLabel(cellDef, rect, pos, text, type)
     lup->lue_rect = *rect;
     lup->lue_pos = pos;
     lup->lue_type = type;
+    lup->lue_flags = flags;
     strcpy(lup->lue_text, text);
 }
 
@@ -471,7 +498,7 @@ dbUndoLabelForw(up)
     register labelUE *up;
 {
     (void) DBPutLabel(dbUndoLastCell, &up->lue_rect, up->lue_pos,
-	up->lue_text, up->lue_type);
+	up->lue_text, up->lue_type, up->lue_flags);
     DBWLabelChanged(dbUndoLastCell, up->lue_text, &up->lue_rect,
 	up->lue_pos, DBW_ALLWINDOWS);
 
