@@ -525,16 +525,24 @@ DBWInitCommands()
 #endif
 
 #ifdef LEF_MODULE
-    /* Add the LEF extensions */
+    /* Add the LEF/DEF extensions */
     WindAddCommand(DBWclientID,
-	"lef [options]	LEF-format output; type \"lef help\"\n\
+	"lef [options]	LEF-format input/output; type \"lef help\"\n\
+			for information on options",
+	CmdLef, FALSE);
+    WindAddCommand(DBWclientID,
+	"def [options]	DEF-format input; type \"def help\"\n\
 			for information on options",
 	CmdLef, FALSE);
 #endif
 #ifdef LEF_AUTO
-    /* Placeholder for LEF extensions */
+    /* Placeholder for LEF/DEF extensions */
     WindAddCommand(DBWclientID,
-	"lef [options]	LEF-format output; type \"lef help\"\n\
+	"lef [options]	LEF-format input/output; type \"lef help\"\n\
+			for information on options",
+	CmdAutoLef, FALSE);
+    WindAddCommand(DBWclientID,
+	"def [options]	DEF-format input; type \"def help\"\n\
 			for information on options",
 	CmdAutoLef, FALSE);
 #endif
