@@ -538,7 +538,7 @@ dbwPaintFunc(tile, cxp)
 	    if ((*GrEventPendingPtr)())
 		sigOnInterrupt(0);
 	    else
-		SigSetTimer();
+		SigSetTimer(0);
 	}
     }
 #endif
@@ -860,7 +860,7 @@ dbwTileFunc(tile)
     }
     else
     {
-    	 (void) sprintf(string, "%x", tile);
+    	 (void) sprintf(string, "%lx", tile);
     }
     
     GeoClip(&r2, &rootClip);
@@ -1001,7 +1001,7 @@ dbwTileFunc(tile)
 	}
 	else if (!dbwSeeTypes)
 	{
-	    (void) sprintf(string, "%x", stitch);
+	    (void) sprintf(string, "%lx", stitch);
 	    GrPutText(string, STYLE_DRAWTILE, &p, pos,
 		GR_TEXT_SMALL, FALSE, &r2, (Rect *) NULL);
 	}
@@ -1460,16 +1460,14 @@ DBWTechAddStyle(sectionName, argc, argv)
     char *argv[];
 {
     TileType t, r;
-    int sidx;
+    int i, sidx;
     static char styleType[50];
 
-    if (argc != 2)
+    if (argc < 2)
     {
 	TechError("Badly formed line in \"style\" section\n");
-	return TRUE;
     }
-
-    if (strcmp(argv[0], "styletype") == 0)
+    else if (strcmp(argv[0], "styletype") == 0)
     {
 	(void) strncpy(styleType, argv[1], 49);
 	styleType[49] = 0;
@@ -1491,31 +1489,34 @@ DBWTechAddStyle(sectionName, argc, argv)
 	   return FALSE;
 
 	GrSetCursor(0);
-
-	return TRUE;
     }
-    if ((t = DBTechNoisyNameType(argv[0])) < 0) return (FALSE);
-
-    sidx = DBWTechParseStyle(argv[1]);
-
-    if (sidx < 0)
+    else
     {
-	TechError("Invalid style \"%s\" for tile type %s\n", argv[1], argv[0]);
-	return TRUE;
+	if ((t = DBTechNoisyNameType(argv[0])) < 0) return (FALSE);
+
+	/* Allow space-separated list of styles for each type */
+
+	for (i = 1; i < argc; i++)
+	{
+	    sidx = DBWTechParseStyle(argv[i]);
+	    if (sidx < 0)
+		TechError("Invalid style \"%s\" for tile type %s\n", argv[i], argv[0]);
+	    else
+	    {
+		TTMaskSetType(&DBWStyleToTypesTbl[sidx], t);
+
+		/* If type t is a contact, then any stacked contact type which	*/
+		/* has t as a residue, and for which the home plane of the	*/
+		/* stacked contact is also the home plane of t, should be added	*/
+		/* to the list of types to be painted with this style.		*/
+
+		if (DBIsContact(t) && (t < DBNumUserLayers))
+		    for (r = DBNumUserLayers; r < DBNumTypes; r++)
+			if (TTMaskHasType(DBResidueMask(r), t)
+				&& (DBPlane(r) == DBPlane(t)))
+			    TTMaskSetType(&DBWStyleToTypesTbl[sidx], r);
+	    }
+	}
     }
-
-    TTMaskSetType(&DBWStyleToTypesTbl[sidx], t);
-
-    /* If type t is a contact, then any stacked contact type which has	*/
-    /* t as a residue, and for which the home plane of the stacked	*/
-    /* contact is also the home plane of t, should be added to the list	*/
-    /* of types to be painted with this style.				*/
-
-    if (DBIsContact(t) && (t < DBNumUserLayers))
-	for (r = DBNumUserLayers; r < DBNumTypes; r++)
-	    if (TTMaskHasType(DBResidueMask(r), t) && (DBPlane(r) ==
-			DBPlane(t)))
-		TTMaskSetType(&DBWStyleToTypesTbl[sidx], r);
-
     return TRUE;
 }
