@@ -8,6 +8,8 @@
  *	and arcs).  These operate very similarly to feedback regions,
  *	in that they are persistant until destroyed, and do not
  *	interact with the layout in any way.
+ * 
+ * Copyright (C) 2003 Open Circuit Design, Inc., for MultiGiG, Ltd.
  */
 
 #include <stdio.h>
@@ -49,6 +51,7 @@ typedef struct _style
 typedef struct _element
 {
     int type;			/* Type of element (see list above) */
+    unsigned char flags;	/* Special flags (depends on element type) */
     CellDef *rootDef;		/* Root definition of windows in which to
 				 * display this element.
 				 */
@@ -76,6 +79,156 @@ static CellDef *dbwelemRootDef;		/* To pass root cell definition from
 					 * DBWElementAdd.
 					 */
 
+void
+AppendString(oldstr, newstr, postfix)
+    char **oldstr;
+    char *newstr;
+    char *postfix;
+{
+    char *tmpstr;
+    int olen = 0;
+    int nlen = strlen(newstr);
+    int plen = 0;
+
+    if (*oldstr != NULL) olen = strlen(*oldstr);
+    if (postfix != NULL) plen = strlen(postfix);
+    tmpstr = (char *)mallocMagic(olen + nlen + plen + 1);
+    if (*oldstr == NULL)
+	strcpy(tmpstr, newstr);
+    else
+    {
+	strcpy(tmpstr, *oldstr);
+	strcat(tmpstr, newstr);
+	FREE(*oldstr);
+    }
+    if (postfix != NULL) strcat(tmpstr, postfix);
+    *oldstr = tmpstr;
+}
+
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * DBWPrintElements --
+ *
+ *	Generate a string containing the names of all elements which are
+ *	defined in CellDef "cellDef" and have flags matching "flagmask".
+ *	Format is appropriate for writing output to a .mag file.
+ *
+ * Results:
+ *	An allocated string, or NULL if no elements were found.
+ *
+ * Side Effects:
+ *	Memory is allocated for the string and must be freed by the calling
+ *	procedure.
+ * ----------------------------------------------------------------------------
+ */
+
+char *
+DBWPrintElements(cellDef, flagmask)
+    CellDef *cellDef;
+    unsigned char flagmask;
+{
+    DBWElement *elem;
+    HashSearch hs;
+    HashEntry *he;
+    char *rstr = NULL;	/* allocated return string */
+    char istr[10];
+    styleptr sptr;
+    int flagset;
+
+    /* These must match the order of text sizes in graphics/graphics.h */
+    static char *textSizes[] = {"small", "medium", "large", "xlarge",
+	"default", NULL}; 
+
+    /* These must match the order of element type definitions above! */
+    char *etypes[] = {"rectangle", "line", "text"};
+
+    HashStartSearch(&hs);
+    while (he = HashNext(&elementTable, &hs))
+    {
+	if (elem = (DBWElement *)HashGetValue(he))
+	{
+	    if ((elem->rootDef == cellDef) && (elem->flags & flagmask))
+	    { 
+		/* print element type */
+		AppendString(&rstr, etypes[elem->type], " ");
+		/* print element name */
+		AppendString(&rstr, (char *)he->h_key.h_name, " ");
+
+		/* print dstyle(s) */
+		for (sptr = elem->stylelist; sptr != NULL; sptr = sptr->next)
+		{
+		    AppendString(&rstr, GrStyleTable[sptr->style].longname, NULL);
+		    if (sptr->next != NULL)
+			AppendString(&rstr, ",", NULL);
+		}
+		AppendString(&rstr, " ");
+		/* print start point */
+		sprintf(istr, "%d", elem->area.r_xbot);
+		AppendString(&rstr, istr, " ");
+		sprintf(istr, "%d", elem->area.r_ybot);
+		AppendString(&rstr, istr, " ");
+		switch (elem->type)
+		{
+		    case ELEMENT_RECT:
+		        /* end point */
+			sprintf(istr, "%d", elem->area.r_xtop);
+			AppendString(&rstr, istr, " ");
+			sprintf(istr, "%d", elem->area.r_ytop);
+			AppendString(&rstr, istr, "\n");
+			/* no flags to write.  Only applicable flag is  */
+			/* temporary/persistent, and temporary elements */
+			/* don't get written to the file.		*/
+		        break;
+		    case ELEMENT_LINE:
+		        /* end point */
+			sprintf(istr, "%d", elem->area.r_xtop);
+			AppendString(&rstr, istr, " ");
+			sprintf(istr, "%d", elem->area.r_ytop);
+			/* any non-default flags? */
+			if (elem->flags & DBW_ELEMENT_LINE_HALFX)
+			    AppendString(&rstr, " halfx");
+			if (elem->flags & DBW_ELEMENT_LINE_HALFY)
+			{
+			    AppendString(&rstr,
+					(elem->flags & DBW_ELEMENT_LINE_HALFX) ?
+					"," : " ");
+			    AppendString(&rstr, "halfy");
+			}
+			AppendString(&rstr, istr, "\n");
+		        break;
+		    case ELEMENT_TEXT:
+		        /* label text */
+		        AppendString(&rstr, "\"", NULL);
+		        AppendString(&rstr, elem->text, NULL);
+		        AppendString(&rstr, "\"", NULL);
+			/* any non-default flags? */
+			flagset = 0x0;
+			if (((elem->flags & DBW_ELEMENT_TEXT_POS) >> 4) !=
+					GEO_CENTER)
+			    flagset |= 0x1;
+			if (((elem->flags & DBW_ELEMENT_TEXT_SIZE) >> 1) !=
+					GR_TEXT_MEDIUM)
+			    flagset |= 0x2;
+			if (flagset != 0)
+			    AppendString(&rstr, " ", NULL);
+			if (flagset & 0x1)
+			    AppendString(&rstr, GeoPosToName((elem->flags &
+					DBW_ELEMENT_TEXT_POS) >> 4),
+					(flagset & 0x2) ? "," : NULL);
+			if (flagset & 0x2)
+			    AppendString(&rstr, textSizes[(elem->flags &
+					DBW_ELEMENT_TEXT_SIZE) >> 1], 
+					NULL);
+		        AppendString(&rstr, "\n", NULL);
+		        break;
+		}
+	    }
+	}
+    }
+    return rstr;
+}
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -133,6 +286,31 @@ DBWElementRedraw(window, plane)
 	 */
 
 	WindSurfaceToScreen(window, &elem->area, &screenArea);
+
+	/* Deal with half-point-offset flags for the line element */
+	if ((elem->type == ELEMENT_LINE) &&
+		(elem->flags & (DBW_ELEMENT_LINE_HALFX | DBW_ELEMENT_LINE_HALFY)))
+	{
+	    static Rect unitArea = {{0, 0}, {1, 1}};
+	    Rect transArea;
+	    int offx, offy;
+
+	    WindSurfaceToScreen(window, &unitArea, &transArea);
+	    offx = (transArea.r_xtop - transArea.r_xbot) >> 1;
+	    offy = (transArea.r_ytop - transArea.r_ybot) >> 1;
+
+	    if (elem->flags & DBW_ELEMENT_LINE_HALFX)
+	    {
+		screenArea.r_xbot += offx;
+		screenArea.r_xtop += offx;
+	    }
+	    if (elem->flags & DBW_ELEMENT_LINE_HALFY)
+	    {
+		screenArea.r_ybot += offy;
+		screenArea.r_ytop += offy;
+	    }
+	}
+
 	if ((screenArea.r_xbot <= screenArea.r_xtop) &&
 		(screenArea.r_ybot <= screenArea.r_ytop))
 	{
@@ -158,8 +336,9 @@ DBWElementRedraw(window, plane)
 			p.p_x = screenArea.r_xbot;
 			p.p_y = screenArea.r_ybot;
 			GrPutText(elem->text, curStyle, &p,
-				GEO_CENTER, GR_TEXT_MEDIUM, FALSE,
-				&(window->w_screenArea), (Rect *)NULL);
+				((elem->flags & DBW_ELEMENT_TEXT_POS) >> 4),
+				((elem->flags & DBW_ELEMENT_TEXT_SIZE) >> 1),
+				FALSE, &(window->w_screenArea), (Rect *)NULL);
 			break;
 		}
 	    }
@@ -185,23 +364,41 @@ dbwElementUndraw(mw, elem)
     CellDef *windowRoot;
     Rect screenArea, textArea;
 
+    if (mw == NULL) return;	/* No window; can't undraw */
     windowRoot = ((CellUse *) (mw->w_surfaceID))->cu_def;
   
     GrLock(mw, TRUE);
 
-    WindSurfaceToScreen(mw, &elem->area, &screenArea);
+    /* Deal with half-point-offset flags for the line element	*/
+    /* by enlarging the area by 1 unit.				*/
+
+    if ((elem->type == ELEMENT_LINE) &&
+		(elem->flags & (DBW_ELEMENT_LINE_HALFX | DBW_ELEMENT_LINE_HALFY)))
+    {
+	Rect newArea = elem->area;
+
+	if (elem->flags & DBW_ELEMENT_LINE_HALFX) newArea.r_xtop++;
+	if (elem->flags & DBW_ELEMENT_LINE_HALFY) newArea.r_ytop++;
+
+	WindSurfaceToScreen(mw, &newArea, &screenArea);
+    }
 
     /* For text, determine the size of the text and expand the	*/
     /* screenArea rectangle accordingly.			*/
 
-    if (elem->type == ELEMENT_TEXT)
+    else if (elem->type == ELEMENT_TEXT)
     {
-	GrLabelSize(elem->text, GEO_CENTER, GR_TEXT_MEDIUM, &textArea);
+	int tpos = (elem->flags & DBW_ELEMENT_TEXT_POS) >> 4;
+	int tsize = (elem->flags & DBW_ELEMENT_TEXT_SIZE) >> 1;
+
+	GrLabelSize(elem->text, tpos, tsize, &textArea);
 	screenArea.r_xbot += textArea.r_xbot;
 	screenArea.r_ybot += textArea.r_ybot;
 	screenArea.r_xtop += textArea.r_xtop;
 	screenArea.r_ytop += textArea.r_ytop;
     }
+    else
+	WindSurfaceToScreen(mw, &elem->area, &screenArea);
 
     if ((screenArea.r_xbot <= screenArea.r_xtop) &&
 		(screenArea.r_ybot <= screenArea.r_ytop))
@@ -249,6 +446,10 @@ DBWElementDelete(MagWindow *w, char *name)
 
     dbwElementUndraw(w, elem);
 
+    /* mark element's cell as having been modified */
+    if (elem->flags & DBW_ELEMENT_PERSISTENT)
+	elem->rootDef->cd_flags |= CDMODIFIED;
+
     for (stylePtr = elem->stylelist; stylePtr != NULL; stylePtr = stylePtr->next)
     {
 	freeMagic(stylePtr);
@@ -403,7 +604,9 @@ DBWElementAdd(w, name, area, cellDef, style)
      */
     
     if (!DBSrRoots(cellDef, &GeoIdentityTransform,
-	dbwelemGetTransform, (ClientData) &transform)) return;
+		dbwelemGetTransform, (ClientData) &transform))
+	if (w != NULL)
+	    return NULL;
 
     /* SigInterruptPending screws up DBSrRoots */
     if (SigInterruptPending)
@@ -420,8 +623,15 @@ DBWElementAdd(w, name, area, cellDef, style)
     elem->stylelist = (styleptr)mallocMagic(sizeof(stylestruct));
     elem->stylelist->style = style;
     elem->stylelist->next = NULL;
-    elem->rootDef = dbwelemRootDef;
+
+    /* For .mag file loads, w will be NULL and cellDef will be	*/
+    /* the root.						*/
+    if (w == NULL)
+        elem->rootDef = cellDef;
+    else
+	elem->rootDef = dbwelemRootDef;
     elem->text = NULL;
+    elem->flags = 0;
 
     return elem;
 }
@@ -439,8 +649,8 @@ DBWElementAddRect(w, name, area, cellDef, style)
     DBWElement *elem;
 
     elem = DBWElementAdd(w, name, area, cellDef, style);
+    if (elem == NULL) return;
     elem->type = ELEMENT_RECT;
-    elem->text = NULL;
 }
 
 void
@@ -456,8 +666,8 @@ DBWElementAddLine(w, name, area, cellDef, style)
     DBWElement *elem;
 
     elem = DBWElementAdd(w, name, area, cellDef, style);
+    if (elem == NULL) return;
     elem->type = ELEMENT_LINE;
-    elem->text = NULL;
 }
 
 void
@@ -480,8 +690,11 @@ DBWElementAddText(w, name, x, y, text, cellDef, style)
     area.r_ytop = y;
 
     elem = DBWElementAdd(w, name, &area, cellDef, style);
+    if (elem == NULL) return;
     elem->type = ELEMENT_TEXT;
     elem->text = StrDup(NULL, text);
+    elem->flags |= (GEO_CENTER << 4);		/* default centered */
+    elem->flags |= (GR_TEXT_MEDIUM << 1);	/* default medium size */
 }
 
 /* This utility procedure is invoked by DBSrRoots.  Save the root definition
@@ -573,6 +786,136 @@ DBWElementText(MagWindow *w, char *ename, char *text)
     }
 }
 
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * DBWElementParseFlags --
+ *
+ * 	Configures flags of any element
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	Element's flags are set.
+ *	If altered, the element is erased and redrawn.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+void
+DBWElementParseFlags(MagWindow *w, char *ename, char *flagstr)
+{
+    DBWElement *elem;
+    HashEntry *entry;
+    int flidx, newflags;
+
+    static char *lineOffset[] = {"halfx", "halfy", "exactx", "exacty", NULL};
+    static char *textSizes[] = {"small", "medium", "large", "xlarge",
+	"default", NULL}; 
+    static char *genFlags[] = {"persistent", "temporary", NULL}; 
+
+    entry = HashFind(&elementTable, ename);
+    if (entry == NULL)
+    {
+	TxError("No such element %s\n", ename);
+	return;
+    }	
+
+    elem = (DBWElement *)HashGetValue(entry);
+    if (elem == NULL) return;
+    newflags = elem->flags;
+
+    /* Return list of known flags */
+    if (flagstr == NULL)
+    {
+#ifdef MAGIC_WRAPPER
+	Tcl_AppendElement(magicinterp, "(flags)");
+#else
+	TxPrintf("%s\n", "(flags)");
+#endif
+    }
+    else
+    {
+	/* Parse string for known flags */
+	flidx = Lookup(flagstr, genFlags);
+	switch (flidx)
+	{
+	    case 0:
+		newflags |= DBW_ELEMENT_PERSISTENT;
+		break;
+	    case 1:
+		newflags &= ~DBW_ELEMENT_PERSISTENT;
+		break;
+	    default:
+		switch (elem->type)
+		{
+		    case ELEMENT_TEXT:
+			flidx = Lookup(flagstr, textSizes);
+			if (flidx >= 0)
+			{
+			    newflags &= ~DBW_ELEMENT_TEXT_SIZE;
+			    newflags |= (flidx << 1) & DBW_ELEMENT_TEXT_SIZE;
+			}
+			else
+			{
+			    flidx = GeoNameToPos(flagstr, FALSE, FALSE);
+			    if (flidx >= 0)
+			    {
+				newflags &= ~DBW_ELEMENT_TEXT_POS;
+				newflags |= (flidx << 4) & DBW_ELEMENT_TEXT_POS;
+			    }
+			    else
+				TxError("No such text element flag \"%s\"\n",
+					flagstr);
+			}
+			break;
+		    case ELEMENT_LINE:
+			flidx = Lookup(flagstr, lineOffset);
+			switch(flidx)
+			{
+			    case 0:
+				newflags |= DBW_ELEMENT_LINE_HALFX;
+				break;
+			    case 1:
+				newflags |= DBW_ELEMENT_LINE_HALFY;
+				break;
+			    case 2:
+				newflags &= ~(DBW_ELEMENT_LINE_HALFX);
+				break;
+			    case 3:
+				newflags &= ~(DBW_ELEMENT_LINE_HALFY);
+				break;
+			    default:
+				TxError("No such line element flag \"%s\"\n",
+					flagstr);
+				break;
+			}
+			break;
+		    case ELEMENT_RECT:
+			TxError("No such rect element flag \"%s\"\n",
+					flagstr);
+			break;
+		}
+		break;
+	}
+
+	if (newflags != elem->flags)
+	{
+	    dbwElementUndraw(w, elem);
+
+	    /* Mark element's cell as having been modified either if	*/
+	    /* it is persistent or if its persistence has changed.	*/
+
+	    if (elem->flags & DBW_ELEMENT_PERSISTENT ||
+			newflags & DBW_ELEMENT_PERSISTENT)
+		elem->rootDef->cd_flags |= CDMODIFIED;
+
+	    elem->flags = newflags;
+	}
+    }
+}
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -672,6 +1015,9 @@ DBWElementStyle(MagWindow *w, char *ename, int style, bool add)
 		sptr->next = sptr->next->next;
 	    }
 	}
+	/* mark element's cell as having been modified */
+	if (elem->flags & DBW_ELEMENT_PERSISTENT)
+	    elem->rootDef->cd_flags |= CDMODIFIED;
     }
 }
 
@@ -737,5 +1083,8 @@ DBWElementPos(MagWindow *w, char *ename, Rect *crect)
     {
 	dbwElementUndraw(w, elem);
 	elem->area = *crect;
+	/* mark element's cell as having been modified */
+	if (elem->flags & DBW_ELEMENT_PERSISTENT)
+	    elem->rootDef->cd_flags |= CDMODIFIED;
     }
 }
