@@ -25,7 +25,7 @@
 
 #ifndef	lint
 static char rcsid[] = "$Header: /ufs/repository/magic/drc/DRCcif.c,v 1.2 2001/01/12 22:12:28 jsolomon Exp $";
-#endif	not lint
+#endif	/* not lint */
 
 #include <sys/types.h>
 #include <stdio.h>
