@@ -18,7 +18,7 @@
 
 #ifndef	lint
 static char rcsid[] = "$Header: /ufs/repository/magic/drc/DRCprint.c,v 1.2 2001/01/12 22:12:29 jsolomon Exp $";
-#endif	not lint
+#endif	/* not lint */
 
 #include <stdio.h>
 #include <sys/types.h>
