@@ -20,7 +20,7 @@
 
 #ifndef	lint
 static char rcsid[] = "$Header: /ufs/repository/magic/drc/DRCsubcell.c,v 1.2 2001/01/12 22:12:29 jsolomon Exp $";
-#endif	not lint
+#endif	/* not lint */
 
 #include <stdio.h>
 #include <sys/types.h>
@@ -103,7 +103,7 @@ drcFindOtherCells(tile, area)
  * drcSubcellTileFunc --
  *
  * 	Called by TiSrArea when looking for interactions in
- *	a given area.  It sees if this subcell tell participates
+ *	a given area.  It sees if this subcell tile participates
  *	in any interactions in the area we're rechecking.
  *
  * Results:
@@ -348,8 +348,6 @@ drcExactOverlapCheck(tile, arg)
     Rect rect;
 
     TiToRect(tile, &rect);
-    if (GEO_SURROUND(arg->dCD_rect, &rect)) return 0;
-
     GeoClip(&rect, arg->dCD_clip);
     if (GEO_RECTNULL(&rect)) return 0;
 
@@ -392,20 +390,94 @@ drcExactOverlapTile(tile, cxp)
 				 */
 {
     struct drcClientData *arg;
-    TileTypeBitMask typeMask;
-    Rect r1, r2;
+    TileTypeBitMask typeMask, invMask;
+    TileType type, t;
+    Tile *tp;
+    Rect r1, r2, r3, rex;
     int i;
     
     arg = (struct drcClientData *) cxp->tc_filter->tf_arg;
     TiToRect(tile, &r1);
     GeoTransRect(&(cxp->tc_scx->scx_trans), &r1, &r2);
-    arg->dCD_rect = &r2;
-    GEO_EXPAND(&r2, 1, &r1);
+
+    GEO_EXPAND(&r2, 1, &rex); 	/* Area includes abutting tiles */
+    GeoClip(&rex, arg->dCD_clip);	/* Except areas outside search area */
+
+    type = TiGetType(tile);
+    TTMaskSetOnlyType(&typeMask, type);
+    for (t = DBNumUserLayers; t < DBNumTypes; t++)
+	if (TTMaskHasType(DBResidueMask(t), type))
+	    TTMaskSetType(&typeMask, t);
+    TTMaskCom2(&invMask, &typeMask);
+
     for (i = PL_TECHDEPBASE; i < DBNumPlanes; i++)
     {
-	TTMaskSetOnlyType(&typeMask, TiGetType(tile));
-        (void) DBSrPaintArea((Tile *) NULL, DRCdef->cd_planes[i],
-	    &r1, &typeMask, drcExactOverlapCheck, (ClientData) arg);
+        if (DBSrPaintArea((Tile *) NULL, DRCdef->cd_planes[i],
+	    &rex, &typeMask, drcAlwaysOne, (ClientData) NULL))
+	{
+	    /* There is an overlap or abutment of ExactOverlap types. */
+	    /* 1) Check if any invalid type is under this tile.	*/
+
+	    arg->dCD_rect = &r2;
+	    DBSrPaintArea((Tile *) NULL, DRCdef->cd_planes[i], &r2,
+			&invMask, drcExactOverlapCheck, (ClientData) arg);
+
+	    /* 2) Search the neighboring tiles for types that do not	*/
+	    /* match the exact overlap type, and flag abutment errors.	*/
+
+	    /* Search bottom */
+	    arg->dCD_rect = &r3;
+	    for (tp = LB(tile); LEFT(tp) < RIGHT(tile); tp = TR(tp))
+		if (TTMaskHasType(&invMask, TiGetType(tp)))
+		{
+		    TiToRect(tp, &r1);
+		    GeoTransRect(&(cxp->tc_scx->scx_trans), &r1, &r3);
+		    GeoClip(&r3, &rex);
+		    if (!GEO_RECTNULL(&r3))
+			DBSrPaintArea((Tile *) NULL, DRCdef->cd_planes[i],
+				&r3, &typeMask, drcExactOverlapCheck,
+				(ClientData) arg);
+		}
+
+	    /* Search right */
+	    for (tp = TR(tile); TOP(tp) > BOTTOM(tile); tp = LB(tp))
+		if (TTMaskHasType(&invMask, TiGetType(tp)))
+		{
+		    TiToRect(tp, &r1);
+		    GeoTransRect(&(cxp->tc_scx->scx_trans), &r1, &r3);
+		    GeoClip(&r3, &rex);
+		    if (!GEO_RECTNULL(&r3))
+			DBSrPaintArea((Tile *) NULL, DRCdef->cd_planes[i],
+				&r3, &typeMask, drcExactOverlapCheck,
+				(ClientData) arg);
+		}
+
+	    /* Search top */
+	    for (tp = RT(tile); RIGHT(tp) > LEFT(tile); tp = BL(tp))
+		if (TTMaskHasType(&invMask, TiGetType(tp)))
+		{
+		    TiToRect(tp, &r1);
+		    GeoTransRect(&(cxp->tc_scx->scx_trans), &r1, &r3);
+		    GeoClip(&r3, &rex);
+		    if (!GEO_RECTNULL(&r3))
+			DBSrPaintArea((Tile *) NULL, DRCdef->cd_planes[i],
+				&r3, &typeMask, drcExactOverlapCheck,
+				(ClientData) arg);
+		}
+
+	    /* Search left */
+	    for (tp = BL(tile); BOTTOM(tp) < TOP(tile); tp = RT(tp))
+		if (TTMaskHasType(&invMask, TiGetType(tp)))
+		{
+		    TiToRect(tp, &r1);
+		    GeoTransRect(&(cxp->tc_scx->scx_trans), &r1, &r3);
+		    GeoClip(&r3, &rex);
+		    if (!GEO_RECTNULL(&r3))
+			DBSrPaintArea((Tile *) NULL, DRCdef->cd_planes[i],
+				&r3, &typeMask, drcExactOverlapCheck,
+				(ClientData) arg);
+		}
+	}
     }
     return 0;
 }
@@ -521,8 +593,8 @@ DRCInteractionCheck(def, area, func, cdarg)
 	    arg.dCD_cptr = &drcSubcellCookie;
 	    arg.dCD_function = func;
 	    arg.dCD_clientData = cdarg;
-	    (void) DBTreeSrTiles(&scx, &DRCExactOverlapTypes, 0,
-		drcExactOverlapTile, (ClientData) &arg);
+	    (void) DBTreeSrUniqueTiles(&scx, &DRCExactOverlapTypes,
+			0, drcExactOverlapTile, (ClientData) &arg);
 	}
     
     /* Update count of interaction tiles processed. */
