@@ -80,6 +80,7 @@ static int drcRulesOptimized = 0;
  * Forward declarations.
  */
 int drcWidth(), drcSpacing(), drcEdge(), drcNoOverlap(), drcExactOverlap();
+int drcSurround(), drcRectOnly(), drcOverhang();
 int drcStepSize();
 #ifdef DRC_EXTENSIONS
 int drcMaxwidth(), drcArea(), drcRectangle();
@@ -228,6 +229,7 @@ DRCTechInit()
 {
     register int i, j, plane;
     register DRCCookie *dp;
+    PaintResultType result;
 
     TechHalo = 0;
     drcRulesOptimized = 0;
@@ -272,32 +274,40 @@ DRCTechInit()
      */
     
     for (plane = 0; plane < DBNumPlanes; plane++)
-    {
-	/* printf("Plane is %s.\n", DBPlaneLongName(plane)); */
 	for (i = 0; i < DBNumTypes; i++)
 	    for (j = 0; j < DBNumTypes; j++)
 	    {
-		PaintResultType result = DBPaintResultTbl[plane][i][j];
+		result = DBPaintResultTbl[plane][i][j];
 		if ((i == TT_ERROR_S) || (j == TT_ERROR_S))
 		    DRCPaintTable[plane][i][j] = TT_ERROR_S;
 		else if ((i == TT_SPACE) || (j == TT_SPACE)
 			|| !DBTypeOnPlane(j, plane)
 			|| !DBPaintOnTypePlanes(i, j))
 		    DRCPaintTable[plane][i][j] = result;
+
+		/* Modified for stackable types (Tim, 10/3/03) */
+		else if ((i >= DBNumUserLayers) ||
+			((result >= DBNumUserLayers)
+			&& (DBTechFindStacking(i, j) == result)))
+		{
+		    DRCPaintTable[plane][i][j] = result;
+		}
+			
 		else if ((!TTMaskHasType(&DBLayerTypeMaskTbl[i], result)
 			&& !TTMaskHasType(&DBLayerTypeMaskTbl[j], result))
 			|| ((result != DBPaintResultTbl[plane][j][i])
 			&& DBTypeOnPlane(i, plane)
-			|| DBPaintOnTypePlanes(j, i)))
+			&& DBPaintOnTypePlanes(j, i)))
 		{
 		    DRCPaintTable[plane][i][j] = TT_ERROR_S;
-		    /* printf("Error: %s on %s, was %s\n", DBTypeShortName(i),
-			    DBTypeShortName(j), DBTypeShortName(result)); */
+		    /* TxError("Error: %s on %s, was %s\n",
+				DBTypeLongNameTbl[i], DBTypeLongNameTbl[j],
+				DBTypeLongNameTbl[result]); */
 		}
 		else
 		    DRCPaintTable[plane][i][j] = result;
 	    }
-    }
+
 #ifdef DRC_CIF_EXTENSIONS
     drcCifInit();
 #endif
@@ -337,6 +347,13 @@ DRCTechInit()
  * ----------------------------------------------------------------------------
  */
 
+/* DRC rule assignment has been modified to limit the mask and corner	*/
+/* types to the "user layers" only.  Because stacked contacts are	*/
+/* amalgams of two or more other types, the DRCTile routine decomposes	*/
+/* each stacked tile encountered into its constituent residue (contact)	*/
+/* types and applies the rules separately to each.  Therefore,		*/
+/* drcc_mask and drcc_corner should never contain stacked types.	*/
+
 #define ASSIGN(cookie,dist,next,mask,corner,why,cdist,flags,plane) \
 	((cookie)->drcc_dist = dist, \
 	(cookie)->drcc_next = next, \
@@ -345,7 +362,9 @@ DRCTechInit()
 	(cookie)->drcc_why = why, \
 	(cookie)->drcc_cdist = cdist, \
 	(cookie)->drcc_flags = flags, \
-	(cookie)->drcc_plane = plane)
+	(cookie)->drcc_plane = plane, \
+	TTMaskAndMask(&(cookie)->drcc_mask, &DBUserLayerBits), \
+	TTMaskAndMask(&(cookie)->drcc_corner, &DBUserLayerBits))
 
 	/* ARGSUSED */
 bool
@@ -372,10 +391,16 @@ DRCTechAddRule(sectionName, argc, argv)
     "layers",
 	"no_overlap",	 3,	3,	drcNoOverlap,
     "layers1 layers2",
+	"overhang",	 5,	5,	drcOverhang,
+    "layers1 layers2 distance why",
+	"rect_only",	 3,	3,	drcRectOnly,
+    "layers why",
 	"spacing",	 6,	6,	drcSpacing,
     "layers1 layers2 separation adjacency why",
 	"stepsize",	 2,	2,	drcStepSize,
     "step_size",
+	"surround",	 6,	6,	drcSurround,
+    "layers1 layers2 distance presence why",
 	"width",	 4,	4,	drcWidth,
     "layers width why",
 	"widespacing",	 7,	7,	drcSpacing,
@@ -473,17 +498,20 @@ drcWidth(argc, argv)
     DBTechNoisyNameMask(layers, &set);
     for (plane = PL_TECHDEPBASE; plane < DBNumPlanes; plane++)
 	if (DBTechSubsetLayers(set, DBPlaneTypes[plane], &tmp1))
-	    goto widthOK;
-    TechError("All layers for \"width\" must be on same plane\n");
-    return (0);
+	    break;
+    
+    if (plane == DBNumPlanes)
+    {
+	TechError("All layers for \"width\" must be on same plane\n");
+	return (0);
+    }
 
-widthOK:
     set = tmp1;
     TTMaskCom2(&setC, &set);
 
-    for (i = 0; i < DBNumTypes; i++)
+    for (i = 0; i < DBNumUserLayers; i++)
     {
-	for (j = 0; j < DBNumTypes; j++)
+	for (j = 0; j < DBNumUserLayers; j++)
 	{
 	    /*
 	     * Must have types in 'set' for at least 'distance'
@@ -548,17 +576,20 @@ drcArea(argc, argv)
     DBTechNoisyNameMask(layers, &set);
     for (plane = PL_TECHDEPBASE; plane < DBNumPlanes; plane++)
 	if (DBTechSubsetLayers(set, DBPlaneTypes[plane], &tmp1))
-	    goto widthOK;
-    TechError("All layers for \"area\" must be on same plane\n");
-    return (0);
+	    break;
+
+    if (plane == DBNumPlanes)
+    {
+	TechError("All layers for \"area\" must be on same plane\n");
+	return (0);
+    }
 
-widthOK:
     set = tmp1;
     TTMaskCom2(&setC, &set);
 
-    for (i = 0; i < DBNumTypes; i++)
+    for (i = 0; i < DBNumUserLayers; i++)
     {
-	for (j = 0; j < DBNumTypes; j++)
+	for (j = 0; j < DBNumUserLayers; j++)
 	{
 	    /*
 	     * Must have types in 'set' for at least 'distance'
@@ -638,11 +669,14 @@ drcMaxwidth(argc, argv)
     DBTechNoisyNameMask(layers, &set);
     for (plane = PL_TECHDEPBASE; plane < DBNumPlanes; plane++)
 	if (DBTechSubsetLayers(set, DBPlaneTypes[plane], &tmp1))
-	    goto widthOK;
-    TechError("All layers for \"area\" must be on same plane\n");
-    return (0);
+	    break;
+
+    if (plane == DBNumPlanes)
+    {
+	TechError("All layers for \"area\" must be on same plane\n");
+	return (0);
+    }
 
-widthOK:
     if (strcmp(bends,"bend_illegal") == 0) bend =0;
     else if (strcmp(bends,"bend_ok") == 0) bend =DRC_BENDS;
     else
@@ -653,9 +687,9 @@ widthOK:
     set = tmp1;
     TTMaskCom2(&setC, &set);
 
-    for (i = 0; i < DBNumTypes; i++)
+    for (i = 0; i < DBNumUserLayers; i++)
     {
-	for (j = 0; j < DBNumTypes; j++)
+	for (j = 0; j < DBNumUserLayers; j++)
 	{
 	    /*
 	     * Must have types in 'set' for at least 'distance'
@@ -827,8 +861,8 @@ drcSpacing(argc, argv)
 	 */
 	if (TTMaskIntersect(&set1, &set2))
 	{
-	    TechError(
-		"Spacing check with touching illegal must be between non intersecting type lists.\n");
+	    TechError("Spacing check with touching illegal must be between "
+			"non-intersecting type lists.\n");
 	    return (0);
 	}
 
@@ -850,9 +884,9 @@ drcSpacing(argc, argv)
 	return (0);
     }
 
-    for (i = 0; i < DBNumTypes; i++)
+    for (i = 0; i < DBNumUserLayers; i++)
     {
-	for (j = 0; j < DBNumTypes; j++)
+	for (j = 0; j < DBNumUserLayers; j++)
 	{
 	    if ((i == j) || !DBTypesOnSamePlane(i, j)) continue;
 
@@ -876,6 +910,7 @@ drcSpacing(argc, argv)
 		    MALLOC(DRCCookie *, dpnew, sizeof (DRCCookie));
 		    TTMaskClearMask3(&tmp1, &DBPlaneTypes[curPlane], &set2);
 		    TTMaskAndMask3(&tmp2, &DBPlaneTypes[curPlane], &setR);
+
 		    if (widerule)
 		    {
 			DRCCookie *dptrig;
@@ -977,6 +1012,7 @@ drcSpacing(argc, argv)
 		    MALLOC(DRCCookie *, dpnew, sizeof (DRCCookie));
 		    TTMaskClearMask3(&tmp1, &DBPlaneTypes[curPlane], &set1);
 		    TTMaskAndMask3(&tmp2, &DBPlaneTypes[curPlane], &setRreverse);
+
 		    ASSIGN(dpnew, distance, dp->drcc_next,
 			tmp1, tmp2, why, distance, DRC_FORWARD, curPlane);
 
@@ -1070,6 +1106,7 @@ drcSpacing(argc, argv)
 		    MALLOC(DRCCookie *, dpnew, sizeof (DRCCookie));
 		    TTMaskClearMask3(&tmp1, &DBPlaneTypes[curPlane], &set2);
 		    TTMaskZero(&tmp2);
+
 		    ASSIGN(dpnew, 1, dp->drcc_next,
 			tmp1, tmp2, why, distance, 
 			DRC_FORWARD | DRC_XPLANE, curPlane);
@@ -1136,23 +1173,23 @@ drcEdge(argc, argv)
      * found on one plane.
      */
     for (plane = PL_TECHDEPBASE; plane < DBNumPlanes; plane++)
-    {
 	if (DBTechSubsetLayers(set1, DBPlaneTypes[plane], &tmp1)
-	    && DBTechSubsetLayers(set2, DBPlaneTypes[plane], &tmp2))
-	    goto edgeOK;
+		&& DBTechSubsetLayers(set2, DBPlaneTypes[plane], &tmp2))
+	    break;
+
+    if (plane == DBNumPlanes)
+    {
+	TechError("All edges in edge rule must lie in one plane.\n");
+	return (0);
     }
-    TechError("All edges in edge rule must lie in one plane.\n");
-    return (0);
 
-edgeOK:
     set1 = tmp1;
     set2 = tmp2;
 
     /* Give warning if types1 and types2 intersect */
-    if(TTMaskIntersect(&set1,&set2))
-    {
-	TechError("Warning:  types1 and types2 have nonempty intersection.  DRC does not check edges with the same type on both sides.\n");
-    }
+    if(TTMaskIntersect(&set1, &set2))
+	TechError("Warning:  types1 and types2 have nonempty intersection.  "
+		"DRC does not check edges with the same type on both sides.\n");
 
     DBTechNoisyNameMask(cornerTypes, &tmp3);
     if (!DBTechSubsetLayers(tmp3, DBPlaneTypes[plane], &setC))
@@ -1161,24 +1198,34 @@ edgeOK:
 	return (0);
     }
 
-    checkPlane = plane;
-    if (argc == 9)
-    {
-	checkPlane = DBTechNoisyNamePlane(argv[8]);
-	if (checkPlane < 0)
-	    return (0);
-    }
-
+    /*
+     * OKtypes determine the checkPlane.  If checkPlane exists, it should
+     * only be used to check against the plane of OKtypes.
+     */
     DBTechNoisyNameMask(okTypes, &setM);
-    if (!DBTechSubsetLayers(setM, DBPlaneTypes[checkPlane], &setM))
+    for (checkPlane = PL_TECHDEPBASE; checkPlane < DBNumPlanes; checkPlane++)
+	if (DBTechSubsetLayers(setM, DBPlaneTypes[checkPlane], &tmp3))
+	    break;
+ 
+    if (checkPlane == DBNumPlanes)
     {
-	TechError("OK types aren't all in the right plane.\n");
+	TechError("All OK types must lie in one plane.\n");
 	return (0);
     }
 
-    for (i = 0; i < DBNumTypes; i++)
+    setM = tmp3;
+
+    /* "plane" argument deprecated; kept for backward compatibility only */
+    if (argc == 9)
     {
-	for (j = 0; j < DBNumTypes; j++)
+	int tmpPlane = DBTechNoisyNamePlane(argv[8]);
+	if ((tmpPlane < 0) || (tmpPlane != checkPlane))
+	    TechError("Ignoring bad plane argument.\n");
+    }
+
+    for (i = 0; i < DBNumUserLayers; i++)
+    {
+	for (j = 0; j < DBNumUserLayers; j++)
 	{
 	    if (TTMaskHasType(&set1, i) && TTMaskHasType(&set2, j))
 	    {
@@ -1210,6 +1257,361 @@ edgeOK:
 
     return (MAX(distance, cdist));
 }
+
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * drcOverhang --
+ *
+ * Process an overhang rule.
+ * This is of the form:
+ *
+ *	overhang layers1 layers2 dist why
+ *
+ * indicating that layers2 must overhang layers1 by a distance of at least
+ * dist.
+ *
+ * This rule is equivalent to:
+ *
+ *	edge4way layers1 space/p2|layers2 dist layers1|layers2 0 0 why
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+drcOverhang(argc, argv)
+    int argc;
+    char *argv[];
+{
+    char *layers2 = argv[1], *layers1 = argv[2];
+    int distance = atoi(argv[3]);
+    char *why = drcWhyDup(argv[4]);
+
+    TileTypeBitMask set1, set2, tmp1, tmp2, setM, setC;
+
+    DRCCookie *dp, *dpnew;
+    int plane;
+    TileType i, j;
+
+    DBTechNoisyNameMask(layers1, &set1);
+    DBTechNoisyNameMask(layers2, &set2);
+
+    /* Give warning if types1 and types2 intersect */
+    if(TTMaskIntersect(&set1, &set2))
+	TechError("Warning:  inside and outside types have nonempty intersection.  "
+		"DRC does not check edges with the same type on both sides.\n");
+
+    /* Check that contents of each set are in the same plane */
+    for (plane = PL_TECHDEPBASE; plane < DBNumPlanes; plane++)
+	if (DBTechSubsetLayers(set1, DBPlaneTypes[plane], &tmp1)
+		&& DBTechSubsetLayers(set2, DBPlaneTypes[plane], &tmp2))
+	    break;
+
+    if (plane == DBNumPlanes)
+    {
+	TechError("All types in \"overhang\" must be on the same plane\n");
+	return (0);
+    }
+
+    set1 = tmp1;
+    set2 = tmp2;
+
+    /* SetM is the union of set1 and set2 */
+    TTMaskSetMask3(&setM, &set1, &set2);
+
+    /* Add space to set2 */
+    TTMaskSetType(&set2, TT_SPACE);
+
+    /* SetC is the empty set */
+    TTMaskZero(&setC);
+
+    for (i = 0; i < DBNumUserLayers; i++)
+    {
+	for (j = 0; j < DBNumUserLayers; j++)
+	{
+	    if (TTMaskHasType(&set1, i) && TTMaskHasType(&set2, j))
+	    {
+		/* Find bucket preceding the new one we wish to insert */
+		dp = drcFindBucket(i, j, distance);
+		MALLOC(DRCCookie *, dpnew, sizeof (DRCCookie));
+		ASSIGN(dpnew, distance, dp->drcc_next,
+			setM, setC, why, 0, DRC_FORWARD, plane);
+		dpnew->drcc_flags |= DRC_BOTHCORNERS;
+		dp->drcc_next = dpnew;
+
+		/* find bucket preceding new one we wish to insert */
+		dp = drcFindBucket(j, i, distance);
+		MALLOC(DRCCookie *, dpnew, sizeof (DRCCookie));
+		ASSIGN(dpnew, distance, dp->drcc_next,
+			setM, setC, why, 0, DRC_REVERSE, plane);
+		dpnew->drcc_flags |= DRC_BOTHCORNERS;
+		dp->drcc_next = dpnew;
+	    }
+	}
+    }
+    return distance;
+}
+
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * drcRectOnly --
+ *
+ * Process a rectangle-only rule.  This rule prohibits non-rectangular
+ * geometry, and is used especially for contacts, as the "squares" operator
+ * in the CIF/GDS output generator can't handle non-rectangular areas.
+ * The rule is of the form:
+ *
+ *	rect_only layers why
+ *
+ * and is equivalent to:
+ *
+ *	edge4way layers ~(layers)/plane 1 ~(layers)/plane (all_layers)/plane 1
+ *
+ * The rect_only rule avoids the above contrived construction, especially the
+ * requirement of specifying "all_layers" as something like (~(x),x)/p, a sure-
+ * fire obfuscation.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+drcRectOnly(argc, argv)
+    int argc;
+    char *argv[];
+{
+    char *layers = argv[1];
+    char *why = drcWhyDup(argv[2]);
+    TileTypeBitMask set1, set2, setC, tmp1;
+    DRCCookie *dp, *dpnew;
+    int plane;
+    TileType i, j;
+
+    DBTechNoisyNameMask(layers, &set1);
+
+    /*
+     * Make sure that all edges between the two sets can be
+     * found on one plane.
+     */
+    for (plane = PL_TECHDEPBASE; plane < DBNumPlanes; plane++)
+	if (DBTechSubsetLayers(set1, DBPlaneTypes[plane], &tmp1))
+	    break;
+
+    if (plane == DBNumPlanes)
+    {
+	TechError("All types must lie in one plane.\n");
+	return (0);
+    }
+
+    set1 = tmp1;
+
+    /* set2 is the inverse of set1 in its plane */
+    TTMaskCom2(&set2, &set1);
+    TTMaskAndMask(&set2, &DBPlaneTypes[plane]);
+
+    /* setM = set2.  setC = all types in plane */
+    TTMaskZero(&setC);
+    TTMaskSetMask(&setC, &DBPlaneTypes[plane]);
+
+    for (i = 0; i < DBNumUserLayers; i++)
+    {
+	for (j = 0; j < DBNumUserLayers; j++)
+	{
+	    if (TTMaskHasType(&set1, i) && TTMaskHasType(&set2, j))
+	    {
+		/* Find bucket preceding the new one we wish to insert */
+		dp = drcFindBucket(i, j, 1);
+		MALLOC(DRCCookie *, dpnew, sizeof (DRCCookie));
+		ASSIGN(dpnew, 1, dp->drcc_next,
+		    set2, setC, why, 1, DRC_FORWARD, plane);
+		dpnew->drcc_flags |= DRC_BOTHCORNERS;
+		dp->drcc_next = dpnew;
+
+		/* find bucket preceding new one we wish to insert */
+		dp = drcFindBucket(j, i, 1);
+		MALLOC(DRCCookie *, dpnew, sizeof (DRCCookie));
+		ASSIGN(dpnew,1,dp->drcc_next,
+			    set2, setC, why, 1, DRC_REVERSE, plane);
+		dpnew->drcc_flags |= DRC_BOTHCORNERS;
+		dp->drcc_next = dpnew;
+	    }
+	}
+    }
+    return 1;
+}
+
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * drcSurround --
+ *
+ * Process a surround rule.
+ * This is of the form:
+ *
+ *	surround layers1 layers2 dist presence why
+ *
+ * indicating that layers2 must surround layers1 by at least distance
+ * dist in all directions.
+ *
+ * This rule is equivalent to:
+ *
+ *	edge4way ~(layers2)/plane2 layers2 dist ~(layers1)/plane1 \
+ *		layers2 dist why
+ *
+ * When presence=absence_illegal, the following additional rule is needed:
+ *
+ *	edge4way layers1 ~(layers2)/plane1 dist NULL ~(layers2)/plane1 \
+ *		dist why
+ *
+ * 
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+drcSurround(argc, argv)
+    int argc;
+    char *argv[];
+{
+    char *layers1 = argv[1], *layers2 = argv[2];
+    int distance = atoi(argv[3]);
+    char *presence = argv[4];
+    char *why = drcWhyDup(argv[5]);
+
+    TileTypeBitMask set1, set2, tmp1, tmp2, setM, invM;
+
+    DRCCookie *dp, *dpnew;
+    int plane1, plane2;
+    TileType i, j;
+
+    DBTechNoisyNameMask(layers1, &setM);
+    DBTechNoisyNameMask(layers2, &set2);
+
+    /* Give warning if types1 and types2 intersect */
+    if(TTMaskIntersect(&set2, &setM))
+	TechError("Warning:  inside and outside types have nonempty intersection.  "
+		"DRC does not check edges with the same type on both sides.\n");
+
+    /* Check that contents of each set are in the same plane */
+    for (plane1 = PL_TECHDEPBASE; plane1 < DBNumPlanes; plane1++)
+	if (DBTechSubsetLayers(setM, DBPlaneTypes[plane1], &tmp2))
+	    break;
+
+    if (plane1 == DBNumPlanes)
+    {
+	TechError("Inside types in \"surround\" must be on the same plane\n");
+	return (0);
+    }
+
+    for (plane2 = PL_TECHDEPBASE; plane2 < DBNumPlanes; plane2++)
+	if (DBTechSubsetLayers(set2, DBPlaneTypes[plane2], &tmp1))
+	    break;
+
+    if (plane2 == DBNumPlanes)
+    {
+	TechError("Outside types in \"surround\" must be on the same plane\n");
+	return (0);
+    }
+
+    set2 = tmp1;
+    setM = tmp2;
+
+    /* set1 is the inverse of set2 (restricted to plane2) */
+    TTMaskCom2(&set1, &set2);
+    TTMaskAndMask(&set1, &DBPlaneTypes[plane2]);
+
+    /* invert setM (restricted to plane1) */
+    TTMaskCom2(&invM, &setM);
+    TTMaskAndMask(&invM, &DBPlaneTypes[plane1]);
+
+    for (i = 0; i < DBNumUserLayers; i++)
+    {
+	for (j = 0; j < DBNumUserLayers; j++)
+	{
+	    if (TTMaskHasType(&set1, i) && TTMaskHasType(&set2, j))
+	    {
+		/* Find bucket preceding the new one we wish to insert */
+		dp = drcFindBucket(i, j, distance);
+		MALLOC(DRCCookie *, dpnew, sizeof (DRCCookie));
+		ASSIGN(dpnew, distance, dp->drcc_next,
+			invM, set2, why, distance, DRC_FORWARD, plane1);
+		if (plane2 != plane1)
+		    dpnew->drcc_flags |= DRC_XPLANE;
+		dpnew->drcc_flags |= DRC_BOTHCORNERS;
+		dp->drcc_next = dpnew;
+
+		/* find bucket preceding new one we wish to insert */
+		dp = drcFindBucket(j, i, distance);
+		MALLOC(DRCCookie *, dpnew, sizeof (DRCCookie));
+		ASSIGN(dpnew, distance, dp->drcc_next,
+			invM, set2, why, distance, DRC_REVERSE, plane1);
+		if (plane2 != plane1)
+		    dpnew->drcc_flags |= DRC_XPLANE;
+		dpnew->drcc_flags |= DRC_BOTHCORNERS;
+		dp->drcc_next = dpnew;
+	    }
+	}
+    }
+
+    if (strcmp(presence, "absence_illegal")) return distance;
+
+    /* To do---add extra rule when presence of the surrounding	*/
+    /* layer is required.  Rule is different if planes match.	*/
+
+    if (plane1 == plane2)
+    {
+	TTMaskZero(&invM);
+	TTMaskSetMask(&invM, &setM);
+	TTMaskSetMask(&invM, &set2);
+	TTMaskCom(&invM);
+	TTMaskAndMask(&invM, &DBPlaneTypes[plane1]);
+
+	TTMaskZero(&set1);
+
+	for (i = 0; i < DBNumUserLayers; i++)
+	    for (j = 0; j < DBNumUserLayers; j++)
+		if (TTMaskHasType(&setM, i) && TTMaskHasType(&invM, j))
+		{
+		    /* Find bucket preceding the new one we wish to insert */
+		    dp = drcFindBucket(i, j, distance);
+		    MALLOC(DRCCookie *, dpnew, sizeof (DRCCookie));
+		    ASSIGN(dpnew, distance, dp->drcc_next, set1,
+				invM, why, distance, DRC_FORWARD, plane2);
+		    dpnew->drcc_flags |= DRC_BOTHCORNERS;
+		    dp->drcc_next = dpnew;
+
+		    /* find bucket preceding new one we wish to insert */
+		    dp = drcFindBucket(j, i, distance);
+		    MALLOC(DRCCookie *, dpnew, sizeof (DRCCookie));
+		    ASSIGN(dpnew,distance,dp->drcc_next, set1,
+				invM, why, distance, DRC_REVERSE, plane2);
+		    dpnew->drcc_flags |= DRC_BOTHCORNERS;
+		    dp->drcc_next = dpnew;
+		}
+    }
+    else
+    {
+	for (i = 0; i < DBNumUserLayers; i++)
+	    for (j = 0; j < DBNumUserLayers; j++)
+		if (TTMaskHasType(&setM, i) && TTMaskHasType(&invM, j))
+		{
+		    /* Find bucket preceding the new one we wish to insert */
+		    dp = drcFindBucket(i, j, distance);
+		    MALLOC(DRCCookie *, dpnew, sizeof (DRCCookie));
+		    ASSIGN(dpnew, distance, dp->drcc_next,
+				set2, invM, why, distance, DRC_FORWARD, plane2);
+		    dpnew->drcc_flags |= DRC_BOTHCORNERS | DRC_XPLANE;
+		    dp->drcc_next = dpnew;
+
+		    /* find bucket preceding new one we wish to insert */
+		    dp = drcFindBucket(j, i, distance);
+		    MALLOC(DRCCookie *, dpnew, sizeof (DRCCookie));
+		    ASSIGN(dpnew,distance,dp->drcc_next,
+				set2, invM, why, distance, DRC_REVERSE, plane2);
+		    dpnew->drcc_flags |= DRC_BOTHCORNERS | DRC_XPLANE;
+		    dp->drcc_next = dpnew;
+		}
+    }
+    return distance;
+}
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -1253,8 +1655,8 @@ drcNoOverlap(argc, argv)
     DBTechNoisyNameMask(layers1, &set1);
     DBTechNoisyNameMask(layers2, &set2);
 
-    for (i = 0; i < DBNumTypes; i++)
-	for (j = 0; j < DBNumTypes; j++)
+    for (i = 0; i < DBNumUserLayers; i++)
+	for (j = 0; j < DBNumUserLayers; j++)
 	    if (TTMaskHasType(&set1, i) && TTMaskHasType(&set2, j))
 		for (plane = 0; plane < DBNumPlanes; plane++)
 		{
@@ -1294,7 +1696,7 @@ drcExactOverlap(argc, argv)
     char *argv[];
 {
     char *layers = argv[1];
-    TileTypeBitMask set;
+    TileTypeBitMask set, rmask;
     int i;
 
     /*
@@ -1304,17 +1706,18 @@ drcExactOverlap(argc, argv)
      */
 
     DBTechNoisyNameMask(layers, &set);
+    TTMaskSetMask(&DRCExactOverlapTypes, &set);
 
     /*
-     * Go through the types and eliminate all but the first image
-     * of each contact.
+     * Stacking types containing types in "set" must also be added
+     * to the list of exact overlap types.
      */
 
-    for (i = 0; i < DBNumTypes; i++)
+    for (i = DBNumUserLayers; i < DBNumTypes; i++)
     {
-	if (!TTMaskHasType(&set, i)) continue;
-	TTMaskSetType(&DRCExactOverlapTypes, i);
-	TTMaskClearMask(&set, &DBLayerTypeMaskTbl[i]);
+	TTMaskAndMask3(&rmask, &set, DBResidueMask(i));
+	if (!TTMaskIsZero(&rmask))
+	    TTMaskSetType(&DRCExactOverlapTypes, i);
     }
     return (0);
 }
@@ -1381,9 +1784,9 @@ drcRectangle(argc, argv)
     /* Install 2 edge rules: one that checks rectangle-ness, and one that
      * checks size
      */
-    for (i = 0; i < DBNumTypes; i++)
+    for (i = 0; i < DBNumUserLayers; i++)
     {
-	for (j = 0; j < DBNumTypes; j++)
+	for (j = 0; j < DBNumUserLayers; j++)
 	{
 	    if (TTMaskHasType(&types, i) && TTMaskHasType(&nottypes, j))
 	    {
@@ -1577,9 +1980,9 @@ DRCTechFinal()
      * case, eliminate all of the edge's rules except one.
      */
     
-    for (i = 0; i < DBNumTypes; i++)
+    for (i = 0; i < DBNumUserLayers; i++)
     {
-	for (j = 0; j < DBNumTypes; j++)
+	for (j = 0; j < DBNumUserLayers; j++)
 	{
 	    DRCCookie *keep = NULL;
 	    
@@ -1630,9 +2033,9 @@ DRCTechFinal()
      *		if A is BOTHCORNERS then B must be, too
      */
 
-    for (i = 0; i < DBNumTypes; i++)
+    for (i = 0; i < DBNumUserLayers; i++)
     {
-	for (j = 0; j < DBNumTypes; j++)
+	for (j = 0; j < DBNumUserLayers; j++)
 	{
 	    for (dp = DRCRulesTbl[i][j]; dp != NULL; dp = dp->drcc_next)
 	    {
