@@ -186,7 +186,7 @@ Exttosim_Init(interp)
 {
     /* Sanity checks! */
     if (interp == NULL) return TCL_ERROR;
-    if (Tcl_PkgRequire(interp, "Tclmagic", "7.2", 0) == NULL) return TCL_ERROR;
+    if (Tcl_PkgRequire(interp, "Tclmagic", "7.3", 0) == NULL) return TCL_ERROR;
     if (Tcl_InitStubs(interp, "8.1", 0) == NULL) return TCL_ERROR;
 
     TxPrintf("Auto-loading EXTTOSIM module\n");
@@ -198,7 +198,7 @@ Exttosim_Init(interp)
     if (WindReplaceCommand(DBWclientID, "exttosim", CmdExtToSim) < 0)
 	return TCL_ERROR;
 
-    Tcl_PkgProvide(interp, "Exttosim", "7.2");
+    Tcl_PkgProvide(interp, "Exttosim", "7.3");
     return TCL_OK;
 }
 
@@ -1005,6 +1005,9 @@ simdevVisit(dev, hierName, trans)
 	case DEV_BJT:
 	    fprintf(esSimF, "b");	/* sim format extension */
 	    break;
+	case DEV_DIODE:
+	    fprintf(esSimF, "D");
+	    break;
 	case DEV_RES:
 	    fprintf(esSimF, "r");
 	    break;
@@ -1076,7 +1079,7 @@ simdevVisit(dev, hierName, trans)
     else if (dev->dev_class == DEV_CAP) {	/* generate a capacitor */
        fprintf(esSimF, " %f", (double)(dev->dev_values.dev_cap));
     }
-    else {
+    else if (dev->dev_class != DEV_DIODE) {
 
        /*
         * Scale L and W appropriately by the same amount as distance
