@@ -76,7 +76,7 @@ float esScale = -1.0 ; /* negative if hspice the EFScale/100 otherwise */
 
 static unsigned short esFormat = SPICE3 ;
 
-int esCapNum, esDevNum, esResNum;
+int esCapNum, esDevNum, esResNum, esDiodeNum;
 int esNodeNum; /* just in case we' re extracting spice2 */
 int esSbckNum; /* used in hspice node name shortening   */
 
@@ -242,7 +242,7 @@ Exttospice_Init(interp)
 {
     /* Sanity checks! */
     if (interp == NULL) return TCL_ERROR;
-    if (Tcl_PkgRequire(interp, "Tclmagic", "7.2", 0) == NULL) return TCL_ERROR;
+    if (Tcl_PkgRequire(interp, "Tclmagic", "7.3", 0) == NULL) return TCL_ERROR;
     if (Tcl_InitStubs(interp, "8.1", 0) == NULL) return TCL_ERROR;
 
     TxPrintf("Auto-loading EXTTOSPICE module\n");
@@ -254,7 +254,7 @@ Exttospice_Init(interp)
     if (WindReplaceCommand(DBWclientID, "exttospice", CmdExtToSpice) < 0)
 	return TCL_ERROR;
 
-    Tcl_PkgProvide(interp, "Exttospice", "7.2");
+    Tcl_PkgProvide(interp, "Exttospice", "7.3");
     return TCL_OK;
 }
 
@@ -319,13 +319,15 @@ CmdExtToSpice(w, cmd)
     static char *cmdMergeTypes[] = {
 	"none			don't merge parallel devices",
 	"conservative		merge devices with same L, W",
-	"aggressive		merge devices with same L"
+	"aggressive		merge devices with same L",
+	NULL
     };
 
     static char *cmdExtToSpcFormat[] = {
 	"spice2",
 	"spice3",
-	"hspice"
+	"hspice",
+	NULL
     };
 
     static char *yesno[] = {
@@ -380,7 +382,7 @@ CmdExtToSpice(w, cmd)
 		return TCL_OK;
 	    }
 	    else if (cmd->tx_argc < 3) goto usage;
-	    idx = Lookup(cmd->tx_argv[2], cmdExtToSpcOption);
+	    idx = Lookup(cmd->tx_argv[2], cmdExtToSpcFormat);
 	    if (idx < 0)
 	    {
 		Tcl_SetResult(magicinterp, "Bad format type.  Formats are:"
@@ -506,6 +508,7 @@ runexttospice:
     esCapNum  = 0;
     esDevNum = 1000;
     esResNum = 0;
+    esDiodeNum = 0;
     esSbckNum = 0;
     esNodeNum = 10; /* just in case we're extracting spice2 */
 
@@ -1211,6 +1214,9 @@ spcdevVisit(dev, hierName, trans)
 	case DEV_BJT:
 	    devchar = 'Q';
 	    break;
+	case DEV_DIODE:
+	    devchar = 'D';
+	    break;
 	case DEV_RES:
 	    devchar = 'R';
 	    break;
@@ -1235,6 +1241,9 @@ spcdevVisit(dev, hierName, trans)
 	    case DEV_RES:
 		(void) fprintf(esSpiceF, "%d", esResNum++);
 		break;
+	    case DEV_DIODE:
+		(void) fprintf(esSpiceF, "%d", esDiodeNum++);
+		break;
 	    case DEV_CAP:
 		(void) fprintf(esSpiceF, "%d", esCapNum++);
 		break;
@@ -1341,6 +1350,17 @@ spcdevVisit(dev, hierName, trans)
 	    }
 	    break;
 
+	case DEV_DIODE:
+
+	    /* Diode is "Dnnn top bottom model"	*/
+
+	    spcdevOutNode(hierName, gate->dterm_node->efnode_name->efnn_hier,
+			name, esSpiceF);
+	    spcdevOutNode(hierName, source->dterm_node->efnode_name->efnn_hier,
+			name, esSpiceF);
+	    fprintf(esSpiceF, " %s", EFDevTypes[dev->dev_type]);
+	    break;
+
 	case DEV_CAP:
 
 	    /* Capacitor is "Cnnn top bottom value"	*/
@@ -1545,11 +1565,12 @@ Void spcnAP(node, resClass, scale, sterm, m, outf, w)
 	char fmt[30];
 	float dsc ;
 
-	if ((node == NULL) || (node->efnode_client == NULL))
+	if ((node == NULL) || (node->efnode_client == (ClientData)NULL))
 	{
 	    TxError("spcnAP: major internal inconsistency\n");
 	    return;
 	}
+
 	if ( esFormat == HSPICE )
 	     sprintf(fmt,"a%s=%%g p%s=%%g ", sterm, sterm);
 	else sprintf(fmt,"a%s=%%gp p%s=%%gu ", sterm, sterm);
@@ -2294,7 +2315,7 @@ EFNode *n;
 	nodeClient *nc;
 	int i;
 
-	if ( n->efnode_client == NULL ) 
+	if ( n->efnode_client == (ClientData)NULL ) 
 		initNodeClient(n);
 	nc = (nodeClient *) n->efnode_client;
 	if ( nc->m_w.widths == NULL ) {
@@ -2350,30 +2371,3 @@ Transform *trans;		/* Coordinate transform not used */
 	return 0;
 }
 
-#ifndef MAGIC_WRAPPER
-
-/* debugging stuff */
-#include <varargs.h>
-
-#define DBG 1
-
-DBPRINT( va_alist )
-  va_dcl
-{
-    va_list  args;
-    char     *fmt;
-    FILE     *fp;
-    char     buff[ 300 ];
-
-    if ( DBG ) {
-	va_start( args );
-	fp = stderr ;
-	fmt = va_arg( args, char * );
-	(void) vsprintf( buff, fmt, args );
-	va_end( args );
-
-	(void) fputs( buff, fp );
-    }
-}
-
-#endif
