@@ -19,12 +19,11 @@
 
 #ifndef lint
 static char rcsid[] = "$Header: /ufs/repository/magic/extflat/EFargs.c,v 1.2 2001/01/12 22:12:34 jsolomon Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #include <ctype.h>
 #include <stdlib.h>
-#include <varargs.h>
 #ifdef SYSV
 #include <string.h>
 #endif
@@ -51,10 +50,17 @@ char *EFArgTech = NULL;		/* -T: Tech specified on command line */
 
     /* Misc globals */
 int EFScale = 0;		/* Uninitialized scale factor */
-char *EFVersion = "5.1";	/* Version number of .ext format we read */
+char *EFVersion = "7.3";	/* Version number of .ext format we read */
 char *EFLibPath = NULL;		/* Library search path for .ext files */
 char *EFTech = NULL;
-char *EFStyle = "";		/* Start with no extraction style */
+char *EFStyle = NULL;		/* Start with no extraction style */
+
+#ifdef MAGIC_WRAPPER
+extern char     *Path;		/* magic's search path---note this should  */
+				/* be done with #include "main/main.h" but */
+				/* this is easier.			   */
+#endif
+
 
 /* -------------------- Visible only inside extflat ------------------- */
 
@@ -71,7 +77,9 @@ bool efResistChanged = FALSE;	/* TRUE if .ext resist classes mismatch */
 bool efScaleChanged = FALSE;	/* TRUE if .ext scales mismatch */
 
     /* Forward declarations */
+#ifndef MAGIC_WRAPPER
 int efLoadPathFunc();
+#endif
 
 /*
  * ----------------------------------------------------------------------------
@@ -132,15 +140,18 @@ int efLoadPathFunc();
  * Side effects:
  *	Can set global variables based on the values of command-line
  *	arguments.
+ *	err_result, if non-null, is set to TRUE if an error occurred.
+ *	err_result is only used by the Tcl version of Magic.
  *
  * ----------------------------------------------------------------------------
  */
 
 char *
-EFArgs(argc, argv, argsProc, cdata)
+EFArgs(argc, argv, err_result, argsProc, cdata)
     int argc;		/* Number of command-line args */
     char *argv[];	/* Vector of command-line args */
-    Void (*argsProc)();	/* Called for args we don't recognize */
+    bool *err_result;	/* Set to TRUE if error occurs */
+    bool (*argsProc)();	/* Called for args we don't recognize */
     ClientData cdata;	/* Passed to (*argsProc)() */
 {
     static char libpath[FNSIZE];
@@ -148,6 +159,8 @@ EFArgs(argc, argv, argsProc, cdata)
     HierName *hierName;
     FILE *f;
 
+    if (err_result != NULL) *err_result = FALSE;
+
     /* Hash table of nodes we're going to watch if -N given */
     HashInitClient(&efWatchTable, 32, HT_CLIENTKEYS,
 	efHNCompare, (char *(*)()) NULL,
@@ -207,10 +220,13 @@ EFArgs(argc, argv, argsProc, cdata)
 		    goto usage;
 		efSymAddFile(cp);
 		break;
+
+#ifndef MAGIC_WRAPPER
 	    case 'T':
 		if ((EFArgTech = ArgStr(&argc, &argv, "tech name")) == NULL)
 		    goto usage;
 		break;
+#endif
 
 	    /*** OPTIONS FOR DEBUGGING ***/
 	    case 'n':
@@ -254,14 +270,23 @@ EFArgs(argc, argv, argsProc, cdata)
 	    default:
 		if (argsProc == NULL)
 		    goto usage;
-		(*argsProc)(&argc, &argv, cdata);
+		if ((*argsProc)(&argc, &argv, cdata))
+		{
+		    TxError("\n");
+		    goto usage;
+		}
 		break;
 	}
     }
 
     /* Find the search path if one was not specified */
     if (EFSearchPath == NULL)
+#ifdef MAGIC_WRAPPER
+	/* Set the search path to be the same as magic's search path */
+	EFSearchPath = StrDup(NULL, Path);
+#else
 	efLoadSearchPath(&EFSearchPath);
+#endif
 
     EFLibPath = libpath;
     *EFLibPath = 0; /* start with no path */
@@ -286,19 +311,21 @@ EFArgs(argc, argv, argsProc, cdata)
     return inname;
 
 usage:
-    TxError("Valid argument usage: [-R] [-C] [-T techname] [-r rthresh] "
-		"[-c cthresh] [-v]\n"
+    TxError("Standard arguments: [-R] [-C] [-r rthresh] [-c cthresh] [-v]\n"
 		"[-p searchpath] [-s sym=value] [-S symfile] [-t trimchars]\n"
 
 #ifdef MAGIC_WRAPPER
 		"[rootfile]\n");
+    if (err_result != NULL) *err_result = TRUE;
     return NULL;
 #else
-		"rootfile\n");
+		"[-T techname] rootfile\n");
     exit (1);
     /*NOTREACHED*/
 #endif
 }
+
+#ifndef MAGIC_WRAPPER
 
 /*
  * ----------------------------------------------------------------------------
@@ -361,3 +388,5 @@ efLoadPathFunc(line, ppath)
     *dp = '\0';
     (void) StrDup(ppath, path);
 }
+
+#endif
