@@ -20,7 +20,7 @@
 
 #ifndef lint
 static char rcsid[] = "$Header: /ufs/repository/magic/extflat/EFdef.c,v 1.2 2001/01/12 22:12:35 jsolomon Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #include "misc/magic.h"
@@ -62,7 +62,7 @@ HashTable efFreeHashTable;
 EFInit()
 {
     EFLayerNumNames = 1;
-    EFFetNumTypes = 0;
+    EFDevNumTypes = 0;
 
     HashInit(&efFreeHashTable, 32, HT_WORDKEYS);
     HashInit(&efDefHashTable, INITDEFSIZE, 0);
@@ -95,7 +95,7 @@ EFDone()
     Kill *kill;
     Def *def;
     Use *use;
-    Fet *fet;
+    Dev *dev;
     int n;
 
     HashStartSearch(&hs);
@@ -118,12 +118,12 @@ EFDone()
 	    efFreeConn(conn);
 	for (conn = def->def_resistors; conn; conn = conn->conn_next)
 	    efFreeConn(conn);
-	for (fet = def->def_fets; fet; fet = fet->fet_next)
+	for (dev = def->def_devs; dev; dev = dev->dev_next)
 	{
-	    for (n = 0; n < (int)fet->fet_nterm; n++)
-		if (fet->fet_terms[n].fterm_attrs)
-		    FREE((char *) fet->fet_terms[n].fterm_attrs);
-	    FREE((char *) fet);
+	    for (n = 0; n < (int)dev->dev_nterm; n++)
+		if (dev->dev_terms[n].dterm_attrs)
+		    FREE((char *) dev->dev_terms[n].dterm_attrs);
+	    FREE((char *) dev);
 	}
 	for (kill = def->def_kills; kill; kill = kill->kill_next)
 	{
@@ -134,7 +134,7 @@ EFDone()
     }
 
     /* Misc cleanup */
-    for (n = 0; n < EFFetNumTypes; n++) FREE(EFFetTypes[n]);
+    for (n = 0; n < EFDevNumTypes; n++) FREE(EFDevTypes[n]);
 
     /* Changed from n = 0 to n = 1; First entry "space" is predefined,	*/
     /* not malloc'd.  ---Tim 9/3/02					*/
@@ -223,7 +223,7 @@ efDefNew(name)
     newdef->def_conns = (Connection *) NULL;
     newdef->def_caps = (Connection *) NULL;
     newdef->def_resistors = (Connection *) NULL;
-    newdef->def_fets = (Fet *) NULL;
+    newdef->def_devs = (Dev *) NULL;
     newdef->def_uses = (Use *) NULL;
     newdef->def_kills = (Kill *) NULL;
 
