@@ -19,7 +19,7 @@
 
 #ifndef lint
 static char rcsid[] = "$Header: /ufs/repository/magic/extflat/EFhier.c,v 1.2 2001/01/12 22:12:35 jsolomon Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #include <math.h>
@@ -76,6 +76,9 @@ efHierSrUses(hc, func, cdata)
     Transform t;
     Use *u;
 
+    /* We should not descend into subcircuit "black boxes" */
+    if (hc->hc_use->use_def->def_flags & DEF_SUBCIRCUIT) return 0;
+
     for (u = hc->hc_use->use_def->def_uses; u; u = u->use_next)
     {
 	newhc.hc_use = u;
@@ -190,7 +193,7 @@ efHierSrArray(hc, conn, proc, cdata)
 		j1lo, c1->cn_subs[1].r_hi,
 		i2lo, c2->cn_subs[0].r_hi,
 		j2lo, c2->cn_subs[1].r_hi);
-#endif	notdef
+#endif	/* notdef */
 	    for (i = i1lo; i <= c1->cn_subs[0].r_hi; i++)
 	    {
 		for (j = j1lo; j <= c1->cn_subs[1].r_hi; j++)
