@@ -35,12 +35,12 @@ static char rcsid[] = "$Header: /ufs/repository/magic/extflat/EFread.c,v 1.2 200
 #include "extflat/extflat.h"
 #include "extflat/EFint.h"
 #include "extract/extract.h"
-#include "graphics/graphics.h"	/* for  GrVfprintf */
 #include "misc/paths.h"
 
 #ifndef MAGIC_WRAPPER
 /* This must match the definition for extDevTable in extract/ExtBasic.c */
-char *extDevTable[] = {"fet", "mosfet", "bjt", "devres", "devcap", "subckt", NULL};
+char *extDevTable[] = {"fet", "mosfet", "bjt", "devres", "devcap",
+		"diode", "subckt", NULL};
 #endif
 
 /*
@@ -288,6 +288,9 @@ readfile:
 		    case DEV_CAP:
 			ac = 8;
 			break;
+		    case DEV_DIODE:
+			ac = 7;
+			break;
 		}
 		if (((argc - ac) % 3) != 0)
 		{
@@ -314,11 +317,18 @@ readfile:
 
 	    /* merge node1 node2 C a1 p1 a2 p2 ... */
 	    case MERGE:
-		if ((argc > 4) && (argc - 4 < 2 * efNumResistClasses))
+		/* Redundant merge lines are purposely generated with	*/
+		/* no area and perimeter values;  these should not be	*/
+		/* flagged as errors.					*/
+
+		/*
+		if (argc > 4) && (argc - 4 < 2 * efNumResistClasses))
 		{
 		    efReadError("Too few area/perim values: "
 				"assuming remainder are zero\n");
 		}
+		*/
+
 		cap = (argc > 3) ? atoCap(argv[3]) * cscale : 0;
 		efBuildConnect(def, argv[1], argv[2], (double) cap, &argv[4], argc - 4);
 		break;
