@@ -106,8 +106,12 @@ efVisitSubcircuits(hc, ca)
     /* as subcircuits via the DEF_SUBCIRCUIT flag.	*/
 
     if (hc->hc_use->use_def->def_flags & DEF_SUBCIRCUIT)
+    {
 	if ((*ca->ca_proc)(hc->hc_use, hc->hc_hierName, NULL))
 	    return 1;
+	else
+	    return 0;
+    }
 
     /* Recursively visit subcircuits in our children last. */
 
@@ -251,6 +255,8 @@ efVisitDevs(hc, ca)
     int scale;
     Transform t;
 
+    if (def->def_flags & DEF_SUBCIRCUIT) return 0;
+
     /* Recursively visit devs in our children first */
     if (efHierSrUses(hc, efVisitDevs, (ClientData) ca))
 	return 1;
@@ -465,6 +471,9 @@ efVisitResists(hc, ca)
     Transform t;
     int scale;
 
+    /* Ignore subcircuits */
+    if (def->def_flags & DEF_SUBCIRCUIT) return 0;
+
     /* Recursively visit resistors in our children first */
     if (efHierSrUses(hc, efVisitResists, (ClientData) ca))
 	return 1;
@@ -643,9 +652,9 @@ EFVisitNodes(nodeProc, cdata)
 	    for (nn = node->efnode_name; nn; nn = nn->efnn_next)
 		if (HashLookOnly(&efWatchTable, (char *) nn->efnn_hier))
 		{
-		    printf("Equivalent nodes:\n");
+		    TxPrintf("Equivalent nodes:\n");
 		    for (nn = node->efnode_name; nn; nn = nn->efnn_next)
-			printf("\t%s\n", EFHNToStr(nn->efnn_hier));
+			TxPrintf("\t%s\n", EFHNToStr(nn->efnn_hier));
 		    break;
 		}
 	}
