@@ -53,7 +53,8 @@ static char sccsid[] = "@(#)ExtBasic.c	4.13 MAGIC (Berkeley) 12/5/85";
 /* Also: Note that this table is repeated in extflat/EFread.c when	   */
 /* ext2spice/ext2sim are compiled as separate programs (i.e., non-Tcl)	   */
 
-char *extDevTable[] = {"fet", "mosfet", "bjt", "devres", "devcap", "subckt", NULL};
+char *extDevTable[] = {"fet", "mosfet", "bjt", "devres", "devcap",
+	"diode", "subckt", NULL};
 
 /* --------------------- Data local to this file ---------------------- */
 
@@ -1058,6 +1059,9 @@ extOutputTrans(def, transList, outFile)
 			(subsName == NULL) ? "None" : subsName);
 		break;
 
+	    case DEV_DIODE:	/* Do nothing */
+		break;
+
 	    case DEV_RES:
 		length = extTransRec.tr_perim;
 		width = 0;
