@@ -577,6 +577,7 @@ extAddCouple(bp, ecs)
     Boundary bpCopy;
     Rect r, ovr;
     CellDef *def = ecs->def;
+    extSidewallStruct esws;
 
     /* Revert any contacts to their residues */
     if (DBIsContact(tin))
@@ -640,10 +641,12 @@ extAddCouple(bp, ecs)
 	for (pNum = PL_TECHDEPBASE; pNum < DBNumPlanes; pNum++)
 	    if (PlaneMaskHasPlane(pMask, pNum))
 	    {
-		bp->b_plane = pNum;
+		esws.bp = bp;
+		esws.plane_of_boundary = ecs->plane;
+		esws.plane_checked = pNum;
 		(void) DBSrPaintArea((Tile *) NULL, def->cd_planes[pNum],
 			&ovr, &ExtCurStyle->exts_sideOverlapOtherTypes[tin][tout],
-			extSideOverlap, (ClientData) bp);
+			extSideOverlap, (ClientData) &esws);
 	    }
     }
 
