@@ -21,7 +21,7 @@
 
 #ifndef lint
 static char rcsid[] = "$Header: /ufs/repository/magic/extract/ExtNghbors.c,v 1.3 2001/11/29 20:08:52 tim Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #include "misc/magic.h"
@@ -193,7 +193,7 @@ donesides:
 		goto fail;
 
 	/* If this is a contact, visit all the other planes */
-	if (TTMaskHasType(&DBImageBits, type))
+	if (DBIsContact(type))
 	{
 	    pMask = DBConnPlanes[type];
 	    for (pNum = tilePlaneNum - 1; pNum <= tilePlaneNum + 1; pNum += 2)
