@@ -159,6 +159,9 @@ static keydesc devTable[] = {
     "resistor",		DEV_RES,		4,	4,
 "name|None res-types terminal-types",
 
+    "diode",		DEV_DIODE,		4,	4,
+"name pos-types neg-types",
+
     "subcircuit",	DEV_SUBCKT,		3,	5,
 "name dev-types [term-types [sub-types]]",
 
@@ -170,6 +173,8 @@ static keydesc devTable[] = {
 /*
  * ----------------------------------------------------------------------------
  *
+ * ExtCompareStyle --
+ *
  *	This routine is designed to work with embedded exttosim and
  *	exttospice.  It determines whether the current extract style
  *	matches the string (picked up from the .ext file).  If so, it
@@ -205,6 +210,8 @@ ExtCompareStyle(stylename)
 /*
  * ----------------------------------------------------------------------------
  *
+ * ExtGetDevInfo --
+ *
  *	This routine is designed to work with the embedded exttosim and
  *	exttospice commands under the Tcl-based magic, such that all
  *	device information needed by these commands can be picked up
@@ -1133,6 +1140,11 @@ ExtTechLine(sectionName, argc, argv)
 		    if (argc > 8) gccap = aToCap(argv[8]);
 		    nterm = 2;
 		    break;
+		case DEV_DIODE:
+		    DBTechNoisyNameMask(argv[4], &types2);	/* negative types */
+		    subsTypes = DBZeroTypeBits;
+		    nterm = 1;
+		    break;
 		case DEV_RES:
 		    DBTechNoisyNameMask(argv[4], &types2);	/* terminals */
 		    subsTypes = DBZeroTypeBits;
@@ -1719,7 +1731,7 @@ ExtTechScale(scalen, scaled)
 	    style->exts_perimCap[i][j] *= scalen;
 	    style->exts_perimCap[i][j] /= scaled;
 	    style->exts_overlapCap[i][j] *= sqn;
-	    style->exts_overlapCap[i][j] *= sqd;
+	    style->exts_overlapCap[i][j] /= sqd;    /* Typo fixed in 7.2.57 */
 	    for (ec = style->exts_sideCoupleCap[i][j]; ec != NULL;
 				ec = ec->ec_next)
 	    {
