@@ -32,16 +32,21 @@
 
 extern int ExtDoWarn;		/* Bitmask of above */
 
-/* Known devices (see ExtTech.c and ExtBasic.c) */
+/* Known devices (see ExtTech.c and ExtBasic.c)			*/
+/* Make sure these match extDevTable in extract/ExtBasic.c and	*/
+/* also extflat/EFread.c					*/
+
 #define DEV_FET		0		/* FET w/area, perimeter declared */
 #define DEV_MOSFET      1		/* FET w/length, width declared   */
 #define DEV_BJT         2		/* Bipolar Junction Transistor */
 #define DEV_RES         3		/* Resistor */
 #define DEV_CAP         4		/* Capacitor */
-#define DEV_SUBCKT      5		/* Anything else... becomes an "X" */
+#define DEV_DIODE	5		/* Diode */
+#define DEV_SUBCKT      6		/* Anything else... becomes an "X" */
 					/* record in SPICE.		   */
 
-/* Device names for .ext file output (new in version 7.2) */
+/* Device names for .ext file output (new in version 7.2)	*/
+/* (defined in extract/ExtBasic.c *and* extflat/EFread.c)	*/
 
 extern char *extDevTable[];
 
