@@ -505,6 +505,39 @@ deleteit: {
     }
 }
 
+/*
+ *---------------------------------------------------------
+ * grAddSegment:
+ *	Add a segment to a linked list of rectangles
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	Allocates memory;  appends LinkedRect structure to
+ *	list "segments".
+ *
+ * Notes:
+ *	INLINE this for speedup?
+ *---------------------------------------------------------
+ */
+
+void
+grAddSegment(llx, lly, urx, ury, segments)
+    int llx, lly, urx, ury;
+    LinkedRect **segments;
+{
+    LinkedRect *curseg;
+
+    curseg = (LinkedRect *)mallocMagic(sizeof(LinkedRect));
+    curseg->r_r.r_xbot = llx;
+    curseg->r_r.r_ybot = lly;
+    curseg->r_r.r_xtop = urx;
+    curseg->r_r.r_ytop = ury;
+    curseg->r_next = *segments;
+    *segments = curseg;
+}
+
 /*---------------------------------------------------------
  * GrBoxOutline:
  *	For box outlines, works around the boundary of
@@ -544,187 +577,181 @@ GrBoxOutline(tile, tilesegs)
     Tile *tpleft, *tpright, *tptop, *tpbot;
     int edgeTop, edgeBot, edgeRight, edgeLeft;
     int isolate = 0;
+    bool sense;
 
     *tilesegs = NULL;
     TiToRect(tile, &rect);
 
 #ifdef NONMANHATTAN
-    if (!IsSplit(tile) || !SplitSide(tile))
+    if (IsSplit(tile) && SplitSide(tile))
+	isolate |= 0x1;
+    else
     {
 #endif
     ttype = TiGetLeftType(tile);   /* = TiGetType when NONMANHATTAN not def'd */
+
+    edgeBot = rect.r_ybot;
+    sense = TRUE;
     for (tpleft = BL(tile); BOTTOM(tpleft) < rect.r_ytop; tpleft = RT(tpleft))
     {
-	if (TiGetRightType(tpleft) == ttype)
-	    continue;
-	edgeTop = MIN(TOP(tpleft), rect.r_ytop);
-	edgeBot = MAX(BOTTOM(tpleft), rect.r_ybot);
-	if (edgeTop <= edgeBot)
-	    continue;
-
-	if ((edgeTop == rect.r_ytop) && (edgeBot == rect.r_ybot))
+	if (TiGetRightType(tpleft) != ttype)
 	{
-	    isolate = 0x1;
-	    break;
+	    if (!sense)
+	    {
+		edgeBot = BOTTOM(tpleft);
+		if (TOP(tpleft) >= rect.r_ytop)
+		    grAddSegment(rect.r_xbot, edgeBot, rect.r_xbot, rect.r_ytop,
+				tilesegs);
+		sense = TRUE;
+	    }
 	}
 	else
 	{
-	    curseg = (LinkedRect *)mallocMagic(sizeof(LinkedRect));
-	    curseg->r_r.r_xbot = rect.r_xbot;
-	    curseg->r_r.r_xtop = rect.r_xbot;
-	    curseg->r_r.r_ybot = edgeBot;
-	    curseg->r_r.r_ytop = edgeTop;
-	    curseg->r_next = *tilesegs;
-	    *tilesegs = curseg;
+	    if (sense)
+	    {
+		edgeTop = BOTTOM(tpleft);
+		if (edgeTop > edgeBot)
+		    grAddSegment(rect.r_xbot, edgeBot, rect.r_xbot, edgeTop,
+				tilesegs);
+		isolate |= 0x1;
+		sense = FALSE;
+	    }
 	}
     }
 
 #ifdef NONMANHATTAN
     }
-    if (!IsSplit(tile) || SplitSide(tile))
+    if (IsSplit(tile) && !SplitSide(tile))
+	isolate |= 0x2;
+    else
     {
 	ttype = TiGetRightType(tile);
 #endif
+    edgeTop = rect.r_ytop;
+    sense = TRUE;
     for (tpright = TR(tile); TOP(tpright) > rect.r_ybot; tpright = LB(tpright))
     {
-	if (TiGetLeftType(tpright) == ttype)
-	    continue;
-	edgeTop = MIN(TOP(tpright), rect.r_ytop);
-	edgeBot = MAX(BOTTOM(tpright), rect.r_ybot);
-	if (edgeTop <= edgeBot)
-	    continue;
-
-	if ((edgeTop == rect.r_ytop) && (edgeBot == rect.r_ybot))
+	if (TiGetLeftType(tpright) != ttype)
 	{
-	    isolate |= 0x2;
-	    break;
+	    if (!sense)
+	    {
+		edgeTop = TOP(tpright);
+		if (BOTTOM(tpright) <= rect.r_ybot)
+		    grAddSegment(rect.r_xtop, rect.r_ybot, rect.r_xtop, edgeTop,
+				tilesegs);
+		sense = TRUE;
+	    }
 	}
 	else
 	{
-	    curseg = (LinkedRect *)mallocMagic(sizeof(LinkedRect));
-	    curseg->r_r.r_xbot = rect.r_xtop;
-	    curseg->r_r.r_xtop = rect.r_xtop;
-	    curseg->r_r.r_ybot = edgeBot;
-	    curseg->r_r.r_ytop = edgeTop;
-	    curseg->r_next = *tilesegs;
-	    *tilesegs = curseg;
+	    if (sense)
+	    {
+		edgeBot = TOP(tpright);
+		if (edgeBot < edgeTop)
+		    grAddSegment(rect.r_xtop, edgeBot, rect.r_xtop, edgeTop,
+				tilesegs);
+		isolate |= 0x2;
+		sense = FALSE;
+	    }
 	}
     }
 
 #ifdef NONMANHATTAN
     }
-    if (!IsSplit(tile) ||
-		(SplitSide(tile) != SplitDirection(tile)))
+    if (IsSplit(tile) &&
+		(SplitSide(tile) == SplitDirection(tile)))
+	isolate |= 0x4;
+    else
     {
 	ttype = TiGetBottomType(tile);
 #endif
+    edgeLeft = rect.r_xbot;
+    sense = TRUE;
     for (tpbot = LB(tile); LEFT(tpbot) < rect.r_xtop; tpbot = TR(tpbot))
     {
-	if (TiGetTopType(tpbot) == ttype)
-	    continue;
-	edgeLeft = MAX(LEFT(tpbot), rect.r_xbot);
-	edgeRight = MIN(RIGHT(tpbot), rect.r_xtop);
-	if (edgeLeft >= edgeRight)
-	    continue;
-
-	if ((edgeLeft == rect.r_xbot) && (edgeRight == rect.r_xtop))
+	if (TiGetTopType(tpbot) != ttype)
 	{
-	    isolate |= 0x4;
-	    break;
+	    if (!sense)
+	    {
+		edgeLeft = LEFT(tpbot);
+		if (RIGHT(tpbot) >= rect.r_xtop)
+		    grAddSegment(edgeLeft, rect.r_ybot, rect.r_xtop, rect.r_ybot,
+				tilesegs);
+		sense = TRUE;
+	    }
 	}
 	else
 	{
-	    curseg = (LinkedRect *)mallocMagic(sizeof(LinkedRect));
-	    curseg->r_r.r_xbot = edgeLeft;
-	    curseg->r_r.r_xtop = edgeRight;
-	    curseg->r_r.r_ybot = rect.r_ybot;
-	    curseg->r_r.r_ytop = rect.r_ybot;
-	    curseg->r_next = *tilesegs;
-	    *tilesegs = curseg;
+	    if (sense)
+	    {
+		edgeRight = LEFT(tpbot);
+		if (edgeRight > edgeLeft)
+		    grAddSegment(edgeLeft, rect.r_ybot, edgeRight, rect.r_ybot,
+				tilesegs);
+		isolate |= 0x4;
+		sense = FALSE;
+	    }
 	}
     }
 
 #ifdef NONMANHATTAN
     }
-    if (!IsSplit(tile) ||
-		(SplitSide(tile) == SplitDirection(tile)))
+    if (IsSplit(tile) &&
+		(SplitSide(tile) != SplitDirection(tile)))
+	isolate |= 0x8;
+    else
     {
 	ttype = TiGetTopType(tile);
 #endif
+    edgeRight = rect.r_xtop;
+    sense = TRUE;
     for (tptop = RT(tile); RIGHT(tptop) > rect.r_xbot; tptop = BL(tptop))
     {
-	if (TiGetBottomType(tptop) == ttype)
-	    continue;
-	edgeLeft = MAX(LEFT(tptop), rect.r_xbot);
-	edgeRight = MIN(RIGHT(tptop), rect.r_xtop);
-	if (edgeLeft >= edgeRight)
-	    continue;
-
-	if ((edgeLeft == rect.r_xbot) && (edgeRight == rect.r_xtop))
+	if (TiGetBottomType(tptop) != ttype)
 	{
-	    isolate |= 0x8;
-	    break;
+	    if (!sense)
+	    {
+		edgeRight = RIGHT(tptop);
+		if (LEFT(tptop) <= rect.r_xbot)
+		    grAddSegment(rect.r_xbot, rect.r_ytop, edgeRight, rect.r_ytop,
+				tilesegs);
+		sense = TRUE;
+	    }
 	}
 	else
 	{
-	    curseg = (LinkedRect *)mallocMagic(sizeof(LinkedRect));
-	    curseg->r_r.r_xbot = edgeLeft;
-	    curseg->r_r.r_xtop = edgeRight;
-	    curseg->r_r.r_ybot = rect.r_ytop;
-	    curseg->r_r.r_ytop = rect.r_ytop;
-	    curseg->r_next = *tilesegs;
-	    *tilesegs = curseg;
+	    if (sense)
+	    {
+		edgeLeft = RIGHT(tptop);
+		if (edgeLeft < edgeRight)
+		    grAddSegment(edgeLeft, rect.r_ytop, edgeRight, rect.r_ytop,
+				tilesegs);
+		isolate |= 0x8;
+		sense = FALSE;
+	    }
 	}
     }
 #ifdef NONMANHATTAN
     }
 #endif
 
-    if (isolate == 0x0f)	/* Common case */
+    if (isolate == 0)	/* Common case */
 	return TRUE;
     else
     {
 	/* Need to malloc segments for isolated sides */
-	if (isolate & 0x1)	/* Left */
-	{
-	    curseg = (LinkedRect *)mallocMagic(sizeof(LinkedRect));
-	    curseg->r_r.r_xbot = rect.r_xbot;
-	    curseg->r_r.r_xtop = rect.r_xbot;
-	    curseg->r_r.r_ybot = rect.r_ybot;
-	    curseg->r_r.r_ytop = rect.r_ytop;
-	    curseg->r_next = *tilesegs;
-	    *tilesegs = curseg;
-	}
-	if (isolate & 0x2)	/* Right */
-	{
-	    curseg = (LinkedRect *)mallocMagic(sizeof(LinkedRect));
-	    curseg->r_r.r_xbot = rect.r_xtop;
-	    curseg->r_r.r_xtop = rect.r_xtop;
-	    curseg->r_r.r_ybot = rect.r_ybot;
-	    curseg->r_r.r_ytop = rect.r_ytop;
-	    curseg->r_next = *tilesegs;
-	    *tilesegs = curseg;
-	}
-	if (isolate & 0x4)	/* Bottom */
-	{
-	    curseg = (LinkedRect *)mallocMagic(sizeof(LinkedRect));
-	    curseg->r_r.r_xbot = rect.r_xbot;
-	    curseg->r_r.r_xtop = rect.r_xtop;
-	    curseg->r_r.r_ybot = rect.r_ybot;
-	    curseg->r_r.r_ytop = rect.r_ybot;
-	    curseg->r_next = *tilesegs;
-	    *tilesegs = curseg;
-	}
-	if (isolate & 0x8)	/* Top */
-	{
-	    curseg = (LinkedRect *)mallocMagic(sizeof(LinkedRect));
-	    curseg->r_r.r_xbot = rect.r_xbot;
-	    curseg->r_r.r_xtop = rect.r_xtop;
-	    curseg->r_r.r_ybot = rect.r_ytop;
-	    curseg->r_r.r_ytop = rect.r_ytop;
-	    curseg->r_next = *tilesegs;
-	    *tilesegs = curseg;
-	}
+	if (!(isolate & 0x1))	/* Left */
+	    grAddSegment(rect.r_xbot, rect.r_ybot, rect.r_xbot, rect.r_ytop,
+			tilesegs);
+	if (!(isolate & 0x2))	/* Right */
+	    grAddSegment(rect.r_xtop, rect.r_ybot, rect.r_xtop, rect.r_ytop,
+			tilesegs);
+	if (!(isolate & 0x4))	/* Bottom */
+	    grAddSegment(rect.r_xbot, rect.r_ybot, rect.r_xtop, rect.r_ybot,
+			tilesegs);
+	if (!(isolate & 0x8))	/* Top */
+	    grAddSegment(rect.r_xbot, rect.r_ytop, rect.r_xtop, rect.r_ytop,
+			tilesegs);
 	return FALSE;
     }
 }
