@@ -13,11 +13,14 @@
  *     * express or implied warranty.  Export of this software outside     * 
  *     * of the United States of America may require an export license.    * 
  *     *********************************************************************
+ *
+ * Portions of this code are Copyright (C) 2003 Open Circuit Design, Inc.,
+ * for MultiGiG Ltd.
  */
 
 #ifndef lint
 static char rcsid[]="$Header: /ufs/repository/magic/graphics/grDStyle.c,v 1.2 2001/01/12 22:12:45 jsolomon Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #include <ctype.h>
@@ -80,6 +83,7 @@ char *fillStyles[] = {
 #define	LAYOUT_STYLES	2
 #define	PALE_STYLES	4
 #define	STIPPLES	8
+#define DISP_VERSION	16
 
 #define	STRLEN	200
 
@@ -90,6 +94,37 @@ GR_STYLE_LINE *GrStyleTable;
 
 
 
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * GrGetStyleFromName --
+ *
+ * Return an integer style number given a "long name".
+ *
+ * Results:
+ *	Integer style number, or -1 if no style matches the name.
+ *
+ * Side effects:
+ *	None.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+int
+GrGetStyleFromName(stylename)
+    char *stylename;
+{
+    int style;
+    int maxstyles = TECHBEGINSTYLES + (DBWNumStyles * 2);
+
+    for (style = 0; style < maxstyles; style++)
+        if (GrStyleTable[style].longname != NULL)
+            if (!strcmp(stylename, GrStyleTable[style].longname))
+                break;                          
+
+    return (style == maxstyles) ? -1 : style;
+}
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -138,8 +173,9 @@ GrResetStyles()
  */
 
 int
-styleBuildDisplayStyle(line)
+styleBuildDisplayStyle(line, version)
     char *line;
+    int version;
 {
     int res, argsread;
     int ord = 1, mask, color, outline, nfill, stipple;
@@ -147,9 +183,14 @@ styleBuildDisplayStyle(line)
     char fill[42], ordstr[12];
     dstylelink *newstyle, *sstyle;
 
+    char v6scanline[] = "%10s %o %o %o %40s %d %c %50s";
+    char v7scanline[] = "%10s %i %i %i %40s %d %c %50s";
+
+    char *scanline = (version > 6) ? v7scanline : v6scanline;
+
     res = TRUE;
 
-    if ((argsread = sscanf(line, "%10s %o %o %o %40s %d %c %50s", 
+    if ((argsread = sscanf(line, scanline,
 		&ordstr, &mask, &color, &outline, fill, &stipple,
 		&shortName, &longName)) < 7)
     {
@@ -209,16 +250,22 @@ styleBuildDisplayStyle(line)
  */
 
 int
-styleBuildStipplesStyle(line)
-char *line;
+styleBuildStipplesStyle(line, version)
+    char *line;
+    int version;
 {
     int res;
     int ord;
     int row[8];
 
+    char v6scanline[] = "%d %o %o %o %o %o %o %o %o";
+    char v7scanline[] = "%d %x %x %x %x %x %x %x %x";
+
+    char *scanline = (version > 6) ? v7scanline : v6scanline;
+
     res = TRUE;
 
-    if (sscanf(line, "%d %o %o %o %o %o %o %o %o",
+    if (sscanf(line, scanline,
 	    &ord, &(row[0]), &(row[1]), &(row[2]), &(row[3]), 
 	    &(row[4]), &(row[5]), &(row[6]), &(row[7]) ) != 9) 
     {
@@ -322,23 +369,34 @@ char *libPath;
 {
     FILE *inp;
     int res = 0;
-    int i, scount, processed = 0;
+    int i, scount, processed = DISP_VERSION;
     char fullName[256];
     dstylelink *sstyle;
     int MaxTechStyles = 0, MaxTileStyles = 0;
 
+    /* The dstyle file format version number was buried in the filename */
+    /* prior to version 6.  Therefore versions which do not have a	*/
+    /* "version" keyword in the file should default to version 5.	*/
+    int version = 5;
+
     /* Reset number of styles to zero */
     GrResetStyles();
 
     for (i = 0; i < 128; i++) GrStyleNames[i] = 0;
 
-    (void) sprintf(fullName, "%.100s.%.100s.dstyle6", techType, grDStyleType);
+    (void) sprintf(fullName, "%.100s.%.100s.dstyle", techType, grDStyleType);
 
     inp = PaOpen(fullName, "r", (char *) NULL, path, libPath, (char **) NULL);
     if (inp == NULL)
     {
-	TxError("Couldn't open display styles file \"%s\"\n", fullName);
-        return(-2);
+	/* Try old format ".dstyle5"? */
+        (void) sprintf(fullName, "%.100s.%.100s.dstyle5", techType, grDStyleType);
+	inp = PaOpen(fullName, "r", (char *) NULL, path, libPath, (char **) NULL);
+	if (inp == NULL)
+	{
+	    TxError("Couldn't open display styles file \"%s\"\n", fullName);
+            return(-2);
+	}
     }
     else
     {
@@ -361,11 +419,21 @@ char *libPath;
 	    {
 		if (sscanf(line, "%s", sectionName) != 1)
 		{
-		    TxError("File contained format error: %s\n", 
-			    "unable to read section name.");
+		    TxError("File contained format error: " 
+			    "unable to read section name.\n");
 		    res = -1;
 		}
-		if (strcmp(sectionName, "display_styles") == 0)
+		if (strcmp(sectionName, "version") == 0)
+		{
+		    if (sscanf(line, "%*s %d", &version) != 1)
+		    {
+		        TxError("DStyle format version could not be "
+				"read: assuming version 6\n");
+			version = 6;
+		    }
+		    section = DISP_VERSION;
+		}
+		else if (strcmp(sectionName, "display_styles") == 0)
 		{
 		    int locbitplanes;
 
@@ -427,12 +495,13 @@ char *libPath;
 		    case LAYOUT_STYLES:
 		    case PALE_STYLES:
 		    case DISP_STYLES:
-			newres = styleBuildDisplayStyle(line);
+			newres = styleBuildDisplayStyle(line, version);
 			scount++;
 			break;
 		    case STIPPLES:
-			newres = styleBuildStipplesStyle(line);
+			newres = styleBuildStipplesStyle(line, version);
 			break;
+		    case DISP_VERSION:
 		    case IGNORE:
 			break;
 		    default:
@@ -451,7 +520,7 @@ char *libPath;
 	TxError("Could not close styles file.\n");
 
     if ((processed | STIPPLES) != (LAYOUT_STYLES | DISP_STYLES
-		| PALE_STYLES | STIPPLES))
+		| PALE_STYLES | STIPPLES | DISP_VERSION))
     {
 	TxError("Not all required style sections were read.  Missing"
 		" sections are:");
@@ -461,6 +530,8 @@ char *libPath;
 	    TxError(" layout_styles");
 	if (!(processed & PALE_STYLES))
 	    TxError(" pale_styles");
+	if (!(processed & DISP_VERSION))
+	    TxError(" version");
 	TxError("\n");
 	res = -1;
     }
