@@ -112,6 +112,14 @@ global int GrPixelCorrect = 1;
  */
 global Rect GrScreenRect = {0, 0, 0, 0};
 
+/*
+ * Interrupt status for the timer.  In TCL, this is used for graphics
+ * interrupts to periodically check the X11 queue for pending events.
+ * In both TCL and non-TCL versions, it can be used for a general-
+ * purpose interrupt timer.
+ */
+global unsigned char GrDisplayStatus = DISPLAY_IDLE;
+
 /* The first of the following tables defines the legal
  * display types and the second table defines an
  * initialization routine for each type.
@@ -120,10 +128,6 @@ global Rect GrScreenRect = {0, 0, 0, 0};
  * be converted to upper case before comparison.
  */
 
-#ifdef MAGIC_WRAPPER
-global unsigned char GrDisplayStatus = DISPLAY_IDLE;
-#endif
-
 static char *grDisplayTypes[] = {
 #ifdef 	SUNVIEW
     "SUN60",
@@ -251,18 +255,6 @@ int grMaxStipples = GR_NUM_STIPPLES;
 int grNumBitPlanes = 0;      /* Number of bit-planes we are using. */
 int grBitPlaneMask = 0;      /* Mask of the valid bit-plane bits. */
 
-/* Procedure to print text on stdout and stderr.  It is located here so that
- * graphics drivers and do something special with it if they want.
- * This routine should only be called by the textio module.
- */
-#ifdef MAGIC_WRAPPER
-extern int Tcl_printf();
-int (*GrVfprintfPtr)() = Tcl_printf;
-#else
-extern int vfprintf();
-Void (*GrVfprintfPtr)() = vfprintf;
-#endif	/* MAGIC_WRAPPER */
-
 /* Procedure to change the default terminal state. It gets passed 2 pointers,
  * one to the terminal input mode structure, and one to the terminal literal
  * characters structure.  See textio module for details.
