@@ -1080,7 +1080,7 @@ GrX11Lock(w, flag)
     bool flag;
 {
     grSimpleLock(w, flag);
-    if ( w != (MagWindow *) GR_LOCK_SCREEN )
+    if ( w != GR_LOCK_SCREEN )
     {
 	grCurrent.mw = w;
 	grCurrent.window = (Window) w->w_grdata;
