@@ -21,7 +21,7 @@
 
 #ifndef lint
 static char rcsid[]="$Header: /ufs/repository/magic/graphics/grSun1.c,v 1.3 2001/01/12 22:12:46 jsolomon Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #ifdef 	sun
 
@@ -461,7 +461,7 @@ sunStdin(fd, cdata)
     event->txe_buttonAction = 0;
     event->txe_ch = ch;
     event->txe_wid = WIND_UNKNOWN_WINDOW;
-    if (!sunGetCursorPos(&(event->txe_p))) {
+    if (!sunGetCursorPos(NULL, &(event->txe_p))) {
         TxError("Could not read cursor position from Sun, using (%d, %d) instead.\n",
 	GR_CURSOR_X, GR_CURSOR_Y);
 	event->txe_p.p_x = GR_CURSOR_X;
@@ -697,4 +697,4 @@ sunSetDisplay(type, colorScreenName, mouseName)
     return TRUE;
 }
 
-#endif	sun
+#endif	/* sun */
