@@ -1181,7 +1181,7 @@ GrTOGLLock(w, flag)
     Window wind;
 
     grSimpleLock(w, flag);
-    if ( w != (MagWindow *) GR_LOCK_SCREEN )
+    if ( w != GR_LOCK_SCREEN )
     {
 	toglCurrent.mw = w;
 
@@ -1223,7 +1223,7 @@ GrTOGLUnlock(w)
     /* GR_TOGL_FLUSH_BATCH(); */
     GrTOGLFlush();	/* (?) Adds glFlush and glFinish to the above. */
 
-    if (w != (MagWindow *) GR_LOCK_SCREEN)
+    if (w != GR_LOCK_SCREEN)
 	if (w->w_flags & WIND_OFFSCREEN)
 	    glXDestroyGLXPixmap(grXdpy, glpmap);
 
