@@ -627,7 +627,7 @@ GrTkInit ()
             break;
         default:
             grDStyleType = "24bit";
-            grCMapType = "7bit";
+            grCMapType = "24bit";
             break;
     }
 
@@ -1523,7 +1523,7 @@ GrTkLock(w, flag)
 {
 
     grSimpleLock(w, flag);
-    if ( w != (MagWindow *) GR_LOCK_SCREEN )
+    if ( w != GR_LOCK_SCREEN )
     {
 	grCurrent.mw = w;
 	if (w->w_flags & WIND_OFFSCREEN)
