@@ -2,6 +2,7 @@
  *
  *	Functions common to all graphics routines running under Tcl/Tk
  *
+ * Copyright 2003 Open Circuit Design, Inc., for MultiGiG Ltd.
  */
 
 /* We should be here if this is not set! */
@@ -22,6 +23,7 @@
 #include "misc/styles.h"
 #include "utils/geometry.h"
 #include "utils/hash.h"
+#include "utils/malloc.h"
 #include "tiles/tile.h"
 #include "database/database.h"
 #include "windows/windows.h"
@@ -30,18 +32,208 @@
 #include "graphics/graphics.h"
 #include "graphics/graphicsInt.h"
 #include "dbwind/dbwind.h"
+#include "graphics/grTkCommon.h"
+#include "graphics/glyphs.h"
+
+/* Variables declared by grClip.c */
 
-extern Display *grXdpy;
-extern Colormap grXcmap;
-extern int grXscrn;
-extern unsigned int grClass;
 extern int grCurFill, grCurOutline, grCurColor;
-extern GC grGCFill, grGCDraw;
-
 extern void grInformDriver();
 
+/* Global variables used by both the Tk and TOGL interfaces	*/
+/* should be defined here.					*/
+
+Display *grXdpy;
+int grXscrn;
+Tk_Cursor grCursors[MAX_CURSORS];
+Tk_Font grTkFonts[4];
+
+/* Used by Tk interface.  Defined for both Tk and TOGL, but 	*/
+/* unused by the TOGL interface.				*/
+
 bool GrTkInstalledCMap = FALSE;
 
+/*---------------------------------------------------------
+ * grTkLoadFont
+ *	This local routine loads the Tk fonts used by Magic.
+ *
+ * Results:	None.
+ *
+ * Side Effects:    None.
+ *---------------------------------------------------------
+ */
+
+Void
+grTkLoadFont()
+{
+    Tk_Window tkwind;
+    int i;
+    char *s;
+    char *unable = "Unable to load font";
+
+    static char *fontnames[4] = {
+      TK_FONT_SMALL,
+      TK_FONT_MEDIUM,
+      TK_FONT_LARGE,
+      TK_FONT_XLARGE };
+    static char *optionnames[4] = {
+      "small",
+      "medium",
+      "large",
+      "xlarge"};
+
+    tkwind = Tk_MainWindow(magicinterp);
+    for (i = 0; i < 4; i++)
+    {
+    	s = XGetDefault(grXdpy, "magic", optionnames[i]);
+	if (s) fontnames[i] = s;
+        if ((grTkFonts[i] = Tk_GetFont(magicinterp,
+			tkwind, fontnames[i])) == NULL) 
+        {
+	    TxError("%s %s\n", unable, fontnames[i]);
+            if ((grTkFonts[i] = Tk_GetFont(magicinterp,
+			tkwind, TK_DEFAULT_FONT)) == NULL) 
+	    {
+	        TxError("%s %s\n", unable, TK_DEFAULT_FONT);
+		return FALSE;
+	    }
+        }
+    }
+    return TRUE;
+}
+
+/*---------------------------------------------------------
+ * grTkFreeFonts
+ *      This local routine frees the Tk font cache.
+ *---------------------------------------------------------
+ */
+
+void
+grTkFreeFonts()
+{
+    int i;
+
+    for (i = 0; i < 4; i++)
+        Tk_FreeFont(grTkFonts[i]);
+}
+
+/*
+ * ----------------------------------------------------------------------------
+ * grTkDefineCursor:
+ *
+ *	Define a new set of cursors.  Use Tk-style cursors (portable)
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	The given matrix is stored in the graphics display, and it can be
+ *	used as the cursor by calling GrSetCursor.
+ * ----------------------------------------------------------------------------
+ */
+
+Void
+grTkDefineCursor(glyphs)
+    GrGlyphs *glyphs;
+{
+    int glyphnum;
+    Rect oldClip;
+    int red, green, blue;
+    bool isblack;
+
+    if (glyphs->gr_num <= 0) return;
+
+    if (glyphs->gr_num > MAX_CURSORS)
+    {
+	TxError("X only has room for %d cursors\n", MAX_CURSORS);
+	return;
+    }
+
+    /* expand clipping amount for off-screen access on the X */
+    GrLock(GR_LOCK_SCREEN, FALSE);
+    oldClip = grCurClip;
+    grCurClip = GrScreenRect;
+    grCurClip.r_ytop += 16;
+    
+    /* what color should the cursor be?  The following makes it the
+       opposite of what the background "mostly" is.
+    */
+    GrGetColor(GrStyleTable[STYLE_TRANSPARENT].color, &red, &green, &blue);
+    isblack = (red + green + blue > 0x180);	/* (0x180 = 128 * 3) */
+
+    /* enter the glyphs */
+    for (glyphnum = 0; glyphnum < glyphs->gr_num; glyphnum++) {
+	int *p;
+	GrGlyph *g;
+	int x, y;
+	unsigned char *curs;
+
+	g = glyphs->gr_glyph[glyphnum];
+	if ((g->gr_xsize != 16) || (g->gr_ysize != 16)) {
+	    TxError("Tk/OpenGL Cursors must be 16 X 16 pixels.\n");
+	    return;
+	}
+	
+	curs = (unsigned char *)mallocMagic(32 * sizeof(unsigned char));
+	g->gr_cache = (ClientData)curs;
+	g->gr_free = freeMagic;
+
+	/* Perform transposition on the glyph matrix since X displays
+	 * the least significant bit on the left hand side.
+	 */
+	p = &(g->gr_pixels[0]);
+	for (y = 0; y < 32; y += 2) {
+	    int i;
+
+	    curs[i = 31 - y - 1] = 0;
+	    for (x = 0; x < 8; x++) 
+	    {
+		if (GrStyleTable[*p].color != 0)
+		{
+		     curs[i] |= 1 << x; 
+		}
+		p++;
+	    }
+	    curs[i += 1] = 0;
+	    for (x = 0; x < 8; x++) 
+	    {
+		if (GrStyleTable[*p].color != 0)
+		{
+		     curs[i] |= 1 << x; 
+		}
+		p++;
+	    }
+	}
+	grCursors[glyphnum] = Tk_GetCursorFromData(magicinterp,
+			Tk_MainWindow(magicinterp),
+			(char *)curs, (char *)curs,
+			16, 16, g->gr_origin.p_x, (15 - g->gr_origin.p_y),
+			((isblack) ? "black" : "white"),
+			((isblack) ? "white" : "black"));
+    }
+
+    /* Restore clipping */
+    grCurClip = oldClip;
+    GrUnlock(GR_LOCK_SCREEN);
+}
+
+/*
+ *-------------------------------------------------------------------------
+ * Get the window name from the indicated MagWindow structure
+ *-------------------------------------------------------------------------
+ */ 
+  
+char *
+GrTkWindowName(mw)
+    MagWindow *mw;
+{    
+    Tk_Window tkwind;
+    char *tkname;
+ 
+    tkwind = (Tk_Window) mw->w_grdata;
+    return Tk_PathName(tkwind);
+}  
+
 /*
  *---------------------------------------------------------
  * GrTkGetColorByName --
@@ -76,7 +268,7 @@ GrTkGetColorByName(name)
     char *name;
 {
     Tk_Window tkwind = Tk_MainWindow(magicinterp);
-    int style;
+    int style, red, green, blue;
     XColor falsecolor;
     char *colstring;
 
@@ -104,8 +296,13 @@ GrTkGetColorByName(name)
     }
     else
     {
-	GrGetColor(falsecolor.pixel, &falsecolor.red,
-			&falsecolor.green, &falsecolor.blue);
+	/* Note that XColor colors are unsigned short, but GrGetColor	*/
+	/* expects an int *.						*/
+
+	GrGetColor(falsecolor.pixel, &red, &green, &blue);
+	falsecolor.red = red;
+	falsecolor.green = green;
+	falsecolor.blue = blue;
         colstring = Tcl_Alloc(8);
         sprintf(colstring, "#%02x%02x%02x", falsecolor.red,
 			falsecolor.green, falsecolor.blue);
@@ -511,7 +708,7 @@ ImgLayerConfigureInstance(instancePtr)
 
 	    pixwind = (Tk_Window)mw->w_grdata;
 
-	    instancePtr->pixmap = XCreatePixmap(grXdpy,
+	    instancePtr->pixmap = Tk_GetPixmap(grXdpy,
 			Tk_WindowId(pixwind),
 			masterPtr->width, masterPtr->height,
 			Tk_Depth(pixwind));
@@ -531,7 +728,7 @@ ImgLayerConfigureInstance(instancePtr)
 	    return;
 	}
 
-	instancePtr->pixmap = XCreatePixmap(grXdpy,
+	instancePtr->pixmap = Tk_GetPixmap(grXdpy,
 		Tk_WindowId(tkwind),
 		masterPtr->width, masterPtr->height,
 		Tk_Depth(tkwind));
