@@ -665,7 +665,7 @@ GrX11Init ()
 	    break;
 	default:
 	    grDStyleType = "24bit";
-	    grCMapType = "7bit";
+	    grCMapType = "24bit";
 	    break;
     }
 
@@ -1305,7 +1305,7 @@ GrX11Lock(w, flag)
     bool flag;
 {
     grSimpleLock(w, flag);
-    if ( w != (MagWindow *) GR_LOCK_SCREEN )
+    if ( w != GR_LOCK_SCREEN )
     {
 	grCurrent.mw = w;
 	grCurrent.window = (Window) w->w_grdata;
@@ -1354,7 +1354,7 @@ Void GrX11IconUpdate(w,text)
      XClassHint	class;
      char	*brack;
      
-     if (w->w_grdata == NULL) return;
+     if (w->w_grdata == (ClientData)NULL) return;
      class.res_name = "magic";
      class.res_class = "magic";
      XSetClassHint( grXdpy, wind, &class);
