@@ -74,7 +74,8 @@ grx11DrawGrid (prect, outline, clip)
 
     xsize = prect->r_xtop - prect->r_xbot;
     ysize = prect->r_ytop - prect->r_ybot;
-    if (GRID_TOO_SMALL(xsize, ysize, GR_NUM_GRIDS)) return FALSE;
+    if (!xsize || !ysize || GRID_TOO_SMALL(xsize, ysize, GR_NUM_GRIDS))
+	return FALSE;
     
     xstart = prect->r_xbot % xsize;
     while (xstart < clip->r_xbot<<12) xstart += xsize;
