@@ -1,16 +1,6 @@
 /* grX11thread.c -
  *
- *     ********************************************************************* 
- *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
- *     * Permission to use, copy, modify, and distribute this              * 
- *     * software and its documentation for any purpose and without        * 
- *     * fee is hereby granted, provided that the above copyright          * 
- *     * notice appear in all copies.  The University of California        * 
- *     * makes no representations about the suitability of this            * 
- *     * software for any purpose.  It is provided "as is" without         * 
- *     * express or implied warranty.  Export of this software outside     * 
- *     * of the United States of America may require an export license.    * 
- *     *********************************************************************
+ * Copyright 2003 Open Circuit Design, Inc., for MultiGiG Ltd.
  *
  * This file contains functions to run the X11 event loop as a separate
  * thread.  Used by X11 and OpenGL running under X11.  Compiled only if
@@ -85,6 +75,15 @@ ParseEvent (event, parentID)
 	    write(writePipe, event, sizeof(XEvent));
 	    write(writePipe, &keywstate, sizeof(int));
 	}
+	else if (!strncmp(XKeysymToString(keysym), "KP_", 3))
+	{
+	    /* keypad key (special case---would like to		*/
+	    /* differentiate between shift-KP-# and # itself)	*/
+	    keymod &= ~ShiftMask;
+	    keywstate = (keymod << 16) | (keysym & 0xffff);
+	    write(writePipe, event, sizeof(XEvent));
+	    write(writePipe, &keywstate, sizeof(int));
+	}
 	else			/* For keys with ASCII equivalent */
 	{
 	    /* If Control or Shift is used alone, it should be removed from  */
