@@ -109,28 +109,12 @@ extern char *(*GrWindowNamePtr)(); 	/* Get the window name from a MagWin ptr.
 extern bool (*GrGetCursorPosPtr)(); 	/* Get the current cursor position
 					 */
 
-/* Text output routine, should be called only by the textio module.  It is 
- * located here so that graphics drivers can do the text in a special way if 
- * they want to.  It is normally set to point to 'vfprintf', a varargs version
- * of fprintf.  If this doesn't exist (i.e. if the #define NOVARARGS is present)
- * then it normally points to fprintf.
- */
-#ifndef	NO_VARARGS
-extern int (*GrVfprintfPtr)();
-#define GrVfprintf (*GrVfprintfPtr)
-#else	/* NO_VARARGS */
-extern int (*GrFprintfPtr)();
-#define GrFprintf (*GrFprintfPtr)
-#endif	/* NO_VARARGS */
-
-
 /* Terminal state routine, called by the Textio module after it decides what
  * mode the terminal driver should be in.  This gives the graphics module a
  * chance to change it.  The pointer is normally NULL, in which case it is 
  * ignored.  */ 
 extern Void (*GrTermStatePtr)();
 
-
 /* Routines called by the signals module just before Magic is stopped
  * (such as via ^Z) and just after it is resumed.
  */
@@ -181,10 +165,8 @@ extern int GrStyleNames[];
 
 #endif
 
-#ifdef MAGIC_WRAPPER
 enum { DISPLAY_IDLE, DISPLAY_IN_PROGRESS, DISPLAY_BREAK_PENDING };
 extern unsigned char GrDisplayStatus;
-#endif
 
 /* constants */
 
@@ -216,4 +198,4 @@ extern unsigned char GrDisplayStatus;
 #define GR_CURSOR_Y	100
 
 /* Special full-screen access for window manager only */
-#define GR_LOCK_SCREEN	-1
+#define GR_LOCK_SCREEN	(MagWindow *)(-1)
