@@ -33,16 +33,20 @@ static char rcsid[] = "$Header: /ufs/repository/magic/lef/lefCmd.c,v 1.0 2003/05
  *
  * CmdLef --
  *
- * Implement the "lef" command:  generate LEF-format output for a layout
+ * Implement the "lef" and "def" commands:  generate LEF-format output
+ * for a layout, or read LEF- or DEF-format files.
  *
  * Usage:
  *	lef [options]
+ *	def [options]
  *
  * Results:
  *	None.
  *
  * Side effects:
- *	Generates LEF-format output on disk somewhere.
+ *	Generates LEF-format output on disk somewhere, or reads
+ *	LEF- or DEF-format files, generating cell definitions and
+ *	uses in the process.
  *
  * ----------------------------------------------------------------------------
  */
@@ -63,16 +67,22 @@ CmdLef(w, cmd)
     char **msg, *namep;
     CellUse *selectedUse;
     CellDef *selectedDef;
+    bool is_lef;
 
     static char *cmdLefOption[] =
     {	
-	"read [filename]	read a file filename[.lef]",
+	"read [filename]	read a file filename[.lef|.def]",
 	"write [cell]		write current or indicated cell",
 	"writeall		write all cells from the top-level cell",
 	"help                   print this help information",
 	NULL
     };
 
+    /* Determine whether this function has been called via the "lef" or
+     * the "def" command.
+     */
+    is_lef = (cmd->tx_argv[0][0] == 'd') ? FALSE : TRUE;
+
     if (cmd->tx_argc < 2)
 	option = LEF_HELP;
     else
@@ -80,7 +90,8 @@ CmdLef(w, cmd)
 	option = Lookup(cmd->tx_argv[1], cmdLefOption);
 	if (option < 0)
 	{
-	    TxError("\"%s\" isn't a valid LEF option.\n", cmd->tx_argv[1]);
+	    TxError("\"%s\" isn't a valid %s option.\n", cmd->tx_argv[1],
+			cmd->tx_argv[0]);
 	    option = LEF_HELP;
 	}
     }
@@ -108,12 +119,23 @@ CmdLef(w, cmd)
 	case LEF_READ:
             if (cmd->tx_argc != 3) goto wrongNumArgs;
             namep = cmd->tx_argv[2];
-	    LefRead(namep);
+	    if (is_lef)
+		LefRead(namep);
+	    else
+		DefRead(namep);
 	    break;
 	case LEF_WRITEALL:
-	    LefWriteAll(selectedUse);
+	    if (!is_lef)
+		TxError("Sorry, can't write DEF files at this time.\n");
+	    else
+		LefWriteAll(selectedUse);
 	    break;
 	case LEF_WRITE:
+	    if (!is_lef)
+	    {
+		TxError("Sorry, can't write DEF files at this time.\n");
+		return;
+	    }
             if (cmd->tx_argc != 3) goto wrongNumArgs;
             namep = cmd->tx_argv[2];
             selectedUse = CmdGetSelectedCell((Transform *) NULL);
@@ -127,10 +149,10 @@ CmdLef(w, cmd)
 	    break;
 	case LEF_HELP:
 wrongNumArgs:
-	    TxPrintf("The \"lef\" options are:\n");
+	    TxPrintf("The \"%s\" options are:\n", cmd->tx_argv[0]);
 	    for (msg = &(cmdLefOption[0]); *msg != NULL; msg++)
 	    {
-		TxPrintf("    lef %s\n", *msg);
+		TxPrintf("    %s %s\n", cmd->tx_argv[0], *msg);
 	    }
 	    break;
     }
