@@ -1,19 +1,20 @@
 /*
- * lef.c --      
+ * lefRead.c --      
  *
  * This module incorporates the LEF/DEF format for standard-cell place and
  * route.
  *
- * Version 0.1 (May 1, 2003):  LEF output for cells, to include pointer to
- * GDS, automatic generation of GDS if not already made, bounding box export,
- * port export, export of irouter "fence", "magnet", and "rotate" layers    
- * for defining router hints, and generating areas for obstructions and
- * pin layers.
+ * Version 0.1 (September 26, 2003):  LEF input handling.  Includes creation
+ * of cells from macro statements, handling of pins, ports, obstructions, and
+ * associated geometry.
  *
+ * Note that techfile compatibility requires that each layer name appearing
+ * in the LEF file should be present as an alias for the appropriate magic
+ * tile type in the technology file.  Layer names are not case sensitive.
  */
 
 #ifndef lint
-static char rcsid[] = "$Header: /ufs/repository/magic/lef/lef.c,v 1.0 2003/05/01 10:37:00 tim Exp $";            
+static char rcsid[] = "$Header: /ufs/repository/magic/lef/lefRead.c,v 1.0 2003/05/01 10:37:00 tim Exp $";            
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -23,6 +24,7 @@ static char rcsid[] = "$Header: /ufs/repository/magic/lef/lef.c,v 1.0 2003/05/01
 #endif
 #include <errno.h>
 #include <stdarg.h>
+#include <sys/time.h>
 
 #include "magic/tclmagic.h"
 #include "misc/magic.h"
@@ -31,8 +33,11 @@ static char rcsid[] = "$Header: /ufs/repository/magic/lef/lef.c,v 1.0 2003/05/01
 #include "utils/hash.h"
 #include "database/database.h"
 #include "windows/windows.h"
+#include "graphics/graphics.h"
 #include "dbwind/dbwind.h"
 #include "utils/malloc.h"
+#include "textio/textio.h"
+#include "cif/cif.h"
 #include "lef/lefInt.h"
 
 /* ---------------------------------------------------------------------
@@ -44,73 +49,211 @@ int lefCurrentLine;
 HashTable LefCellTable;
 HashTable lefDefInitHash;
 
-/* Some constants for LEF files */
+/*
+ *------------------------------------------------------------
+ *
+ * LefEstimate --
+ *
+ *	Estimate the time to completion based on the time,
+ *	the total number of items to process, and the number
+ *	of items processed so far.  Attempts to report about
+ *	every 5 seconds or so.
+ *
+ * Results:
+ *	None.
+ *
+ * Side Effects:
+ *	May print to output.
+ *
+ *------------------------------------------------------------
+ */
+
+#define PRINT_INTERVAL 5	/* print status at 4 second intervals */
+
+void
+LefEstimate(processed, total, item_name)
+   int processed;
+   int total;
+   char *item_name;
+{
+    static int check_interval, partition;
+    static struct timeval tv_start;
+    static float last_time;
+    struct timeval tv;
+    struct timezone tz;
+    float cur_time, time_left;
+
+    if (!total) return;
+
+    if (processed == 0)		/* Initialization */
+    {
+	gettimeofday(&tv_start, &tz);
+	GrDisplayStatus = DISPLAY_IN_PROGRESS;
+	SigSetTimer(PRINT_INTERVAL);
+    }
+    else if (processed == total - 1)
+    {
+	GrDisplayStatus = DISPLAY_IDLE;
+	SigRemoveTimer();
+    }
+    else if (GrDisplayStatus == DISPLAY_BREAK_PENDING)
+    {
+	gettimeofday(&tv, &tz);
+	cur_time = (float)(tv.tv_sec - tv_start.tv_sec)
+		+ ((float)(tv.tv_usec - tv_start.tv_usec) / 1.0e6);
+	time_left = (((float)total / (float)processed) - 1) * cur_time;
+
+	/* not likely to happen, but we don't want a divide-by-0 error */
+	if (cur_time == 0.0) cur_time = 1.0e-6;
+
+	TxPrintf("  Processed %d of %d %s (%2.1f%%).", processed, total,
+			item_name, (float)(100 * processed) / (float)total);
+	TxPrintf("  Est. time remaining: %2.1fs\n", time_left);
+	TxFlushOut();
+
+#ifdef MAGIC_WRAPPER
+	/* We need to let Tk paint the console display */
+	while (Tcl_DoOneEvent(TCL_DONT_WAIT) != 0);
+#endif
+
+	GrDisplayStatus = DISPLAY_IN_PROGRESS;
+	SigSetTimer(PRINT_INTERVAL);
+    }
+}
+
+/* This is the original version, which doesn't use the system itimer	*/
+/* and which is not very good at maintaining constant intervals.	*/
+
+#if 0
+void
+LefEstimate(processed, total, item_name)
+   int processed;
+   int total;
+   char *item_name;
+{
+    static int check_interval, partition;
+    static struct timeval tv_start;
+    static float last_time;
+    struct timeval tv;
+    struct timezone tz;
+    float cur_time, time_left;
+
+    if (!total) return;
+
+    if (processed == 0)		/* Initialization */
+    {
+	GrDisplayStatus = DISPLAY_IN_PROGRESS;
+
+	check_interval = 100;
+	gettimeofday(&tv_start, &tz);
+	last_time = 0.0;
+    }
+	
+    if (processed > check_interval)
+    {
+	gettimeofday(&tv, &tz);
+	cur_time = (float)(tv.tv_sec - tv_start.tv_sec)
+		+ ((float)(tv.tv_usec - tv_start.tv_usec) / 1.0e6);
+	time_left = (((float)total / (float)processed) - 1) * cur_time;
+
+	/* not likely to happen, but we don't want a divide-by-0 error */
+	if (cur_time == 0.0) cur_time = 1.0e-6;
+
+	partition = (int)((float)processed * (float)PRINT_INTERVAL / cur_time);
+
+	/* partition is never less than 1 nor greater than 5% of the total */
+	if (partition == 0) partition = 1;
+	if (partition > (total / 20)) partition = (total / 20);
+
+	check_interval += partition;
+
+	/* Don't print anything in intervals faster than 1 second */
+	if ((cur_time - last_time) < 1.0) return;
+	last_time = cur_time;
+
+	TxPrintf("  Processed %d of %d %s (%2.1f%%).", processed, total,
+			item_name, (float)(100 * processed) / (float)total);
+	TxPrintf("  Est. time remaining: %2.1fs\n", time_left);
+	TxFlushOut();
+    }
+}
+
+#endif /* 0 */
 
-#define LEF_LINE_MAX 2048  /* Maximum length fixed by LEF specifications */
-#define LEF_MAX_ERRORS 100 /* Max # errors to report; limits output if */
-			   /* something is really wrong about the file */
 
 /*
  *------------------------------------------------------------
  *
- * LefNextLine --
+ * LefNextToken --
  *
- *	Find the next line to process in a LEF file
+ *	Move to the next token in the stream input.
+ *	If "ignore_eol" is FALSE, then the end-of-line character
+ *	"\n" will be returned as a token when encountered.
+ *	Otherwise, end-of-line will be ignored.
  *
  * Results:
- *	Pointer to next line to parse
+ *	Pointer to next token to parse
  *
  * Side Effects:
  *	May read a new line from the specified file.
  *
+ * Warnings:
+ *	The return result of LefNextToken will be overwritten by
+ *	subsequent calls to LefNextToken if more than one line of
+ *	input is parsed.
+ *
  *------------------------------------------------------------
  */
 
 char *
-LefNextLine(f)
+LefNextToken(f, ignore_eol)
     FILE *f;
+    bool ignore_eol;
 {
-    static char *nexttoken = NULL;
-    static char line[LEF_LINE_MAX];
-    char *lineptr;
-    char *lineend;
+    static char line[LEF_LINE_MAX + 2];	/* input buffer */
+    static char *nexttoken = NULL;	/* pointer to next token */
+    static char *curtoken;		/* pointer to current token */
+    static char eol_token='\n';
+
+    /* Read a new line if necessary */
 
     if (nexttoken == NULL)
     {
-	/* Read another line from the file */
-
-	while (nexttoken == NULL)
+	for(;;)
 	{
-	    if (fgets(line, LEF_LINE_MAX, f) == NULL)
-		return NULL; 
+	    if (fgets(line, LEF_LINE_MAX + 1, f) == NULL) return NULL;
 	    lefCurrentLine++;
-	    nexttoken = line;
-	    while (isspace(*nexttoken) || (*nexttoken == ';'))
-		nexttoken++;
-	    if (*nexttoken == '\0')
-		nexttoken = NULL;
-	}
-	lineptr = nexttoken;
+	    curtoken = line;
+	    while (isspace(*curtoken) && (*curtoken != '\n') && (*curtoken != '\0'))
+		curtoken++;		/* skip leading whitespace */
 
-	/* Remove any newline character from the string */
-	if ((lineend = strchr(lineptr, '\n')) != NULL) *lineend = '\0';
+	    if ((*curtoken != '#') && (*curtoken != '\n') && (*curtoken != '\0'))
+	    {
+		nexttoken = curtoken;
+		break;
+	    }
+	}
+	if (!ignore_eol)
+	    return &eol_token;
     }
     else
-	lineptr = nexttoken;
+	curtoken = nexttoken;
 
-    /* Find the next complete LEF statement and set "nexttoken" to */
-    /* point to it.  If we encounter EOL, set nexttoken to NULL.   */
+    /* Find the next token; set to NULL if none (end-of-line). */
 
-    nexttoken = strchr(nexttoken, ';');
-    if (nexttoken != NULL)
-    {
-	*nexttoken++ = '\0';
-	while (isspace(*nexttoken) || (*nexttoken == ';'))
-	    nexttoken++;
-	if (*nexttoken == '\0')
-	    nexttoken = NULL;
-    }
-    return lineptr;
+    while (!isspace(*nexttoken) && (*nexttoken != '\0') && (*nexttoken != '\n'))
+	nexttoken++;	/* skip non-whitespace (move past current token) */
+
+    /* Terminate the current token */
+    if (*nexttoken != '\0') *nexttoken++ = '\0';
+
+    while (isspace(*nexttoken) && (*nexttoken != '\0') && (*nexttoken != '\n'))
+	nexttoken++;	/* skip any whitespace */
+
+    if ((*nexttoken == '#') || (*nexttoken == '\n') || (*nexttoken == '\0'))
+	nexttoken = NULL;
+
+    return curtoken;
 }
 
 /*
@@ -136,11 +279,14 @@ LefError(char *fmt, ...)
     static int errors = 0;
     va_list args;
 
-    if (fmt == NULL)
+    if (fmt == NULL)  /* Special case:  report any errors and reset */
     {
 	if (errors)
+	{
 	    TxPrintf("LEF Read: encountered %d error%s total.\n", errors,
 			(errors == 1) ? "" : "s");
+	    errors = 0;
+	}
 	return;
     }
 
@@ -148,8 +294,9 @@ LefError(char *fmt, ...)
     {
 	TxError("LEF Read, Line %d: ", lefCurrentLine);
 	va_start(args, fmt);
-	(void) GrVfprintf(stderr, fmt, args);
+	(void) Vfprintf(stderr, fmt, args);
 	va_end(args);
+	TxFlushErr();
     }
     else if (errors == LEF_MAX_ERRORS)
 	TxError("LEF Read:  Further errors will not be reported.\n");
@@ -178,27 +325,35 @@ LefError(char *fmt, ...)
  */
 
 bool
-LefParseEndStatement(lineptr, match)
-    char *lineptr;
+LefParseEndStatement(f, match)
+    FILE *f;
     char *match;
 {
-    char token[256];
+    char *token;
     int keyword, words;
     char *match_name[2];
 
     match_name[0] = match;
     match_name[1] = NULL;
 
-    words = sscanf(lineptr, "%*s %255s", token);
-
-    if ((words <= 0) && (match == NULL)) return TRUE;
-    else if (words != 1) return FALSE;
-
-    keyword = Lookup(token, match_name);
-    if (keyword == 0)
-	return TRUE;
-    else
+    token = LefNextToken(f, (match == NULL) ? FALSE : TRUE);
+    if (token == NULL)
+    {
+	LefError("Bad file read while looking for END statement\n");
 	return FALSE;
+    }
+
+    /* END or ENDEXT */
+    if ((*token == '\n') && (match == NULL)) return TRUE;
+
+    /* END <section_name> */
+    else {
+	keyword = Lookup(token, match_name);
+	if (keyword == 0)
+	    return TRUE;
+	else
+	    return FALSE;
+    }
 }
 
 /*
@@ -228,18 +383,27 @@ LefSkipSection(f, section)
     FILE *f;
     char *section;
 {
-    char *lineptr;
-    char token[256];
+    char *token;
+    int keyword;
     static char *end_section[] = {
 	"END",
+	"ENDEXT",
 	NULL
     };
 
-    while ((lineptr = LefNextLine(f)) != NULL)
-	if (sscanf(lineptr, "%255s", token) == 1)
-	    if (Lookup(token, end_section) == 0)
-		if (LefParseEndStatement(lineptr, section))
-		    return;
+    while ((token = LefNextToken(f, TRUE)) != NULL)
+    {
+	if ((keyword = Lookup(token, end_section)) == 0)
+	{
+	    if (LefParseEndStatement(f, section))
+		return;
+	}
+	else if (keyword == 1)
+	{
+	    if (!strcmp(section, "BEGINEXT"))
+		return;
+	}
+    }
 
     LefError("Section %s has no END record!\n", section);
     return;
@@ -277,39 +441,23 @@ lefFindCell(name)
 /*
  *------------------------------------------------------------
  *
- * LefSkipToken --
+ * LefLower --
  *
- *	Skip past the current token in the specified line.
- *	Write an end-of-string in front of all whitespace
- *	at the end of the line.
- *
- * Results:
- *	Pointer to the start of the next keyword (token).
- *
- * Side effects:
- *	None.
+ *	Convert a token in a LEF or DEF file to all-lowercase.
  *
  *------------------------------------------------------------
  */
 
 char *
-LefSkipToken(lineptr)
-    char *lineptr;
+LefLower(token)
+    char *token;
 {
-    char *tokptr = lineptr;
-    char *endptr;
+    char *tptr;
 
-    while (isspace(*tokptr)) tokptr++;  /* skip whitespace */
-    while (!isspace(*tokptr)) tokptr++;	/* skip keyword */
-    while (isspace(*tokptr)) tokptr++;  /* skip whitespace */
+    for (tptr = token; *tptr != '\0'; tptr++)
+	*tptr = tolower(*tptr);
 
-    endptr = tokptr;
-    while (*endptr != '\0') endptr++;	/* go to end of line */
-    while (isspace(*(endptr - 1)))
-	endptr--;			/* skip whitespace */
-    *endptr = '\0';			/* truncate line */
-
-    return tokptr;
+    return token;
 }
 
 /*
@@ -333,28 +481,21 @@ LefSkipToken(lineptr)
  *------------------------------------------------------------
  */
 
-typedef struct _linkedRect {
-   Rect	     area;
-   TileType  type;
-   struct _linkedRect *rect_next;
-} linkedRect; 
-
 enum lef_geometry_keys {LEF_LAYER = 0, LEF_WIDTH, LEF_PATH,
 	LEF_RECT, LEF_POLYGON, LEF_VIA, LEF_GEOMETRY_END};
 
 linkedRect *
-LefReadGeometry(lefMacro, f, do_list)
+LefReadGeometry(lefMacro, f, oscale, do_list)
     CellDef *lefMacro;
     FILE *f;
+    float oscale;
     bool do_list;
 {
-    TileType curlayer;
+    TileType curlayer = -1;
 
-    char token[256], *tptr;
-    char *lineptr;
+    char *token;
     int keyword;
     float llx, lly, urx, ury;
-    float oscale;
     Rect paintrect;
     linkedRect *newRect, *rectList;
 
@@ -369,44 +510,36 @@ LefReadGeometry(lefMacro, f, do_list)
 	NULL
     };
 
-    oscale = CIFGetOutputScale(1000);
     rectList = NULL;
 
-    while ((lineptr = LefNextLine(f)) != NULL)
+    while ((token = LefNextToken(f, TRUE)) != NULL)
     {
-	if (sscanf(lineptr, "%s", token) == 0)
-	    continue;	/* Blank line or null statement */
-
 	keyword = Lookup(token, geometry_keys);
 	if (keyword < 0)
 	{
 	    LefError("Unknown keyword \"%s\" in LEF file; ignoring.\n", token);
+	    LefEndStatement(f);
 	    continue;
 	}
 	switch (keyword)
 	{
 	    case LEF_LAYER:
-		if (sscanf(lineptr, "%*s %s", token) != 1)
+		token = LefNextToken(f, TRUE);
+		if (*token == ';')
 		    LefError("Bad Layer statement\n");
 		else
 		{
 		    /* NOTE:  magic-to-LEF layer mappings should be	*/
-		    /* declared in the technology file.  This ad-hoc	*/
-		    /* mapping is temporary!				*/
+		    /* declared in the technology file as aliases for	*/
+		    /* magic layer names.  The primary type is case-	*/
+		    /* sensitive, but this routine will also check the	*/
+		    /* lowercase equivalent (prevents having to add	*/
+		    /* entries for things like METAL1, etc.).		*/
 
-		    for (tptr = token; *tptr != '\0'; *tptr++)
-			*tptr = tolower(*tptr);
-	
-		    if (!strncmp(token, "metal", 5))
-		    {
-			curlayer = DBTechNameType(token);
-		    }
-		    else if (!strncmp(token, "via", 3))
-		    {
-			curlayer = DBTechNameType(token);
-		    }
-		    else if (!strncmp(token, "poly", 4))
+		    curlayer = DBTechNameType(token);
+		    if (curlayer < 0)
 		    {
+			(void) LefLower(token);
 			curlayer = DBTechNameType(token);
 		    }
 		    if (curlayer < 0)
@@ -414,16 +547,24 @@ LefReadGeometry(lefMacro, f, do_list)
 			LefError("Don't know how to parse layer \"%s\"\n", token);
 			LefError("Try adding this name to the technology styles\n");
 		    }
+		    LefEndStatement(f);
 		}
 		break;
 	    case LEF_WIDTH:
+		LefEndStatement(f);
 		break;
 	    case LEF_PATH:
+		LefEndStatement(f);
 		break;
 	    case LEF_RECT:
-		tptr = LefSkipToken(lineptr);
-		if (sscanf(tptr, "%f %f %f %f", &llx, &lly, &urx, &ury) != 4)
-		    LefError("Bad port geometry: RECT requires 4 values.\n");
+		token = LefNextToken(f, TRUE);
+		if (!token || sscanf(token, "%f", &llx) != 1) goto parse_error;
+		token = LefNextToken(f, TRUE);
+		if (!token || sscanf(token, "%f", &lly) != 1) goto parse_error;
+		token = LefNextToken(f, TRUE);
+		if (!token || sscanf(token, "%f", &urx) != 1) goto parse_error;
+		token = LefNextToken(f, TRUE);
+		if (!token || sscanf(token, "%f", &ury) != 1) goto parse_error;
 		if (curlayer < 0)
 		    LefError("No layer defined for RECT.\n");
 		else
@@ -449,19 +590,29 @@ LefReadGeometry(lefMacro, f, do_list)
 			rectList = newRect;
 		    }
 
+		    /* Diagnostic */
+		    /*
 		    TxPrintf("   Painting %s at (%d, %d), (%d, %d) in cell %s.\n",
 			DBTypeLongNameTbl[curlayer],
 			paintrect.r_xbot, paintrect.r_ybot,
 			paintrect.r_xtop, paintrect.r_ytop,
 			lefMacro->cd_name);
+		    */
 		}
+		LefEndStatement(f);
+		break;
+parse_error:
+		LefError("Bad port geometry: RECT requires 4 values.\n");
+		LefEndStatement(f);
 		break;
 	    case LEF_POLYGON:
+		LefEndStatement(f);
 		break;
 	    case LEF_VIA:
+		LefEndStatement(f);
 		break;
 	    case LEF_GEOMETRY_END:
-		if (!LefParseEndStatement(lineptr, NULL))
+		if (!LefParseEndStatement(f, NULL))
 		{
 		    LefError("Geometry (PORT or OBS) END statement missing.\n");
 		    keyword = -1;
@@ -492,16 +643,17 @@ LefReadGeometry(lefMacro, f, do_list)
  */
 
 void
-LefReadPort(lefMacro, f, pinName, pinNum, pinDir, pinUse)
+LefReadPort(lefMacro, f, pinName, pinNum, pinDir, pinUse, oscale)
     CellDef *lefMacro;
     FILE *f;
     char *pinName;
     int pinNum, pinDir, pinUse;
+    float oscale;
 {
     Label *newlab;
     linkedRect *rectList;
 
-    rectList = LefReadGeometry(lefMacro, f);
+    rectList = LefReadGeometry(lefMacro, f, oscale, TRUE);
 
     while (rectList != NULL)
     {
@@ -516,15 +668,13 @@ LefReadPort(lefMacro, f, pinName, pinNum, pinDir, pinUse)
 		LefError("Internal error: No labels in cell!\n");
 	    else
 	    {
-		for (newlab = lefMacro->cd_labels; newlab->lab_next != NULL;
-				newlab = newlab->lab_next);  /* Nothing */
-
+		newlab = lefMacro->cd_lastLabel;
 		if (strcmp(newlab->lab_text, pinName))
 		    LefError("Internal error:  Can't find the label!\n");
 		else /* Make this a port */
 		    newlab->lab_flags = pinNum | pinUse | pinDir | PORT_DIR_MASK;
 	    }
-	    DBAdjustLabels(lefMacro, &rectList->area);
+	    /* DBAdjustLabels(lefMacro, &rectList->area); */
 	}
 
 	freeMagic((char *)rectList);
@@ -548,17 +698,18 @@ LefReadPort(lefMacro, f, pinName, pinNum, pinDir, pinUse)
  *------------------------------------------------------------
  */
 
-enum lef_pin_keys {LEF_DIRECTION = 0, LEF_USE, LEF_PORT, LEF_PIN_END};
+enum lef_pin_keys {LEF_DIRECTION = 0, LEF_USE, LEF_PORT, LEF_CAPACITANCE,
+	LEF_PIN_END};
 
 void
-LefReadPin(lefMacro, f, pinname, pinNum)
+LefReadPin(lefMacro, f, pinname, pinNum, oscale)
    CellDef *lefMacro;
    FILE *f;
    char *pinname;
    int pinNum;
+   float oscale;
 {
-    char token[256], *tptr;
-    char *lineptr;
+    char *token;
     int keyword, subkey;
     int pinDir = PORT_CLASS_DEFAULT;
     int pinUse = PORT_USE_DEFAULT;
@@ -567,6 +718,7 @@ LefReadPin(lefMacro, f, pinname, pinNum)
 	"DIRECTION",
 	"USE",
 	"PORT",
+	"CAPACITANCE",
 	"END",
 	NULL
     };
@@ -609,41 +761,43 @@ LefReadPin(lefMacro, f, pinname, pinNum)
 	PORT_USE_CLOCK
     };
 
-    while ((lineptr = LefNextLine(f)) != NULL)
+    while ((token = LefNextToken(f, TRUE)) != NULL)
     {
-	if (sscanf(lineptr, "%s", token) == 0)
-	    continue;	/* Blank line or null statement */
-
 	keyword = Lookup(token, pin_keys);
 	if (keyword < 0)
 	{
 	    LefError("Unknown keyword \"%s\" in LEF file; ignoring.\n", token);
+	    LefEndStatement(f);
 	    continue;
 	}
 	switch (keyword)
 	{
 	    case LEF_DIRECTION:
-		tptr = LefSkipToken(lineptr);
-		sscanf(lineptr, "%*s %255s", token);
-		subkey = Lookup(tptr, pin_classes);
+		token = LefNextToken(f, TRUE);
+		subkey = Lookup(token, pin_classes);
 		if (subkey < 0)
 		    LefError("Improper DIRECTION statement\n");
 		else
 		    pinDir = lef_class_to_bitmask[subkey];
+		LefEndStatement(f);
 		break;
 	    case LEF_USE:
-		tptr = LefSkipToken(lineptr);
-		subkey = Lookup(tptr, pin_uses);
+		token = LefNextToken(f, TRUE);
+		subkey = Lookup(token, pin_uses);
 		if (subkey < 0)
 		    LefError("Improper USE statement\n");
 		else
 		    pinUse = lef_use_to_bitmask[subkey];
+		LefEndStatement(f);
 		break;
 	    case LEF_PORT:
-		LefReadPort(lefMacro, f, pinname, pinNum, pinDir, pinUse);
+		LefReadPort(lefMacro, f, pinname, pinNum, pinDir, pinUse, oscale);
+		break;
+	    case LEF_CAPACITANCE:
+		LefEndStatement(f);	/* Ignore. . . */
 		break;
 	    case LEF_PIN_END:
-		if (!LefParseEndStatement(lineptr, pinname))
+		if (!LefParseEndStatement(f, pinname))
 		{
 		    LefError("Pin END statement missing.\n");
 		    keyword = -1;
@@ -652,8 +806,25 @@ LefReadPin(lefMacro, f, pinname, pinNum)
 	}
 	if (keyword == LEF_PIN_END) break;
     }
+}
 
-    /* Add use and direction information to the pin. */
+/*
+ *------------------------------------------------------------
+ * LefEndStatement --
+ *
+ *	Read file input to EOF or a ';' token (end-of-statement)
+ *
+ *------------------------------------------------------------
+ */
+
+void
+LefEndStatement(f)
+    FILE *f;
+{
+    char *token;
+
+    while ((token = LefNextToken(f, TRUE)) != NULL)
+	if (*token == ';') break;
 }
 
 /*
@@ -674,20 +845,20 @@ LefReadPin(lefMacro, f, pinname, pinNum)
 
 enum lef_macro_keys {LEF_CLASS = 0, LEF_SIZE, LEF_ORIGIN,
 	LEF_SYMMETRY, LEF_SOURCE, LEF_SITE, LEF_PIN, LEF_OBS,
-	LEF_TIMING, LEF_MACRO_END};
+	LEF_TIMING, LEF_FOREIGN, LEF_MACRO_END};
 
 void
-LefReadMacro(f, mname)
+LefReadMacro(f, mname, oscale)
     FILE *f;			/* LEF file being read	*/
     char *mname;		/* name of the macro 	*/
+    float oscale;		/* scale factor um->magic units */
 {
     CellDef *lefMacro;
     HashEntry *he;
 
-    char token[256], *tptr;
-    char *lineptr;
+    char *token, tsave[128];
     int keyword, pinNum;
-    float x, y, oscale;
+    float x, y;
     bool has_size;
     Rect lefBBox;
 
@@ -701,6 +872,7 @@ LefReadMacro(f, mname)
 	"PIN",
 	"OBS",
 	"TIMING",
+	"FOREIGN",
 	"END",
 	NULL
     };
@@ -732,85 +904,104 @@ LefReadMacro(f, mname)
     /* Initial values */
     pinNum = 1;
     has_size = FALSE;
-    oscale = CIFGetOutputScale(1000);
     lefBBox.r_xbot = 0;
     lefBBox.r_ybot = 0;
 
-    while ((lineptr = LefNextLine(f)) != NULL)
+    while ((token = LefNextToken(f, TRUE)) != NULL)
     {
-	if (sscanf(lineptr, "%s", token) == 0)
-	    continue;	/* Blank line or null statement */
-
 	keyword = Lookup(token, macro_keys);
 	if (keyword < 0)
 	{
 	    LefError("Unknown keyword \"%s\" in LEF file; ignoring.\n", token);
+	    LefEndStatement(f);
 	    continue;
 	}
 	switch (keyword)
 	{
 	    case LEF_CLASS:
-		tptr = LefSkipToken(lineptr);
-		if (*tptr != '\0')
-		    DBPropPut(lefMacro, "LEFclass", tptr);
+		token = LefNextToken(f, TRUE);
+		if (*token != '\n')
+		    DBPropPut(lefMacro, "LEFclass", token);
+		LefEndStatement(f);
 		break;
 	    case LEF_SIZE:
-		tptr = LefSkipToken(lineptr);
-		if (sscanf(tptr, "%f %*s %f", &x, &y) != 2)
-		    LefError("Bad macro SIZE; requires values X BY Y.\n");
-		else
-		{
-		    lefBBox.r_xtop = (int)(x / oscale) + lefBBox.r_xbot;
-		    lefBBox.r_ytop = (int)(y / oscale) + lefBBox.r_ybot;
-		    has_size = TRUE;
-		}
+		token = LefNextToken(f, TRUE);
+		if (!token || sscanf(token, "%f", &x) != 1) goto size_error;
+		token = LefNextToken(f, TRUE);		/* skip keyword "BY" */
+		if (!token) goto size_error;
+		token = LefNextToken(f, TRUE);
+		if (!token || sscanf(token, "%f", &y) != 1) goto size_error;
+
+		lefBBox.r_xtop = (int)(x / oscale) + lefBBox.r_xbot;
+		lefBBox.r_ytop = (int)(y / oscale) + lefBBox.r_ybot;
+		has_size = TRUE;
+		LefEndStatement(f);
+		break;
+size_error:
+		LefError("Bad macro SIZE; requires values X BY Y.\n");
+		LefEndStatement(f);
 		break;
 	    case LEF_ORIGIN:
-		tptr = LefSkipToken(lineptr);
-		if (sscanf(tptr, "%f %f", &x, &y) != 2)
-		    LefError("Bad macro ORIGIN; requires 2 values.\n");
-		else
+		token = LefNextToken(f, TRUE);
+		if (!token || sscanf(token, "%f", &x) != 1) goto origin_error;
+		token = LefNextToken(f, TRUE);
+		if (!token || sscanf(token, "%f", &y) != 1) goto origin_error;
+
+		lefBBox.r_xbot = -(int)(x / oscale);
+		lefBBox.r_ybot = -(int)(y / oscale);
+		if (has_size)
 		{
-		    lefBBox.r_xbot = (int)(x / oscale);
-		    lefBBox.r_ybot = (int)(y / oscale);
-		    if (has_size)
-		    {
-			lefBBox.r_xtop += lefBBox.r_xbot;
-			lefBBox.r_ytop += lefBBox.r_ybot;
-		    }
+		    lefBBox.r_xtop += lefBBox.r_xbot;
+		    lefBBox.r_ytop += lefBBox.r_ybot;
 		}
+		LefEndStatement(f);
+		break;
+origin_error:
+		LefError("Bad macro ORIGIN; requires 2 values.\n");
+		LefEndStatement(f);
 		break;
 	    case LEF_SYMMETRY:
-		tptr = LefSkipToken(lineptr);
-		if (*tptr != '\0')
-		    DBPropPut(lefMacro, "LEFsymmetry", tptr);
+		token = LefNextToken(f, TRUE);
+		if (*token != '\n')
+		    DBPropPut(lefMacro, "LEFsymmetry", token + strlen(token) + 1);
+		LefEndStatement(f);
 		break;
 	    case LEF_SOURCE:
-		tptr = LefSkipToken(lineptr);
-		if (*tptr != '\0')
-		    DBPropPut(lefMacro, "LEFsource", tptr);
+		token = LefNextToken(f, TRUE);
+		if (*token != '\n')
+		    DBPropPut(lefMacro, "LEFsource", token);
+		LefEndStatement(f);
 		break;
 	    case LEF_SITE:
-		tptr = LefSkipToken(lineptr);
-		if (*tptr != '\0')
-		    DBPropPut(lefMacro, "LEFsite", tptr);
-		break;
+		token = LefNextToken(f, TRUE);
+		if (*token != '\n')
+		    DBPropPut(lefMacro, "LEFsite", token);
+		LefEndStatement(f);
 		break;
 	    case LEF_PIN:
-		sscanf(lineptr, "%*s %255s", token);
+		token = LefNextToken(f, TRUE);
+		/* Diagnostic */
+		/*
 		TxPrintf("   Macro defines pin %s\n", token);
-		LefReadPin(lefMacro, f, token, pinNum++);
+		*/
+		sprintf(tsave, "%.127s", token);
+		LefReadPin(lefMacro, f, tsave, pinNum++, oscale);
 		break;
 	    case LEF_OBS:
+		/* Diagnostic */
+		/*
 		TxPrintf("   Macro defines obstruction\n");
-		LefReadGeometry(lefMacro, f, NULL);
-		LefSkipSection(f, NULL);
+		*/
+		LefReadGeometry(lefMacro, f, oscale, FALSE);
 		break;
 	    case LEF_TIMING:
 		LefSkipSection(f, macro_keys[LEF_TIMING]);
 		break;
+	    case LEF_FOREIGN:
+		LefEndStatement(f);
+		break;
 	    case LEF_MACRO_END:
-		if (!LefParseEndStatement(lineptr, mname))
+		if (!LefParseEndStatement(f, mname))
 		{
 		    LefError("Macro END statement missing.\n");
 		    keyword = -1;
@@ -833,7 +1024,7 @@ LefReadMacro(f, mname)
 	lefMacro->cd_bbox = lefBBox;
 
     /* Fix the bounding box and do not allow edits */
-    lefMacro->cd_flags |= CDNOEDIT | CDFIXEDBBOX;
+    lefMacro->cd_flags |= /* CDNOEDIT | */ CDFIXEDBBOX;
 
     DRCCheckThis(lefMacro, TT_CHECKPAINT, &lefMacro->cd_bbox);
     DBWAreaChanged(lefMacro, &lefMacro->cd_bbox, DBW_ALLWINDOWS,
@@ -863,9 +1054,8 @@ enum lef_sections {LEF_VERSION = 0, LEF_NAMESCASESENSITIVE,
 	LEF_SECTION_VIA, LEF_SECTION_VIARULE,
 	LEF_SECTION_SPACING, LEF_SECTION_SITE, LEF_PROPERTY,
 	LEF_NOISETABLE, LEF_CORRECTIONTABLE, LEF_IRDROP,
-	LEF_MACRO, LEF_END};
-enum lef_units_keys {LEF_DATABASE = 0};
-enum lef_library_keys {LEF_LIBRARY = 0};
+	LEF_ARRAY, LEF_SECTION_TIMING, LEF_EXTENSION, LEF_MACRO,
+	LEF_END};
 
 void
 LefRead(inName)
@@ -873,9 +1063,10 @@ LefRead(inName)
 {
     FILE *f;
     char *filename;
-    char token[256];
-    char *lineptr;
+    char *token;
+    char tsave[128];
     int keyword;
+    float oscale;
 
     static char *sections[] = {
 	"VERSION",
@@ -891,16 +1082,15 @@ LefRead(inName)
 	"NOISETABLE",
 	"CORRECTIONTABLE",
 	"IRDROP",
+	"ARRAY",
+	"TIMING",
+	"BEGINEXT",
 	"MACRO",
 	"END",
 	NULL
     };
-    static char *units_keys[] = {
-	"DATABASE",
-	NULL
-    };
 
-    f = lefFileOpen(NULL, inName, "r", &filename);
+    f = lefFileOpen(NULL, inName, ".lef", "r", &filename);
 
     if (f == NULL)
     {
@@ -921,23 +1111,24 @@ LefRead(inName)
     /* Initialize */
     HashInit(&LefCellTable, 32, HT_STRINGKEYS);
     HashInit(&lefDefInitHash, 32, HT_STRINGKEYS);
+    oscale = CIFGetOutputScale(1000);
 
-    while ((lineptr = LefNextLine(f)) != NULL)
+    while ((token = LefNextToken(f, TRUE)) != NULL)
     {
-	if (sscanf(lineptr, "%s", token) == 0)
-	    continue;	/* Blank line or null statement */
-
 	keyword = Lookup(token, sections);
 	if (keyword < 0)
 	{
 	    LefError("Unknown keyword \"%s\" in LEF file; ignoring.\n", token);
+	    LefEndStatement(f);
 	    continue;
 	}
 	switch (keyword)
 	{
 	    case LEF_VERSION:
+		LefEndStatement(f);
 		break;
 	    case LEF_NAMESCASESENSITIVE:
+		LefEndStatement(f);
 		break;
 	    case LEF_PROPERTYDEFS:
 		LefSkipSection(f, sections[LEF_PROPERTYDEFS]);
@@ -946,27 +1137,31 @@ LefRead(inName)
 		LefSkipSection(f, sections[LEF_UNITS]);
 		break;
 	    case LEF_SECTION_LAYER:
-		sscanf(lineptr, "%*s %255s", token);
-		TxPrintf("LEF file:  Defines layer %s\n", token);
-		LefSkipSection(f, token);
+		token = LefNextToken(f, TRUE);
+		TxPrintf("LEF file:  Defines layer %s (ignored)\n", token);
+		sprintf(tsave, "%.127s", token);
+		LefSkipSection(f, tsave);
 		break;
 	    case LEF_SECTION_VIA:
-		sscanf(lineptr, "%*s %255s", token);
-		TxPrintf("LEF file:  Defines via %s\n", token);
-		LefSkipSection(f, token);
+		token = LefNextToken(f, TRUE);
+		TxPrintf("LEF file:  Defines via %s (ignored)\n", token);
+		sprintf(tsave, "%.127s", token);
+		LefSkipSection(f, tsave);
 		break;
 	    case LEF_SECTION_VIARULE:
-		sscanf(lineptr, "%*s %255s", token);
-		TxPrintf("LEF file:  Defines via rule %s\n", token);
-		LefSkipSection(f, token);
+		token = LefNextToken(f, TRUE);
+		TxPrintf("LEF file:  Defines via rule %s (ignored)\n", token);
+		sprintf(tsave, "%.127s", token);
+		LefSkipSection(f, tsave);
 		break;
 	    case LEF_SECTION_SPACING:
 		LefSkipSection(f, sections[LEF_SECTION_SPACING]);
 		break;
 	    case LEF_SECTION_SITE:
-		sscanf(lineptr, "%*s %255s", token);
-		TxPrintf("LEF file:  Defines site %s\n", token);
-		LefSkipSection(f, token);
+		token = LefNextToken(f, TRUE);
+		TxPrintf("LEF file:  Defines site %s (ignored)\n", token);
+		sprintf(tsave, "%.127s", token);
+		LefSkipSection(f, tsave);
 		break;
 	    case LEF_PROPERTY:
 		LefSkipSection(f, NULL);
@@ -980,13 +1175,26 @@ LefRead(inName)
 	    case LEF_IRDROP:
 		LefSkipSection(f, sections[LEF_IRDROP]);
 		break;
+	    case LEF_ARRAY:
+		LefSkipSection(f, sections[LEF_ARRAY]);
+		break;
+	    case LEF_SECTION_TIMING:
+		LefSkipSection(f, sections[LEF_SECTION_TIMING]);
+		break;
+	    case LEF_EXTENSION:
+		LefSkipSection(f, sections[LEF_EXTENSION]);
+		break;
 	    case LEF_MACRO:
-		sscanf(lineptr, "%*s %255s", token);
+		token = LefNextToken(f, TRUE);
+		/* Diagnostic */
+		/*
 		TxPrintf("LEF file:  Defines new cell %s\n", token);
-		LefReadMacro(f, token);
+		*/
+		sprintf(tsave, "%.127s", token);
+		LefReadMacro(f, tsave, oscale);
 		break;
 	    case LEF_END:
-		if (!LefParseEndStatement(lineptr, "LIBRARY"))
+		if (!LefParseEndStatement(f, "LIBRARY"))
 		{
 		    LefError("END statement out of context.\n");
 		    keyword = -1;
@@ -995,7 +1203,7 @@ LefRead(inName)
 	}
 	if (keyword == LEF_END) break;
     }
-    TxPrintf("LEF read:  Done reading file\n");
+    TxPrintf("LEF read: Processed %d lines.\n", lefCurrentLine);
     LefError(NULL);	/* print statement of errors, if any */
 
     /* Cleanup */
