@@ -541,7 +541,7 @@ proc magic::boxview {win} {
       }
 
       set framename [winfo parent $win]
-      set lambda [tech lambda]
+      set lambda [${win} tech lambda]
       set lr [expr {(0.0 + [lindex $lambda 0]) / [lindex $lambda 1]}]
       set bval [${win} box values]
       set bllx [expr {[lindex $bval 0] * $lr }]
@@ -562,7 +562,7 @@ proc magic::cursorview {win} {
       return
    }
    set framename [winfo parent $win]
-   set lambda [tech lambda]
+   set lambda [${win} tech lambda]
    set lr [expr {(0.0 + [lindex $lambda 0]) / [lindex $lambda 1]}]
    set olst [${win} cursor]
    set olstx [expr {[lindex $olst 0] * $lr }]
