@@ -24,6 +24,7 @@
  * included before this magic.h file.
  */
 
+/* Below is deprecated---magic from 7.2 rev. 56 is 64-bit clean! */
 #ifdef ALPHA32BIT
 /* must be before any declarations with pointers in them! */
 #pragma pointer_size(short)
@@ -32,10 +33,31 @@
 #endif
 #endif
 
+/* ------------------- Universal pointer typecast --------------------- */
+
+/* Set default value for backwards compatibility with non-autoconf make */
+#ifndef SIZEOF_UNSIGNED_INT
+#define SIZEOF_UNSIGNED_INT 4
+#endif
+#ifndef SIZEOF_UNSIGNED_LONG
+#define SIZEOF_UNSIGNED_LONG 4
+#endif
+#ifndef SIZEOF_VOID_P
+#define SIZEOF_VOID_P SIZEOF_UNSIGNED_INT
+#endif
+
+#if SIZEOF_VOID_P == SIZEOF_UNSIGNED_INT
+typedef unsigned int pointertype;
+#elif SIZEOF_VOID_P == SIZEOF_UNSIGNED_LONG
+typedef unsigned long pointertype;
+#else
+ERROR: Cannot compile without knowing the size of a pointer.  See misc/magic.h
+#endif
+
 /* --------------------- Universal pointer type ----------------------- */
 
 #ifndef _CLIENTDATA
-typedef char *	ClientData;
+typedef pointertype ClientData;
 #endif
 
 /* --------------------------- Booleans ------------------------------- */
