@@ -20,7 +20,7 @@
 
 #ifndef lint
 static char rcsid[] = "$Header: /ufs/repository/magic/mzrouter/mzInit.c,v 1.2 2001/01/12 22:13:02 jsolomon Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 /*--- includes --- */
 #include <stdio.h>
