@@ -22,7 +22,7 @@
 
 #ifndef lint
 static char rcsid[] = "$Header: /ufs/repository/magic/netmenu/NMmain.c,v 1.2 2001/01/12 22:13:07 jsolomon Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #include "misc/magic.h"
@@ -41,70 +41,12 @@ static char rcsid[] = "$Header: /ufs/repository/magic/netmenu/NMmain.c,v 1.2 200
 #include "netmenu/netmenu.h"
 #include "undo/undo.h"
 
+global WindClient NMClientID;
+
 /* Forward declaration: */
 
 extern Void nmNotDefined();
 
-/* The comand table defined below is empty.  This means that in
- * netlist windows the only way to invoke a command is by
- * pushing a button.
- */
-
-global char * NMcommands[] =
-{
-    "add term1 term2         add term1 to net of term2",
-    "cleanup                 interactively cleanup netlist",
-    "cull                    remove fully-wired nets from the current netlist",
-    "dnet name name ...      delete the net(s) containing name(s)\n\
-                        or current net if no name(s) given",
-    "dterm name name ...     delete terminals from nets",
-    "extract                 generate net for terminals connected to box",
-    "find pattern [layers]   find all occurrences of any labels matching\n\
-                        pattern beneath the box (on layers if specified)\n\
-                        and leave as feedback",
-    "flush [netlist]         flush changes to netlist (current list default)",
-    "join term1 term2        join the nets containing term1 and term2",
-    "netlist [name]          switch current netlist to name.net (default\n\
-			is edit cell name)",
-    "print [name]            print all terminals in name, or in current net\n\
-			if no name given",
-    "ripup [netlist]         ripup edit cell paint connected to paint under\n\
-			box, or ripup current netlist if \"netlist\"\n\
-                        typed as argument",
-    "savenetlist [file]      write out current netlist",
-    "shownet                 highlight edit cell paint connected to paint\n\
-			under box",
-    "showterms               generate feedback for all terminals in netlist",
-    "trace [name]            highlight material connected to a net's \n\
-                        terminals (use current net if no name given)",
-    "verify                  make sure current netlist is correctly wired",
-    "writeall                write out all modified netlists",
-    NULL
-};
-
-
-Void (*(NMrtns[]))() =
-{
-    NMCmdAdd,
-    NMCmdCleanup,
-    NMCmdCull,
-    NMCmdDnet,
-    NMCmdDterm,
-    NMCmdExtract,
-    NMCmdFindLabels,
-    NMCmdFlush,
-    NMCmdJoin,
-    NMCmdNetlist,
-    NMCmdPrint,
-    NMCmdRipup,
-    NMCmdSavenetlist,
-    NMCmdShownet,
-    NMCmdShowterms,
-    NMCmdTrace,
-    NMCmdVerify,
-    NMCmdWriteall,
-    0,
-};
     
 /* The declarations below define the layout of the netlist
  * window.  Everything is defined in "surface" coordinates,
@@ -425,11 +367,7 @@ NMcommand(w, cmd)
 
     if (cmd->tx_button == TX_NO_BUTTON)
     {
-	if (cmd->tx_argc > 0)
-	{
-	    cmdNum = Lookup(cmd->tx_argv[0], NMcommands);
-	    (void) (*NMrtns[cmdNum])(w, cmd);
-	}
+	WindExecute(w, NMClientID, cmd);
 	goto done;
     }
 
@@ -525,15 +463,82 @@ NMinit()
    middle  - join current net and net containing terminal nearest the cursor\n\
 ";
 
-    (void) WindAddClient("netlist", NMcreate, NMdelete,
+    NMClientID = WindAddClient("netlist", NMcreate, NMdelete,
 	NMredisplay, NMcommand, (Void (*)()) NULL,
-	NMCheckWritten, NMreposition, NMcommands,
+	NMCheckWritten, NMreposition,
 	(GrGlyph *) NULL);
     DBWHLAddClient(NMRedrawPoints);
     DBWHLAddClient(NMRedrawCell);
     DBWAddButtonHandler("netlist", NMButtonProc, STYLE_CURS_NET, doc);
     NMUndoInit();
+
+    /* Register commands with the client */
+
+    WindAddCommand(NMClientID,
+	"add term1 term2         add term1 to net of term2",
+	NMCmdAdd, FALSE);
+    WindAddCommand(NMClientID,
+	"cleanup                 interactively cleanup netlist",
+	NMCmdCleanup, FALSE);
+    WindAddCommand(NMClientID,
+	"cull                    remove fully-wired nets from the current netlist",
+	NMCmdCull, FALSE);
+    WindAddCommand(NMClientID,
+	"dnet name name ...      delete the net(s) containing name(s)\n\
+                        or current net if no name(s) given",
+	NMCmdDnet, FALSE);
+    WindAddCommand(NMClientID,
+	"dterm name name ...     delete terminals from nets",
+	NMCmdDterm, FALSE);
+    WindAddCommand(NMClientID,
+	"extract                 generate net for terminals connected to box",
+	NMCmdExtract, FALSE);
+    WindAddCommand(NMClientID,
+	"find pattern [layers]   find all occurrences of any labels matching\n\
+                        pattern beneath the box (on layers if specified)\n\
+                        and leave as feedback",
+	NMCmdFindLabels, FALSE);
+    WindAddCommand(NMClientID,
+	"flush [netlist]         flush changes to netlist (current list default)",
+	NMCmdFlush, FALSE);
+    WindAddCommand(NMClientID,
+	"join term1 term2        join the nets containing term1 and term2",
+	NMCmdJoin, FALSE);
+    WindAddCommand(NMClientID,
+	"netlist [name]          switch current netlist to name.net (default\n\
+			is edit cell name)",
+	NMCmdNetlist, FALSE);
+    WindAddCommand(NMClientID,
+	"print [name]            print all terminals in name, or in current net\n\
+			if no name given",
+	NMCmdPrint, FALSE);
+    WindAddCommand(NMClientID,
+	"ripup [netlist]         ripup edit cell paint connected to paint under\n\
+			box, or ripup current netlist if \"netlist\"\n\
+                        typed as argument",
+	NMCmdRipup, FALSE);
+    WindAddCommand(NMClientID,
+	"savenetlist [file]      write out current netlist",
+	NMCmdSavenetlist, FALSE);
+    WindAddCommand(NMClientID,
+	"shownet                 highlight edit cell paint connected to paint\n\
+			under box",
+	NMCmdShownet, FALSE);
+    WindAddCommand(NMClientID,
+	"showterms               generate feedback for all terminals in netlist",
+	NMCmdShowterms, FALSE);
+    WindAddCommand(NMClientID,
+	"trace [name]            highlight material connected to a net's \n\
+                        terminals (use current net if no name given)",
+	NMCmdTrace, FALSE);
+    WindAddCommand(NMClientID,
+	"verify                  make sure current netlist is correctly wired",
+	NMCmdVerify, FALSE);
+    WindAddCommand(NMClientID,
+	"writeall                write out all modified netlists",
+	NMCmdWriteall, FALSE);
 }
+
 
 /*
  * ----------------------------------------------------------------------------
