@@ -24,10 +24,10 @@
 
 #ifndef	_GEOMETRY
     err1 = Need_to_include_utils/geometry_header;
-#endif	_GEOMETRY
+#endif	/* _GEOMETRY */
 #ifndef _WINDOWS
     err2 = Need_to_include_windows/windows_header;
-#endif	_WINDOWS
+#endif	/* _WINDOWS */
 
 /* The declarations below define the elements displayed in the
  * net-list menu.  There are three kinds of things:
