@@ -21,10 +21,10 @@
 
 #ifndef	_MAGIC
 int err0 = Need_to_include_misc/magic_header;
-#endif	_MAGIC
+#endif	/* _MAGIC */
 #ifndef	_GEOMETRY
 int err1 = Need_to_include_utils/geometry_header;
-#endif	_GEOMETRY
+#endif	/* _GEOMETRY */
 
 /* system V machines lack vfont.h, so include the defs below. */
 #if  !defined(SYSV) && !defined(__FreeBSD__) && !defined(__NetBSD__) && !defined(CYGWIN) && !defined(macosx)
@@ -44,7 +44,7 @@ struct dispatch {
         char up,down,left,right;
         short width;
 };
-#endif	SYSV
+#endif	/* SYSV */
 
 /* The structure below is used for raster output, e.g. to a Versatec
  * printer.  It defines a rectangular raster area in which to fill
