@@ -796,7 +796,7 @@ PlotLoadStyles()
 	newsec = FALSE;
       }
       else {
-	if (sscanf (buf, "%d %o %o %o %40s %d %c %126s",
+	if (sscanf (buf, "%d %d %d %d %40s %d %c %126s",
 		    &ord, &mask, &color, &outline, fill, &stipple,
 		    &shortname, longname) !=  8) {
 	  goto err;
