@@ -638,8 +638,8 @@ plotPSPaint(tile, cxp)
     if (tileArea.r_ybot > TiPlaneRect.r_ybot)
     {
 	edge.r_ybot = edge.r_ytop = tileArea.r_ybot;
-	for (neighbor = tile->ti_lb; LEFT(neighbor) < tileArea.r_xtop;
-		neighbor = neighbor->ti_tr)
+	for (neighbor = LB(tile); LEFT(neighbor) < tileArea.r_xtop;
+		neighbor = TR(neighbor))
 	{
 #ifdef NONMANHATTAN
 	    ntype = TiGetTopType(neighbor);
@@ -669,8 +669,8 @@ searchleft:
     if (tileArea.r_xbot > TiPlaneRect.r_xbot)
     {
 	edge.r_xbot = edge.r_xtop = tileArea.r_xbot;
-	for (neighbor = tile->ti_bl; BOTTOM(neighbor) < tileArea.r_ytop;
-		neighbor = neighbor->ti_rt)
+	for (neighbor = BL(tile); BOTTOM(neighbor) < tileArea.r_ytop;
+		neighbor = RT(neighbor))
 	{
 #ifdef NONMANHATTAN
  	    ntype = TiGetRightType(neighbor);
@@ -698,8 +698,8 @@ searchtop:
     if (tileArea.r_ytop < TiPlaneRect.r_ytop)
     {
 	edge.r_ybot = edge.r_ytop = tileArea.r_ytop;
-	for (neighbor = tile->ti_rt; RIGHT(neighbor) > tileArea.r_xbot;
-		neighbor = neighbor->ti_bl)
+	for (neighbor = RT(tile); RIGHT(neighbor) > tileArea.r_xbot;
+		neighbor = BL(neighbor))
 	{
 #ifdef NONMANHATTAN
 	    ntype = TiGetBottomType(neighbor);
@@ -727,8 +727,8 @@ searchright:
     if (tileArea.r_xtop < TiPlaneRect.r_xtop)
     {
 	edge.r_xbot = edge.r_xtop = tileArea.r_xtop;
-	for (neighbor = tile->ti_tr; TOP(neighbor) > tileArea.r_ybot;
-		neighbor = neighbor->ti_lb)
+	for (neighbor = TR(tile); TOP(neighbor) > tileArea.r_ybot;
+		neighbor = LB(neighbor))
 	{
 #ifdef NONMANHATTAN
 	    ntype = TiGetLeftType(neighbor);
