@@ -21,7 +21,7 @@
 
 #ifndef lint
 static char rcsid[]="$Header: /ufs/repository/magic/plot/plotPixels.c,v 1.3 2001/01/12 22:13:10 jsolomon Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <errno.h>
 #include <stdio.h>
@@ -45,9 +45,6 @@ static char rcsid[]="$Header: /ufs/repository/magic/plot/plotPixels.c,v 1.3 2001
 #include "graphics/graphics.h"	
 #include "dbwind/dbwind.h"
     
-/* Library imports: */
-extern char *mktemp();
-
 /* forward declarations */
 
 void PlotPixFatLine();
@@ -821,7 +818,7 @@ PlotPixels(scx, layers, xMask, width)
     char fileName[200], answer[32];
     float mBytes;
     Transform tinv;
-    int action;
+    int action, result;
     FILE *file;
 
     /*
@@ -923,7 +920,12 @@ PlotPixels(scx, layers, xMask, width)
 
     sprintf(fileName, "%s/magicPlot-%d-%d-XXXXXX", PlotTempDirectory,
 	    PlotPixWidth, PlotPixHeight*swathsDown);
-    mktemp(fileName);
+    result = mkstemp(fileName);
+    if (result == -1)
+    {
+	TxError("Couldn't create temporary filename for %s\n", fileName);
+	return;
+    }
     file = PaOpen(fileName, "w", (char *) NULL, (char *) NULL, (char *) NULL,
 		  (char **) NULL);
     if (file == NULL)
@@ -1342,19 +1344,19 @@ PlotDumpPixRaster(pr, file)
 	if(putc(r,file)==EOF)	/* red */
 	{
 	    TxError("I/O error in writing PixRaster file:  %s.\n",
-		    STRERROR(errno));
+		    strerror(errno));
 	    return 1;
 	}
 	if(putc(g,file)==EOF) /* green */
 	{
 	    TxError("I/O error in writing PixRaster file:  %s.\n",
-		    STRERROR(errno));
+		    strerror(errno));
 	    return 1;
 	}
 	if(putc(b,file)==EOF) /* blue */
 	{
 	    TxError("I/O error in writing PixRaster file:  %s.\n",
-		    STRERROR(errno));
+		    strerror(errno));
 	    return 1;
 	}
     }
