@@ -20,7 +20,7 @@
 
 #ifndef lint
 static char rcsid[]="$Header: /ufs/repository/magic/plow/PlowMain.c,v 1.3 2001/06/15 15:33:39 tim Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <sys/time.h>
 #include <sys/resource.h>
@@ -43,11 +43,7 @@ static char rcsid[]="$Header: /ufs/repository/magic/plow/PlowMain.c,v 1.3 2001/0
 #include "signals/signals.h"
 #include "main/main.h"
 
-#ifdef	SYSV
-# define	NO_RUSAGE
-#endif
-
-#ifdef	macII
+#if defined(SYSV) || defined(macosx)
 # define	NO_RUSAGE
 #endif
 
@@ -480,7 +476,7 @@ Plow(def, userRect, layers, direction)
 #ifdef	COUNTWIDTHCALLS
     extern int plowWidthNumCalls;
     extern int plowWidthNumChoices;
-#endif	COUNTWIDTHCALLS
+#endif	/* COUNTWIDTHCALLS */
     TileTypeBitMask lc;
     Rect changedArea;
     bool firstTime;
@@ -512,7 +508,7 @@ Plow(def, userRect, layers, direction)
     TxPrintf("Choices = %d Calls = %d Choices/Call = %.1f\n",
 	plowWidthNumChoices, plowWidthNumCalls,
 	((double) plowWidthNumChoices) / ((double) plowWidthNumCalls));
-#endif	COUNTWIDTHCALLS
+#endif	/* COUNTWIDTHCALLS */
     return (firstTime);
 }
 
@@ -1023,7 +1019,7 @@ plowSelPaintPlow(rect, type, distance)
     plowLHS.r_xtop = plowLHS.r_xbot + distance;
 #ifdef	notdef
     plowAtomize(DBPlane(type), &plowLHS, plowSelPaintAdd, (ClientData) NULL);
-#endif	notdef
+#endif	/* notdef */
     plowLHS.r_xbot--;
     plowSrShadow(DBPlane(type), &plowLHS, DBZeroTypeBits,
 			plowInitialPaint, (ClientData) plowLHS.r_xtop);
@@ -1033,7 +1029,7 @@ plowSelPaintPlow(rect, type, distance)
     plowRHS.r_xtop += distance;
 #ifdef	notdef
     plowAtomize(DBPlane(type), &plowRHS, plowSelPaintAdd, (ClientData) NULL);
-#endif	notdef
+#endif	/* notdef */
     plowRHS.r_xbot--;
     TTMaskSetOnlyType(&mask, type);
     plowSrShadow(DBPlane(type), &plowRHS, mask,
