@@ -22,9 +22,10 @@
 
 #ifndef lint
 static char rcsid[]="$Header: /ufs/repository/magic/plow/PlowRandom.c,v 1.2 2001/01/12 22:13:11 jsolomon Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
+#include <stdlib.h>
 #include <sys/types.h>
 #include <sys/file.h>
 #include <sys/stat.h>
@@ -80,7 +81,7 @@ PlowRandomTest(def)
     static char *temptemp = "/tmp/PlowTempaXXXXX";
     char goodName[64], tempName[64], goodExt[64], tempExt[64];
     char command[256];
-#endif	notdef
+#endif	/* notdef */
     static int dirs[] = { GEO_NORTH, GEO_SOUTH, GEO_EAST, GEO_WEST };
     static char *dirnames[] = { "up", "down", "right", "left" };
     Rect plowRect;
@@ -89,8 +90,8 @@ PlowRandomTest(def)
 #ifdef	notdef
     strcpy(goodName, tempgood);
     strcpy(tempName, temptemp);
-    mktemp(goodName);
-    mktemp(tempName);
+    mkstemp(goodName);
+    mkstemp(tempName);
     sprintf(goodExt, "%s.ext", goodName);
     sprintf(tempExt, "%s.ext", tempName);
 
@@ -98,7 +99,7 @@ PlowRandomTest(def)
     ExtCell(def, goodName);
     (void) sprintf(command, "sedplow %s", goodExt);
     system(command);
-#endif	notdef
+#endif	/* notdef */
 
     /* Repeatedly try to break plowing */
     while (!SigInterruptPending)
@@ -142,10 +143,10 @@ PlowRandomTest(def)
 			plowRect.r_xtop, plowRect.r_ytop);
 	    TxFlush();
 	}
-#endif	notdef
+#endif	/* notdef */
 
 	/* Make sure there's always something to undo */
-	DBPutLabel(def, &def->cd_bbox, -1, "dummylabel", TT_SPACE);
+	DBPutLabel(def, &def->cd_bbox, -1, "dummylabel", TT_SPACE, 0);
 
 	/* Undo */
 	UndoBackward(1);
@@ -154,7 +155,7 @@ PlowRandomTest(def)
 #ifdef	notdef
     (void) unlink(goodExt);
     (void) unlink(tempExt);
-#endif	notdef
+#endif	/* notdef */
 }
 
 /*
@@ -252,7 +253,7 @@ plowGenRandom(lo, hi)
     int r = rand();
 #else
     int r = random();
-#endif	SYSV
+#endif	/* SYSV */
 
     return ((r % range) + lo);
 }
