@@ -19,7 +19,7 @@
 
 #ifndef lint
 static char rcsid[]="$Header: /ufs/repository/magic/plow/PlowSearch.c,v 1.4 2001/06/15 17:14:05 tim Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #include "misc/magic.h"
