@@ -20,7 +20,7 @@
 
 #ifndef lint
 static char rcsid[]="$Header: /ufs/repository/magic/plow/PlowTech.c,v 1.2 2001/01/12 22:13:12 jsolomon Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #include "misc/magic.h"
@@ -51,10 +51,11 @@ TileTypeBitMask PlowDragTypes;		/* All types that drag along trailing
 					 * minimum-width material when they
 					 * move.
 					 */
-extern TileTypeBitMask PlowFixedTypes;	/* Fixed-width types (e.g, fets).
-					 * This one is declared extern because
-					 * it is used in select/selOps.c, so
-					 * it must exist outside of the module.
+TileTypeBitMask PlowFixedTypes;		/* Fixed-width types (e.g, fets).
+					 * Note that this is the one variable
+					 * used outside the plow module (see
+					 * select/selOps.c), for the "stretch"
+					 * command.
 					 */
 
 /*
@@ -227,7 +228,7 @@ plowWidthRule(argc, argv)
 	{
 	    for (j = 0; j < DBNumTypes; j++)
 	    {
-		if (SamePlane(i, j) && TTMaskHasType(&set, j))
+		if (DBTypesOnSamePlane(i, j) && TTMaskHasType(&set, j))
 		{
 		    MALLOC(PlowRule *, pr, sizeof (PlowRule));
 		    pr->pr_dist = distance;
@@ -326,7 +327,7 @@ plowSpacingRule(argc, argv)
     {
 	for (j = 0; j < DBNumTypes; j++)
 	{
-	    if (i == j || !SamePlane(i, j)) continue;
+	    if (i == j || !DBTypesOnSamePlane(i, j)) continue;
 
 	    /* LHS is an element of set1 and RHS is an element of setR */
 	    if (TTMaskHasType(&set1, i) && TTMaskHasType(&setR, j))
@@ -817,7 +818,13 @@ PlowTechLine(sectionName, argc, argv)
 void
 PlowTechFinal()
 {
-    PlowContactTypes = DBImageBits;
+    TileType t;
+
+    TTMaskZero(&PlowContactTypes);
+    for (t = TT_TECHDEPBASE; t < DBNumTypes; t++)
+	if (DBIsContact(t))
+	    TTMaskSetType(&PlowContactTypes, t);
+
     TTMaskSetMask(&PlowFixedTypes, &PlowContactTypes);
 }
 
