@@ -19,7 +19,7 @@
 
 #ifndef lint
 static char rcsid[]="$Header: /ufs/repository/magic/plow/PlowWidth.c,v 1.3 2001/06/15 15:33:40 tim Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #include "misc/magic.h"
@@ -41,7 +41,7 @@ int plowInitWidthBackFunc(), plowWidthBackFunc();
 #ifdef	COUNTWIDTHCALLS
 int plowWidthNumCalls = 0;
 int plowWidthNumChoices = 0;
-#endif	COUNTWIDTHCALLS
+#endif	/* COUNTWIDTHCALLS */
 
 /*
  * ----------------------------------------------------------------------------
@@ -82,7 +82,7 @@ plowFindWidth(edge, types, bbox, prect)
     TTMaskCom2(&ctypes, &types);
 #ifdef	COUNTWIDTHCALLS
     plowWidthNumCalls++;
-#endif	COUNTWIDTHCALLS
+#endif	/* COUNTWIDTHCALLS */
 
     /*
      * Start with wc.wc_area extending from the edge rightward to the
@@ -100,7 +100,7 @@ plowFindWidth(edge, types, bbox, prect)
     TxPrintf("Initial area: X: %d .. %d  Y: %d .. %d\n",
 	wc.wc_area.r_xbot, wc.wc_area.r_xtop,
 	wc.wc_area.r_ybot, wc.wc_area.r_ytop);
-#endif	DEBUGWIDTH
+#endif	/* DEBUGWIDTH */
     (void) DBSrPaintArea((Tile *) NULL,
 		plane, &wc.wc_area, &ctypes,
 		plowInitWidthFunc, (ClientData) &wc);
@@ -108,7 +108,7 @@ plowFindWidth(edge, types, bbox, prect)
     TxPrintf("After first search, area: X: %d .. %d  Y: %d .. %d\n",
 	wc.wc_area.r_xbot, wc.wc_area.r_xtop,
 	wc.wc_area.r_ybot, wc.wc_area.r_ytop);
-#endif	DEBUGWIDTH
+#endif	/* DEBUGWIDTH */
 
     /*
      * Repeatedly search wc.wc_area for tiles whose
@@ -125,7 +125,7 @@ plowFindWidth(edge, types, bbox, prect)
 	TxPrintf("Next area: X: %d .. %d  Y: %d .. %d\n",
 	    wc.wc_area.r_xbot, wc.wc_area.r_xtop,
 	    wc.wc_area.r_ybot, wc.wc_area.r_ytop);
-#endif	DEBUGWIDTH
+#endif	/* DEBUGWIDTH */
 	if (wc.wc_area.r_xbot == wc.wc_area.r_xtop)
 	    break;
     }
@@ -135,7 +135,7 @@ plowFindWidth(edge, types, bbox, prect)
     y = wc.wc_area.r_ytop - wc.wc_area.r_ybot;
 #ifdef	DEBUGWIDTH
     TxPrintf("Width = %d\n", MIN(x, y));
-#endif	DEBUGWIDTH
+#endif	/* DEBUGWIDTH */
     return (MIN(x, y));
 }
 
@@ -178,7 +178,7 @@ plowInitWidthFunc(tile, wc)
 #ifdef	DEBUGWIDTH
     TxPrintf("Tile type = %s, edgeHeight = %d, upperBound = %d\n",
 	DBTypeLongName(TiGetTypeExact(tile)), edge->e_ytop-edge->e_ybot, upperBound);
-#endif	DEBUGWIDTH
+#endif	/* DEBUGWIDTH */
     wc->wc_area.r_ytop = MAX(edge->e_ybot + upperBound, edge->e_ytop);
     wc->wc_area.r_ybot = MIN(edge->e_ytop - upperBound, edge->e_ybot);
     wc->wc_area.r_xtop = LEFT(tile);
@@ -247,7 +247,7 @@ plowWidthFunc(tile, wc)
      */
 #ifdef	COUNTWIDTHCALLS
     plowWidthNumChoices++;
-#endif	COUNTWIDTHCALLS
+#endif	/* COUNTWIDTHCALLS */
     if (BOTTOM(tile) >= edge->e_ytop)
     {
 	yw = BOTTOM(tile) - wc->wc_area.r_ybot;
@@ -352,7 +352,7 @@ plowFindWidthBack(edge, types, bbox, prect)
     TxPrintf("Initial area: X: %d .. %d  Y: %d .. %d\n",
 	wc.wc_area.r_xbot, wc.wc_area.r_xtop,
 	wc.wc_area.r_ybot, wc.wc_area.r_ytop);
-#endif	DEBUGWIDTH
+#endif	/* DEBUGWIDTH */
     (void) DBSrPaintArea((Tile *) NULL,
 		plane, &wc.wc_area, &ctypes,
 		plowInitWidthBackFunc, (ClientData) &wc);
@@ -360,7 +360,7 @@ plowFindWidthBack(edge, types, bbox, prect)
     TxPrintf("After first search, area: X: %d .. %d  Y: %d .. %d\n",
 	wc.wc_area.r_xbot, wc.wc_area.r_xtop,
 	wc.wc_area.r_ybot, wc.wc_area.r_ytop);
-#endif	DEBUGWIDTH
+#endif	/* DEBUGWIDTH */
 
     /*
      * Repeatedly search wc.wc_area for tiles whose
@@ -377,7 +377,7 @@ plowFindWidthBack(edge, types, bbox, prect)
 	TxPrintf("Next area: X: %d .. %d  Y: %d .. %d\n",
 	    wc.wc_area.r_xbot, wc.wc_area.r_xtop,
 	    wc.wc_area.r_ybot, wc.wc_area.r_ytop);
-#endif	DEBUGWIDTH
+#endif	/* DEBUGWIDTH */
 	if (wc.wc_area.r_xbot == wc.wc_area.r_xtop)
 	    break;
     }
@@ -387,7 +387,7 @@ plowFindWidthBack(edge, types, bbox, prect)
     y = wc.wc_area.r_ytop - wc.wc_area.r_ybot;
 #ifdef	DEBUGWIDTH
     TxPrintf("Width = %d\n", MIN(x, y));
-#endif	DEBUGWIDTH
+#endif	/* DEBUGWIDTH */
     return (MIN(x, y));
 }
 
@@ -430,7 +430,7 @@ plowInitWidthBackFunc(tile, wc)
 #ifdef	DEBUGWIDTH
     TxPrintf("Tile type = %s, edgeHeight = %d, upperBound = %d\n",
 	DBTypeLongName(TiGetTypeExact(tile)), edge->e_ytop-edge->e_ybot, upperBound);
-#endif	DEBUGWIDTH
+#endif	/* DEBUGWIDTH */
     wc->wc_area.r_ytop = MAX(edge->e_ybot + upperBound, edge->e_ytop);
     wc->wc_area.r_ybot = MIN(edge->e_ytop - upperBound, edge->e_ybot);
     wc->wc_area.r_xbot = RIGHT(tile);
