@@ -20,7 +20,7 @@
 
 #ifndef lint
 static char rcsid[]="$Header: /ufs/repository/magic/plow/PlowYank.c,v 1.3 2001/06/15 15:33:40 tim Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #include "misc/magic.h"
@@ -385,10 +385,12 @@ plowUpdateLabels(yankDef, origDef, origArea)
 	    lu.lu_rect.r_xbot += lu.lu_adjust;
 	    lu.lu_rect.r_xtop += lu.lu_adjust;
 	    DBUndoEraseLabel(origDef, &origLab->lab_rect, origLab->lab_pos,
-				origLab->lab_text, origLab->lab_type);
+				origLab->lab_text, origLab->lab_type,
+				origLab->lab_flags);
 	    GeoTransRect(&plowInverseTrans, &lu.lu_rect, &origLab->lab_rect);
 	    DBUndoPutLabel(origDef, &origLab->lab_rect, origLab->lab_pos,
-				origLab->lab_text, origLab->lab_type);
+				origLab->lab_text, origLab->lab_type,
+				origLab->lab_flags);
 	    plowLabelsChanged = TRUE;
 	}
     }
