@@ -5,7 +5,7 @@
 
 #ifndef lint
 static char rcsid[] = "$Header: /ufs/repository/magic/resis/ResConDCS.c,v 1.3 2001/05/08 18:29:20 tim Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #include <ctype.h>
@@ -21,7 +21,6 @@ static char rcsid[] = "$Header: /ufs/repository/magic/resis/ResConDCS.c,v 1.3 20
 #include "tiles/tile.h"
 #include "utils/hash.h"
 #include "database/database.h"
-#include "database/databaseInt.h"
 #include "utils/malloc.h"
 #include "textio/textio.h"
 #include "extract/extract.h"
@@ -31,19 +30,33 @@ static char rcsid[] = "$Header: /ufs/repository/magic/resis/ResConDCS.c,v 1.3 20
 #include "dbwind/dbwind.h"
 #include "tech/tech.h"
 #include "textio/txcommands.h"
-#include	"resis/resis.h"
+#include "resis/resis.h"
+
+typedef struct
+{
+    Rect		area;		/* Area to process */
+    TileTypeBitMask	*connectMask;	/* Connection mask for search */
+} conSrArea;
 
 struct conSrArg2
 {
-    Rect csa2_area;
-    int csa2_type;
-    struct conSrArg2 *csa2_next;
+    CellUse             *csa2_use;      /* Destination use */
+    TileTypeBitMask     *csa2_connect;  /* Table indicating what connects
+                                         * to what.
+                                         */
+    Rect                *csa2_bounds;   /* Area that limits the search */
+
+    conSrArea		*csa2_list;	/* List of areas to process */
+    int			csa2_top;	/* Index of next area to process */
+    int			csa2_size;	/* Max. number bins in area list */
 };
 
+#define CSA2_LIST_START_SIZE 256
+
 extern int dbcUnconnectFunc();
 extern int dbcConnectFuncDCS();
 extern int resSubSearchFunc();
-extern int dbcReturn;
+
 static ResTranTile  *TransList = NULL;
 static TileTypeBitMask	DiffTypeBitMask;
 TileTypeBitMask	ResSubsTypeBitMask;
@@ -72,22 +85,26 @@ dbcConnectFuncDCS(tile, cx)
     TreeContext *cx;
 
 {
-    struct conSrArg2 **pHead = (struct conSrArg2 **) cx->tc_filter->tf_arg;
-    struct conSrArg2 *newCsa2;
-    Rect tileArea, *srArea,tranArea;
+    struct conSrArg2	*csa2;
+    Rect 		tileArea, *srArea, tranArea, newarea;
     ResTranTile		*thisTran;
-    register Tile	*tp;
-    register TileType	t2,t1;
+    TileTypeBitMask	notConnectMask, *connectMask;
+    Tile		*tp;
+    TileType		t2, t1, loctype, newtype;
     SearchContext	*scx = cx->tc_scx;
+    int			pNum;
+    CellDef		*def;
 
     TiToRect(tile, &tileArea);
     srArea = &scx->scx_area;
-    if (((tileArea.r_xbot >= srArea->r_xtop-1) ||
-        (tileArea.r_xtop <= srArea->r_xbot+1)) &&
-	((tileArea.r_ybot >= srArea->r_ytop-1) ||
-	(tileArea.r_ytop <= srArea->r_ybot+1))) return 0;
 
-    MALLOC(struct conSrArg2 *, newCsa2, sizeof (struct conSrArg2));
+    if (((tileArea.r_xbot >= srArea->r_xtop-1) ||
+		(tileArea.r_xtop <= srArea->r_xbot+1)) &&
+		((tileArea.r_ybot >= srArea->r_ytop-1) ||
+		(tileArea.r_ytop <= srArea->r_ybot+1)))
+	return 0;
+ 
+
     
     t1 = TiGetType(tile);
     if TTMaskHasType(&DiffTypeBitMask,t1)
@@ -188,8 +205,11 @@ dbcConnectFuncDCS(tile, cx)
          }
     }
 #endif
-    GeoTransRect(&scx->scx_trans, &tileArea, &newCsa2->csa2_area);
-    newCsa2->csa2_type = TiGetTypeExact(tile);
+    GeoTransRect(&scx->scx_trans, &tileArea, &newarea);
+
+    csa2 = (struct conSrArg2 *)cx->tc_filter->tf_arg;
+    GeoClip(&newarea, csa2->csa2_bounds);
+    if (GEO_RECTNULL(&newarea)) return 0;
 
 #ifdef NONMANHATTAN
     /* Resolve geometric transformations on diagonally-split tiles */
@@ -197,7 +217,6 @@ dbcConnectFuncDCS(tile, cx)
     if (IsSplit(tile))
     {
 	int o1, o2, o3, dir, side;
-	TileType loctype = newCsa2->csa2_type;
 
 	o1 = ((scx->scx_trans.t_e > 0) || (scx->scx_trans.t_d > 0)) ? 1 : 0;
 	o2 = ((scx->scx_trans.t_a > 0) || (scx->scx_trans.t_b > 0)) ? 1 : 0;
@@ -206,20 +225,106 @@ dbcConnectFuncDCS(tile, cx)
 	side = SplitSide(tile) ^ o2 ^ (SplitDirection(tile) | o3);
 
 	if (side == SplitSide(tile))
-            newCsa2->csa2_type = TT_DIAGONAL | SplitLeftType(tile) |
+            newtype = TT_DIAGONAL | SplitLeftType(tile) |
                         (SplitRightType(tile) << 14);
 	else
-            newCsa2->csa2_type = TT_DIAGONAL | SplitRightType(tile) |
+            newtype = TT_DIAGONAL | SplitRightType(tile) |
                         (SplitLeftType(tile) << 14);
 
-	if (side) newCsa2->csa2_type |= TT_SIDE;
-	if (dir) newCsa2->csa2_type |= TT_DIRECTION;
+	if (dir) newtype |= TT_DIRECTION;
+	if (side)
+	{
+	    newtype |= TT_SIDE;
+	    loctype = (newtype & TT_RIGHTMASK) >> 14;
+	}
+	else
+	    loctype = newtype & TT_LEFTMASK;
+    }
+    else
+#endif
+    loctype = newtype = TiGetTypeExact(tile);
+
+    pNum = DBPlane(loctype);
+    connectMask = &csa2->csa2_connect[loctype];
+
+    if (DBIsContact(loctype))
+    {
+	TTMaskSetOnlyType(&notConnectMask, loctype);
+	TTMaskCom(&notConnectMask);
+    }
+    else
+    {
+	TTMaskCom2(&notConnectMask, connectMask);
+    }
+
+    def = csa2->csa2_use->cu_def;
+    if (DBSrPaintArea((Tile *) NULL, def->cd_planes[pNum],
+	    &newarea, &notConnectMask, dbcUnconnectFunc,
+	    (ClientData) connectMask) == 0)
+	return 0;
+
+#ifdef NONMANHATTAN
+    DBNMPaintPlane(def->cd_planes[pNum], newtype,
+		&newarea, DBStdPaintTbl(loctype, pNum),
+		(PaintUndoInfo *) NULL);
+#else
+    DBPaintPlane(def->cd_planes[pNum], &newarea,
+	    DBStdPaintTbl(loctype, pNum), (PaintUndoInfo *) NULL);
+#endif
+
+#ifdef NONMANHATTAN
+    /* Only extend those sides bordering the diagonal tile */
+
+    if (newtype & TT_DIAGONAL)
+    {
+	if (newtype & TT_SIDE)               /* right */
+	    newarea.r_xtop += 1;
+	else                                            /* left */
+	    newarea.r_xbot -= 1;
+	if (((newtype & TT_SIDE) >> 1)
+		== (newtype & TT_DIRECTION)) /* top */
+	    newarea.r_ytop += 1;
+	else                                            /* bottom */
+	    newarea.r_ybot -= 1;
+    }
+    else
+    {
+#endif
+    newarea.r_xbot -= 1;
+    newarea.r_ybot -= 1;
+    newarea.r_xtop += 1;
+    newarea.r_ytop += 1;
+#ifdef NONMANHATTAN
     }
 #endif
 
-    newCsa2->csa2_next = *pHead;
-    *pHead = newCsa2;
-    return dbcReturn;
+    /* Register the area and connection mask as needing to be processed */
+
+    if (++csa2->csa2_top == csa2->csa2_size)
+    {
+	/* Reached list size limit---need to enlarge the list	   */
+	/* Double the size of the list every time we hit the limit */
+
+	conSrArea *newlist;
+	int i, lastsize = csa2->csa2_size;
+
+	csa2->csa2_size *= 2;
+
+	newlist = (conSrArea *)mallocMagic(csa2->csa2_size * sizeof(conSrArea));
+	for (i = 0; i < lastsize; i++)
+	{
+	    newlist[i].area = csa2->csa2_list[i].area;
+	    newlist[i].connectMask = csa2->csa2_list[i].connectMask;
+	    
+	}
+	freeMagic((char *)csa2->csa2_list);
+	csa2->csa2_list = newlist;
+    }
+
+    csa2->csa2_list[csa2->csa2_top].area = newarea;
+    csa2->csa2_list[csa2->csa2_top].connectMask = connectMask;
+
+    return 0;
 }
 
 
@@ -234,9 +339,10 @@ dbcConnectFuncDCS(tile, cx)
  *
  *-------------------------------------------------------------------------
  */
-ResCalcPerimOverlap(trans,tile)
-	ResTranTile	*trans;
-	Tile		*tile;
+
+ResCalcPerimOverlap(trans, tile)
+    ResTranTile	*trans;
+    Tile	*tile;
 
 {
     Tile	*tp;
@@ -318,16 +424,21 @@ DBTreeCopyConnectDCS(scx, mask, xMask, connect, area, destUse)
     CellUse *destUse;
 
 {
-    SearchContext scx2;
-    TileTypeBitMask notConnectMask, *connectMask;
-    TileType loctype;
-    struct conSrArg2 *list, *current;
-    CellDef *def = destUse->cu_def;
-    Cvals		cdata;
-    int			pNum;
-    ResTranTile		*CurrentT;
     static int 		first = 1;
-    int			tran;
+    struct conSrArg2	csa2; 
+    int			tran, pNum;
+    TileTypeBitMask	*newmask;
+    ResTranTile		*CurrentT;
+    CellDef		*def = destUse->cu_def;
+
+    csa2.csa2_use = destUse;
+    csa2.csa2_bounds = area;
+    csa2.csa2_connect = connect;
+
+    csa2.csa2_size = CSA2_LIST_START_SIZE;
+    csa2.csa2_list = (conSrArea *)mallocMagic(CSA2_LIST_START_SIZE
+		* sizeof(conSrArea));
+    csa2.csa2_top = -1;
 
     if (first)
     {
@@ -341,124 +452,33 @@ DBTreeCopyConnectDCS(scx, mask, xMask, connect, area, destUse)
 	      		&(ExtCurStyle->exts_transSubstrateTypes[tran]));
 	 }
 	 first = 0;
-
     }
-    /* Get one initial tile and put it our list. */
 
     TransList = NULL;
-    list = NULL;
-    dbcReturn = 1;
-    (void) DBTreeSrTiles(scx, mask, xMask, dbcConnectFuncDCS, (ClientData) &list);
-    dbcReturn = 0;
-
-    scx2 = *scx;
-    while (list != NULL)
+    DBTreeSrTiles(scx, mask, xMask, dbcConnectFuncDCS, (ClientData) &csa2);
+    while (csa2.csa2_top >= 0)
     {
-	current = list;
-	list = current->csa2_next;
-
-	GeoClip(&current->csa2_area, area);
-	if (GEO_RECTNULL(&current->csa2_area)) goto endOfLoop;
-
-#ifdef NONMANHATTAN
-	if (current->csa2_type & TT_DIAGONAL)
-	{
-	    loctype = (current->csa2_type & TT_SIDE) ?
-			((current->csa2_type & TT_RIGHTMASK) >> 14) :
-			current->csa2_type & TT_LEFTMASK;
-	}
-else
-#endif
-	loctype = current->csa2_type;
-
-	connectMask = &connect[loctype];
-
-        if (TTMaskHasType(&DBContactBits, loctype))
-        {
-             TTMaskSetOnlyType(&notConnectMask, loctype);
-             TTMaskCom(&notConnectMask);
-        }
-        else
-        {
-             TTMaskCom2(&notConnectMask, connectMask);
-        }
-	pNum = DBPlane(loctype);
-
-	cdata.area = &(current->csa2_area);
-	cdata.mask = connectMask;
-
-	if (DBSrPaintArea((Tile *) NULL, def->cd_planes[pNum],
-	    &current->csa2_area, &notConnectMask,
-	    dbcUnconnectFunc, (ClientData) &cdata) != 0)
-	{
-
-	    scx2.scx_area = current->csa2_area;
-
-#ifdef NONMANHATTAN
-	    /* Only extend those sides bordering the diagonal tile */
-
-	    if (current->csa2_type & TT_DIAGONAL)
-	    {
-		if (current->csa2_type & TT_SIDE)               /* right */
-		    scx2.scx_area.r_xtop += 1;
-		else                                            /* left */
-		    scx2.scx_area.r_xbot -= 1;
-		if (((current->csa2_type & TT_SIDE) >> 1)
-			== (current->csa2_type & TT_DIRECTION)) /* top */
-		    scx2.scx_area.r_ytop += 1;
-		else                                            /* bottom */
-		    scx2.scx_area.r_ybot -= 1;
-	    }
-	    else
-	    {
-#endif
-	    scx2.scx_area.r_xbot -= 1;
-	    scx2.scx_area.r_ybot -= 1;
-	    scx2.scx_area.r_xtop += 1;
-	    scx2.scx_area.r_ytop += 1;
-#ifdef NONMANHATTAN
-            }
-#endif
-	    (void) DBTreeSrTiles(&scx2, connectMask,
-		xMask, dbcConnectFuncDCS, (ClientData) &list);
-	}
-
-#ifdef NONMANHATTAN
-	DBNMPaintPlane(def->cd_planes[pNum], current->csa2_type,
-		&current->csa2_area, DBStdPaintTbl(loctype, pNum),
-		(PaintUndoInfo *) NULL);
-#else
-	DBPaintPlane(def->cd_planes[pNum], &current->csa2_area,
-	    DBStdPaintTbl(loctype, pNum), (PaintUndoInfo *) NULL);
-#endif
-	endOfLoop: FREE((char *) current);
+	newmask = csa2.csa2_list[csa2.csa2_top].connectMask;
+	scx->scx_area = csa2.csa2_list[csa2.csa2_top].area;
+	csa2.csa2_top--;
+	DBTreeSrTiles(scx, newmask, xMask, dbcConnectFuncDCS, (ClientData) &csa2);
     }
+    freeMagic((char *)csa2.csa2_list);
 
     for (CurrentT = TransList; CurrentT != NULL; CurrentT=CurrentT->nextTran)
     {
-	TileType t = CurrentT->type;
-	TileType nt;
-	LayerInfo	*lp = &dbLayerInfo[t];
-	int origin;
-	     
+	TileType  t = CurrentT->type;
+	TileType  nt;
+	TileTypeBitMask *residues = DBResidueMask(t);
 
-	pNum = DBPlane(t);
-	DBPaintPlane(def->cd_planes[pNum], &CurrentT->area,
-			DBStdPaintTbl(t, pNum), (PaintUndoInfo *) NULL);
-
-	for (origin = 0; origin != lp->l_nresidues; origin++)
+	for (nt = TT_TECHDEPBASE; nt < DBNumTypes; nt++)
 	{
-	    if (lp->l_images[origin] == t) break;
-	}
-	if ((nt=lp->l_images[origin+1]) != TT_SPACE)
-	{
-	    DBPaintPlane(def->cd_planes[pNum+1], &CurrentT->area,
-			DBStdPaintTbl(nt, pNum+1), (PaintUndoInfo *) NULL);
-	}
-	if (origin && (nt=lp->l_images[origin-1]) != TT_SPACE)
-	{
-	    DBPaintPlane(def->cd_planes[pNum-1], &CurrentT->area,
-		DBStdPaintTbl(nt, pNum-1), (PaintUndoInfo *) NULL);
+	    if (TTMaskHasType(residues, nt))
+	    {
+		pNum = DBPlane(nt);
+		DBPaintPlane(def->cd_planes[pNum], &CurrentT->area,
+			DBStdPaintTbl(nt, pNum), (PaintUndoInfo *) NULL);
+	    }
 	}
     }
 
@@ -495,18 +515,6 @@ resSubSearchFunc(tile,cx)
      TiToRect(tile, &tranArea);
      MALLOC(ResTranTile *, thisTran, sizeof(ResTranTile));
      GeoTransRect(&cx->tc_scx->scx_trans, &tranArea, &thisTran->area);
-     if (t >= DBNumUserLayers)
-     {
-     	  Tile *tp;
-	  int	pNum = DBPlane(t);
-	  LayerInfo	*lp = &dbLayerInfo[t];
-
-	  if (lp->l_rbelow == TT_SPACE) pNum++;
-	  else pNum--;
-     	  tp = cx->tc_scx->scx_use->cu_def->cd_planes[pNum]->pl_hint;
-	  GOTOPOINT(tp,&tile->ti_ll);
-	  t = TiGetType(tp);
-     }
      thisTran->type = t;
      thisTran->nextTran = TransList;
      TransList = thisTran;
