@@ -769,10 +769,10 @@ ResTriangleCheck(resptr)
       {
       	   ResRemoveFromQueue(node2,pendingList);
       }
-      if (node2->rn_client != NULL)
+      if (node2->rn_client != (ClientData)NULL)
       {
 	    FREE((char *)node2->rn_client);
-	    node2->rn_client = NULL;
+	    node2->rn_client = (ClientData)NULL;
       }
       {
 	    node2->rn_re = (resElement *) MINFINITY;
@@ -916,10 +916,10 @@ ResCleanNode(resptr,junk,homelist1,homelist2)
      }
      if (junk == TRUE)
      {
-	  if (resptr->rn_client != NULL)
+	  if (resptr->rn_client != (ClientData)NULL)
 	  {
 	       FREE((char *)resptr->rn_client);
-	       resptr->rn_client = NULL;
+	       resptr->rn_client = (ClientData)NULL;
 	  }
 	  while (resptr->rn_te != NULL)
           {
