@@ -1,7 +1,7 @@
 
 #ifndef lint
 static char rcsid[] = "$Header: /ufs/repository/magic/resis/ResRex.c,v 1.3 2001/08/23 13:48:56 tim Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -73,9 +73,9 @@ FILE	*ResExtFile;
 FILE	*ResLumpFile;
 FILE	*ResGeomFile;
 
+int	ResPortIndex;	/* Port ordering to backannotate into magic */
+
 /* external declarations */
-extern SegList    *extGetSubcircuitBoundary();
-extern bool	  ExtSegCoincident();
 extern ResSimNode *ResInitializeNode();
 extern CellUse	  *CmdGetSelectedCell();
 
@@ -126,8 +126,8 @@ ExtResisForDef(celldef, resisdata)
 	    ResCheckPorts(celldef);
 
 	    /* Extract networks for nets that require it. */
-	    if (!(ResOptionsFlags & ResOpt_Geometry) || (celldef->cd_flags
-			& CDSUBCIRCUIT))
+	    if (!(ResOptionsFlags & ResOpt_Geometry) || 
+			DBIsSubcircuit(celldef))
 		ResCheckSimNodes(celldef, resisdata);
 
 	    if (ResOptionsFlags & ResOpt_Stat)
@@ -434,7 +434,7 @@ CmdExtResis(win, cmd)
     resisdata.mainDef = mainDef;
 
     /* Do subcircuits (if any) first */
-    (void) DBCellSrDefs(CDSUBCIRCUIT, resSubcircuitFunc, (ClientData) &resisdata);
+    (void) DBCellSrDefs(0, resSubcircuitFunc, (ClientData) &resisdata);
 
     ExtResisForDef(mainDef, &resisdata);
 
@@ -466,7 +466,8 @@ resSubcircuitFunc(cellDef, rdata)
     ResisData *rdata;
 {
     if (cellDef != rdata->mainDef)
-	ExtResisForDef(cellDef, rdata);
+	if (DBIsSubcircuit(cellDef))
+	    ExtResisForDef(cellDef, rdata);
     return 0;
 }
 
@@ -494,20 +495,32 @@ void
 ResCheckPorts(cellDef)
     CellDef *cellDef;
 {
-    SegList *subckt;
     Point portloc;
     Label *lab;
     HashEntry *entry;
     ResSimNode *node;
 
-    if (!(cellDef->cd_flags & CDSUBCIRCUIT)) return;
-
-    subckt = extGetSubcircuitBoundary(cellDef);
-
     for (lab = cellDef->cd_labels; lab; lab = lab->lab_next)
     {
-	if (ExtSegCoincident(&(lab->lab_rect), subckt, &portloc))
+	if (lab->lab_flags & PORT_DIR_MASK)
 	{
+	    /* Get drivepoint from the port connection direction(s) */
+	    /* NOTE:  This is not rigorous! */
+
+	    if (lab->lab_flags & (PORT_DIR_NORTH | PORT_DIR_SOUTH))
+		portloc.p_x = (lab->lab_rect.r_xbot + lab->lab_rect.r_xtop) >> 1;
+	    else if (lab->lab_flags & (PORT_DIR_EAST | PORT_DIR_WEST))
+		portloc.p_y = (lab->lab_rect.r_ybot + lab->lab_rect.r_ytop) >> 1;
+
+	    if (lab->lab_flags & PORT_DIR_NORTH)
+		portloc.p_y = lab->lab_rect.r_ytop;
+	    if (lab->lab_flags & PORT_DIR_SOUTH)
+		portloc.p_y = lab->lab_rect.r_ybot;
+	    if (lab->lab_flags & PORT_DIR_EAST)
+		portloc.p_x = lab->lab_rect.r_xtop;
+	    if (lab->lab_flags & PORT_DIR_WEST)
+		portloc.p_x = lab->lab_rect.r_xbot;
+
 	    entry = HashFind(&ResNodeTable, lab->lab_text);
 	    if ((node = (ResSimNode *) HashGetValue(entry)) != NULL)
 	    {
@@ -541,7 +554,6 @@ ResCheckPorts(cellDef)
 	    node->type = lab->lab_type;
 	}
     }
-    DBFreeLayerBoundary(subckt);
 }
  
 /*
@@ -594,7 +606,10 @@ ResCheckSimNodes(celldef, resisdata)
     }
     if (ResOptionsFlags & ResOpt_Geometry)
     {
-        ResGeomFile = PaOpen(outfile,"w",".fh",".",(char *) NULL, (char **) NULL);
+	char *geofilename;
+        ResGeomFile = PaOpen(outfile,"w",".fh",".",(char *) NULL, &geofilename);
+	TxPrintf("Writing FastHenry-format geometry file \"%s\"\n", geofilename);
+	ResPortIndex = 0;
     }
     else
     {
@@ -780,7 +795,8 @@ ResCheckSimNodes(celldef, resisdata)
 			(ResOptionsFlags & ResOpt_ExtractAll))
 		{
 		    resNodeNum = 0;
-		    failed3 += ResWriteExtFile(node, tol, rctol, &nidx, &eidx);
+		    failed3 += ResWriteExtFile(celldef, node, tol, rctol,
+				&nidx, &eidx);
 		}
 	    }
 #ifdef PARANOID
@@ -807,8 +823,25 @@ ResCheckSimNodes(celldef, resisdata)
 
     if (ResOptionsFlags & ResOpt_Geometry)
     {
+	Label *lab;
+
 	fprintf(ResGeomFile, "\n.freq fmin=%2.1g fmax=%2.1g\n",
 			resisdata->frequency, resisdata->frequency);
+
+	/*----------------------------------------------------------------------*/
+	/* Write (in comment lines) the order in which arguments are written	*/
+	/* to a SPICE subcircuit call when Magic runs ext2spice (exttospice).	*/
+	/* At present, it is the responsibility of the program that generates	*/
+	/* SPICE from FastHenry output to use this information to appropriately	*/
+	/* order the arguments in the ".subckt" definition.			*/
+	/*----------------------------------------------------------------------*/
+
+	fprintf(ResGeomFile, "\n* Order of arguments to SPICE subcircuit call:\n");
+	for (lab = celldef->cd_labels; lab != NULL; lab = lab->lab_next)
+	    if (lab->lab_flags & PORT_DIR_MASK)
+		fprintf(ResGeomFile, "* %d %s\n", lab->lab_flags & PORT_NUM_MASK,
+			lab->lab_text);
+
 	fprintf(ResGeomFile, "\n.end\n");
     }
 
@@ -1327,7 +1360,8 @@ ResAlignNodes(nodelist, reslist)
  */
 
 int
-ResWriteExtFile(node, tol, rctol, nidx, eidx)
+ResWriteExtFile(celldef, node, tol, rctol, nidx, eidx)
+    CellDef	*celldef;
     ResSimNode	*node;
     float	tol, rctol;
     int		*nidx, *eidx;
@@ -1371,8 +1405,9 @@ ResWriteExtFile(node, tol, rctol, nidx, eidx)
         }
 	if (ResOptionsFlags & ResOpt_Geometry)
 	{
-	    ResAlignNodes(ResNodeList, ResResList);
-	    ResPrintGeoNodes(ResGeomFile, ResNodeList, node->name, nidx);
+	    if (ResResList)
+		ResAlignNodes(ResNodeList, ResResList);
+	    ResPrintGeoNodes(ResGeomFile, ResNodeList, node->name, nidx, celldef);
 	    ResPrintGeoRects(ResGeomFile, ResResList, newname, eidx);
 	}
 	return 1;
