@@ -853,7 +853,7 @@ ResPruneTree(node,minTdi,nodelist1,nodelist2,resistorlist)
 	 node->rn_re->re_nextEl == NULL &&
 	 node->rn_re->re_thisEl->rr_connection2 == node)
      {
-     	  if (node->rn_client == NULL)
+     	  if (node->rn_client == (ClientData)NULL)
 	  {
 	       TxError("Internal Error in Tree Pruning: Missing TDi value.\n");
 	  }
