@@ -630,7 +630,7 @@ extern void			ResSortBreaks();
 	  (node)->rn_id=0;\
 	  (node)->rn_float.rn_area = 0.0;\
 	  (node)->rn_name = NULL;\
-	  (node)->rn_client = NULL;\
+	  (node)->rn_client = (ClientData)NULL;\
 	  (node)->rn_noderes = RES_INFINITY;\
 	  (node)->rn_je = NULL;\
 	  (node)->rn_status = FALSE;\
