@@ -17,7 +17,7 @@
 
 #ifndef lint
 static char rcsid[]="$Header: /ufs/repository/magic/router/rtrChannel.c,v 1.2 2001/01/12 22:13:18 jsolomon Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #include "misc/magic.h"
@@ -57,7 +57,7 @@ GCRChannel *RtrChannelList = NULL;
 float RtrEndConst = 1.0;
 #else
 float RtrEndConst;	/* Sun lint brain death */
-#endif	lint
+#endif	/* lint */
 
 /* Forward declarations */
 int rtrChannelObstacleMark();
@@ -538,7 +538,7 @@ RtrChannelDensity(ch)
 		(ch->gcr_width + 2) * sizeof (short));
     bcopy((char *) ch->gcr_dRowsByCol, (char *) ch->gcr_iRowsByCol,
 		(ch->gcr_length + 2) * sizeof (short));
-#endif	IDENSITY
+#endif	/* IDENSITY */
 
     /* Compute the maximum row and column density */
 
