@@ -18,7 +18,7 @@
 
 #ifndef lint
 static char rcsid[] = "$Header: /ufs/repository/magic/router/routeCmd.c,v 1.0 2001/03/29 19:50:05 tim Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #include <ctype.h>
@@ -779,3 +779,4 @@ CmdSeeFlags(w, cmd)
 	TxError("%s:  flag highlights turned off.\n", cmd->tx_argv[0]);
     }
 }
+
