@@ -175,7 +175,7 @@ rtrMakeChannel(tile, clipBox)
     Rect bbox;
 
     if (SigInterruptPending) return (1);
-    if (TiGetBody(tile) != NULL) return (0);
+    if (TiGetBody(tile) != (ClientData) NULL) return (0);
 
     entry = HashFind(&RtrTileToChannel, (char *) tile);
     ASSERT(HashGetValue(entry) == (char *) 0, "rtrMakeChannel");
