@@ -21,7 +21,7 @@
 
 #ifndef lint
 static char rcsid[]="$Header: /ufs/repository/magic/select/selEnum.c,v 1.3 2001/03/29 19:50:11 tim Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #include "misc/magic.h"
@@ -144,7 +144,7 @@ SelEnumPaint(layers, editOnly, foundNonEdit, func, clientData)
  * in the select cell.  Collect all of the sub-areas of this piece that
  * are also in the edit cell, then call the client function for each
  * one of them.  It's important to collect the pieces first, then call
- * the client:  if we call the client while the edit cell seardh is
+ * the client:  if we call the client while the edit cell search is
  * underway, the client might trash the tile plane underneath us.
  */
 
@@ -170,16 +170,49 @@ selEnumPFunc1(tile, arg)
 #endif
     arg->sea_type = TiGetType(tile);
 
-    /* If this tile is a contact's secondary image, ignore it:  the
-     * primary image will take care of everything (otherwise when
-     * the second image is processed, the contact might not be there
-     * anymore).
-     */
+    /* Only process contacts once:  only process the home plane */
+    /* image of any contact type.  Stacked contacts are treated	*/
+    /* as whichever residue (contact) is in its home plane.	*/
 
-    if (arg->sea_type >= DBNumUserLayers) return 0;
+    if (DBIsContact(arg->sea_type))
+    {
+	if (arg->sea_type < DBNumUserLayers)
+	{
+	    if (arg->sea_plane != DBPlane(arg->sea_type)) return 0;
+#ifdef NONMANHATTAN
+	    arg->sea_type = TiGetTypeExact(tile);
+#endif
+	}
+	else	/* stacked contact type */
+	{
+	    TileTypeBitMask *rmask = DBResidueMask(arg->sea_type);
+	    TileType rtype;
+	    for (rtype = TT_TECHDEPBASE; rtype < DBNumUserLayers; rtype++)
+		if (TTMaskHasType(rmask, rtype) &&
+				arg->sea_plane == DBPlane(rtype))
+		    break;
+	    if (rtype == DBNumUserLayers) return 0;
+
+	    /* Continue, with the tile type recast as rtype */
 
 #ifdef NONMANHATTAN
-    arg->sea_type = TiGetTypeExact(tile);
+	    if (IsSplit(tile))
+	    {
+		if (SplitSide(tile))
+		    arg->sea_type = (rtype << 14) | SplitLeftType(tile);
+		else
+		    arg->sea_type = rtype | (SplitRightType(tile) << 14);
+		arg->sea_type |= TT_DIAGONAL | TT_DIRECTION | TT_SIDE;
+	    }
+	    else
+#endif
+	    arg->sea_type = rtype;
+	}
+    }
+
+#ifdef NONMANHATTAN
+    else
+	arg->sea_type = TiGetTypeExact(tile);
 #endif
 
     /* If the paint doesn't have to be in the edit cell, life's pretty
@@ -208,6 +241,7 @@ selEnumPFunc1(tile, arg)
     {
 	GeoTransRect(&EditToRootTransform, &arg->sea_rectList->r_r, &rootRect);
 	GeoClip(&rootRect, &rect);
+	
 	if ((*arg->sea_func)(&rootRect, arg->sea_type, arg->sea_cdarg) != 0)
 	    return 1;
 	freeMagic((char *) arg->sea_rectList);
@@ -228,17 +262,18 @@ selEnumPFunc2(tile, arg)
 {
     LinkedRect *lr;
     register ttype;
+    TileType loctype;
 
 #ifdef NONMANHATTAN
-    TileType loctype;
-     
     if (arg->sea_type & TT_DIAGONAL)
 	loctype = (arg->sea_type & TT_SIDE) ?
 		(arg->sea_type & TT_RIGHTMASK) >> 14 :
 		arg->sea_type & TT_LEFTMASK;
     else
-	loctype = arg->sea_type;
+#endif
+    loctype = arg->sea_type;
      
+#ifdef NONMANHATTAN
     if (IsSplit(tile))
     {
 	ttype = SplitLeftType(tile);
@@ -248,11 +283,25 @@ selEnumPFunc2(tile, arg)
     else
 	ttype = TiGetType(tile);
         
-    if (loctype != ttype)
 #else
     ttype = TiGetTypeExact(tile);
-    if (arg->sea_type != ttype)
 #endif
+
+    /* Handle stacked contact types */
+    if (ttype >= DBNumUserLayers)
+    {
+	TileTypeBitMask *rmask = DBResidueMask(ttype);
+	TileType rtype;
+	for (rtype = TT_TECHDEPBASE; rtype < DBNumUserLayers; rtype++)
+	    if (TTMaskHasType(rmask, rtype))
+		if (arg->sea_plane == DBPlane(rtype))
+		{
+		    ttype = rtype;
+		    break;
+		}
+    }
+
+    if (loctype != ttype)
     {
 	if (arg->sea_nonEdit != NULL) *(arg->sea_nonEdit) = TRUE;
 	return 0;
