@@ -946,7 +946,6 @@ selStretchFillFunc2(tile, area)
     for (pNum = PL_PAINTBASE; pNum < DBNumPlanes; pNum++)
 	if (DBPaintOnPlane(selStretchType, pNum))
 	    (void) DBSrPaintArea((Tile *) NULL,
-//			EditCellUse->cu_def->cd_planes[DBPlane(selStretchType)],
 			EditCellUse->cu_def->cd_planes[pNum],
 			&editArea, &DBAllButSpaceAndDRCBits, selStretchFillFunc3,
 			(ClientData) &spaceArea);
@@ -971,7 +970,7 @@ selStretchFillFunc3(tile, area)
 				 */
 {
     Rect editArea, rootArea;
-    TileType type;
+    TileType type, stype;
     register TileTypeBitMask *mask;
     StretchArea *sa;
 
@@ -1030,7 +1029,8 @@ selStretchFillFunc3(tile, area)
     /* we should be filling in the original type.			*/
 
     if (!DBIsContact(type) || !DBIsContact(selStretchType) ||
-		(DBTechFindStacking(type, selStretchType) < DBNumUserLayers))
+		(((stype = DBTechFindStacking(type, selStretchType))
+		< DBNumUserLayers) && (stype >= TT_TECHDEPBASE)))
     {
 	/* If type is a residue of selStretchType then we always fill	*/
 	/* with type.  Otherwise, we check the list of plow types to	*/
@@ -1042,6 +1042,29 @@ selStretchFillFunc3(tile, area)
 		type = selStretchType;
     }
 
+    /* Otherwise, if we have two fixed types, and they both border the	*/
+    /* selection area (as opposed to overlapping), and they share a	*/
+    /* residue, then the type to paint is the shared residue type.	*/
+
+    else if (DBIsContact(type) && DBIsContact(selStretchType))
+    {
+	if (((selStretchX < 0) && (editArea.r_xtop == area->r_xbot)) ||
+	    ((selStretchX > 0) && (editArea.r_xbot == area->r_xtop)) ||
+	    ((selStretchY < 0) && (editArea.r_ytop == area->r_ybot)) ||
+	    ((selStretchY > 0) && (editArea.r_ybot == area->r_ytop)))
+	{
+	    TileTypeBitMask rmask;
+	    TTMaskAndMask3(&rmask, DBResidueMask(type),
+			DBResidueMask(selStretchType));
+	    for (type = TT_TECHDEPBASE; type < DBNumUserLayers; type++)
+		if (TTMaskHasType(&rmask, type))
+		    break;
+
+	    if (type == DBNumUserLayers)
+		return 0;	/* Types do not share any residues */
+	}
+    }
+
     /* Save around the area we just found. */
 
     sa = (StretchArea *) mallocMagic(sizeof(StretchArea));
