@@ -22,9 +22,7 @@ static char rcsid[]="$Header: /ufs/repository/magic/signals/signals.c,v 1.7 2001
 
 #include <stdio.h>
 #include <signal.h>
-#ifdef MAGIC_WRAPPER
 #include <sys/time.h>
-#endif
 
 #ifndef	SIGEMT
 #define	SIGEMT	7	/* EMT instruction (SIGUNUSED) */
@@ -106,8 +104,6 @@ static int sigNumDisables = 0;
 
 extern bool mainDebug;
 
-#ifdef MAGIC_WRAPPER
-
 /*---------------------------------------------------------
  * While we can conveniently run Ctrl-C interrupts from
  * the terminal at any time, we can't do this from the
@@ -115,11 +111,14 @@ extern bool mainDebug;
  * a separate process.  So we run a timer process during
  * WindUpdate() which periodically processes Tk events
  * in the window.
+ *
+ * This timer can be set for "secs" second intervals.  If
+ * "secs" is zero, it defaults to a 1/4 second interval.
  *---------------------------------------------------------
  */
 
 void
-SigSetTimer()
+SigSetTimer(int secs)
 {
     struct itimerval subsecond;	/* one-quarter second interval */
  
@@ -133,8 +132,8 @@ SigSetTimer()
    
     subsecond.it_interval.tv_sec = 0;
     subsecond.it_interval.tv_usec = 0;
-    subsecond.it_value.tv_sec = 0;
-    subsecond.it_value.tv_usec = 250000;
+    subsecond.it_value.tv_sec = secs;
+    subsecond.it_value.tv_usec = (secs == 0) ? 250000 : 0;
 
     setitimer(ITIMER_REAL, &subsecond, NULL);
 }
@@ -161,8 +160,6 @@ sigOnAlarm(int signo)
 	GrDisplayStatus = DISPLAY_BREAK_PENDING;
 }
 
-#endif
-
 /*---------------------------------------------------------
  * sigOnStop:
  *	This procedure handles stop signals.
