@@ -70,16 +70,27 @@ struct conSrArg
  * hierarchical stuff.
  */
 
+typedef struct
+{
+    Rect		area;		/* Area to process */
+    TileTypeBitMask	*connectMask;	/* Connection mask for search */
+} conSrArea;
+
 struct conSrArg2
 {
-    Rect csa2_area;			/* Area that we know is connected, but
-					 * which may be connected to other
-					 * stuff.
+    CellUse		*csa2_use;	/* Destination use */
+    TileTypeBitMask	*csa2_connect;	/* Table indicating what connects
+					 * to what.
 					 */
-    int csa2_type;			/* Type of material we found. */
-    struct conSrArg2 *csa2_next;	/* Pointer to next in list. */
+    Rect		*csa2_bounds;	/* Area that limits the search */
+
+    conSrArea		*csa2_list;	/* List of areas to process */
+    int			csa2_top;	/* Index of next area to process */
+    int			csa2_size;	/* Max. number bins in area list */
 };
 
+#define CSA2_LIST_START_SIZE 256
+
 /* We need to maintain where in the cell hierarchy we are as we search
  * the tile database when extracting a node name.  To do this, a stack is
  * maintained of the hierarchy of cell definitions (a search context)
@@ -101,99 +112,6 @@ static ScxStackElt	*stackBase = (ScxStackElt *) NULL;
 					/* stack of search contexts */
 static char 		bestName[256];
 
-
-/*
- * ----------------------------------------------------------------------------
- *
- *	SimFirstTile
- *
- *	This procedure is based upon the function dbcConnectFunc in the
- *	database module.
- *
- * 	This procedure is invoked by SimTreeSrTiles from SimTreeCopyConnect,
- *	whenever a tile is found that is connected to the current area
- *	being processed.  If the tile overlaps the search area in a non-
- *	trivial way (i.e. more than a 1x1 square of overlap at a corner)
- *	then the area of the tile is added onto the list of things to check.
- *	The "non-trivial" overlap check is needed to prevent caddy-corner
- *	tiles from being considered as connected.
- *
- * Results:
- *	Always returns 1 when a tile has been found.
- *
- * Side effects:
- *	Adds a new record to the current check list.
- *
- * ----------------------------------------------------------------------------
- */
-
-int
-SimFirstTile(tile, cx)
-    Tile *tile;			/* Tile found. */
-    TreeContext *cx;		/* Describes context of search.  The client
-				 * data is a pointer to the list head of
-				 * the conSrArg2's describing the areas
-				 * left to check.
-				 */
-{
-    struct conSrArg2 **pHead = (struct conSrArg2 **) cx->tc_filter->tf_arg;
-    struct conSrArg2 *newCsa2;
-    Rect tileArea, *srArea;
-    SearchContext *scx = cx->tc_scx;
-    static char currentName[256];
-    static char buff[256];
-    int i;
-
-    TiToRect(tile, &tileArea);
-    srArea = &cx->tc_scx->scx_area;
-    if (((tileArea.r_xbot >= srArea->r_xtop-1) ||
-        (tileArea.r_xtop <= srArea->r_xbot+1)) &&
-	((tileArea.r_ybot >= srArea->r_ytop-1) ||
-	(tileArea.r_ytop <= srArea->r_ybot+1)))
-    {
-	/* If the search area is only one unit wide or tall, then it's
-	 * OK to have only a small overlap.  This happens only when
-	 * looking for an initial search tile.
-	 */
-
-	 if (((srArea->r_xtop-1) != srArea->r_xbot)
-	    && ((srArea->r_ytop-1) != srArea->r_ybot)) return 0;
-    }
-    MALLOC(struct conSrArg2 *, newCsa2, sizeof (struct conSrArg2));
-    GeoTransRect(&cx->tc_scx->scx_trans, &tileArea, &newCsa2->csa2_area);
-    newCsa2->csa2_type = TiGetTypeExact(tile);
-
-#ifdef NONMANHATTAN
-    /* Resolve geometric transformations on diagonally-split tiles */
-   
-    if (IsSplit(tile))
-    {
-        int o1, o2, o3, dir, side;
-        TileType loctype = newCsa2->csa2_type;
-
-        o1 = ((scx->scx_trans.t_e > 0) || (scx->scx_trans.t_d > 0)) ? 1 : 0;
-        o2 = ((scx->scx_trans.t_a > 0) || (scx->scx_trans.t_b > 0)) ? 1 : 0;
-        o3 = (scx->scx_trans.t_a != 0) ? 1 : 0;
-        dir = SplitDirection(tile) ^ o1 ^ o2;
-        side = SplitSide(tile) ^ o2 ^ (SplitDirection(tile) | o3);
-
-        if (side == SplitSide(tile))
-            newCsa2->csa2_type = TT_DIAGONAL | SplitLeftType(tile) |
-                        (SplitRightType(tile) << 14);
-        else
-            newCsa2->csa2_type = TT_DIAGONAL | SplitRightType(tile) |
-                        (SplitLeftType(tile) << 14);
-
-        if (side) newCsa2->csa2_type |= TT_SIDE;
-        if (dir) newCsa2->csa2_type |= TT_DIRECTION;
-    }
-#endif
-
-    newCsa2->csa2_next = *pHead;
-    *pHead = newCsa2;
-
-    return 1;
-}
 
 /*
  * ----------------------------------------------------------------------------
@@ -212,10 +130,10 @@ SimFirstTile(tile, cx)
  *	tiles from being considered as connected.
  *
  * Results:
- *	Always returns 0 to keep the search from aborting.
+ *	Returns 0 normally, 1 if an abort condition has been encountered.
  *
  * Side effects:
- *	Adds a new record to the current check list.
+ *	Paints into the destination definition.
  *
  * ----------------------------------------------------------------------------
  */
@@ -229,26 +147,41 @@ SimConnectFunc(tile, cx)
 				 * left to check.
 				 */
 {
-    struct conSrArg2	**pHead = (struct conSrArg2 **) cx->tc_filter->tf_arg;
-    struct conSrArg2	*newCsa2;
-    Rect 		tileArea, *srArea;
+    struct conSrArg2	*csa2;
+    Rect 		tileArea, *srArea, newarea;
     SearchContext	*scx = cx->tc_scx;
+    TileTypeBitMask	notConnectMask, *connectMask;
+    TileType		newtype, loctype;
+    int 		i, pNum;
     char		pathName[256];
     static char		nodeName[256];
-    int i;
+    CellDef 		*def;
 
     TiToRect(tile, &tileArea);
     srArea = &scx->scx_area;
+
     if (((tileArea.r_xbot >= srArea->r_xtop-1) ||
         (tileArea.r_xtop <= srArea->r_xbot+1)) &&
 	((tileArea.r_ybot >= srArea->r_ytop-1) ||
 	(tileArea.r_ytop <= srArea->r_ybot+1)))
     {
-	return 0;
+	/* If the search area is only one unit wide or tall, then it's
+	 * OK to have only a small overlap.  This happens only when
+	 * looking for an initial search tile.
+	 */
+
+	if (((srArea->r_xtop-1) != srArea->r_xbot)
+		&& ((srArea->r_ytop-1) != srArea->r_ybot)) return 0;
     }
-    MALLOC(struct conSrArg2 *, newCsa2, sizeof (struct conSrArg2));
-    GeoTransRect(&scx->scx_trans, &tileArea, &newCsa2->csa2_area);
-    newCsa2->csa2_type = TiGetTypeExact(tile);
+    GeoTransRect(&scx->scx_trans, &tileArea, &newarea);
+
+    /* Clip the current area down to something that overlaps the
+     * area of interest.
+     */
+	
+    csa2 = (struct conSrArg2 *)cx->tc_filter->tf_arg;
+    GeoClip(&newarea, csa2->csa2_bounds);
+    if (GEO_RECTNULL(&newarea)) return 0;
 
 #ifdef NONMANHATTAN
     /* Resolve geometric transformations on diagonally-split tiles */
@@ -256,7 +189,6 @@ SimConnectFunc(tile, cx)
     if (IsSplit(tile))
     {
         int o1, o2, o3, dir, side;
-        TileType loctype = newCsa2->csa2_type;
 
         o1 = ((scx->scx_trans.t_e > 0) || (scx->scx_trans.t_d > 0)) ? 1 : 0;
         o2 = ((scx->scx_trans.t_a > 0) || (scx->scx_trans.t_b > 0)) ? 1 : 0;
@@ -265,43 +197,147 @@ SimConnectFunc(tile, cx)
         side = SplitSide(tile) ^ o2 ^ (SplitDirection(tile) | o3);
 
         if (side == SplitSide(tile))
-            newCsa2->csa2_type = TT_DIAGONAL | SplitLeftType(tile) |
+            newtype = TT_DIAGONAL | SplitLeftType(tile) |
                         (SplitRightType(tile) << 14);
         else
-            newCsa2->csa2_type = TT_DIAGONAL | SplitRightType(tile) |
+            newtype = TT_DIAGONAL | SplitRightType(tile) |
                         (SplitLeftType(tile) << 14);
 
-        if (side) newCsa2->csa2_type |= TT_SIDE;
-        if (dir) newCsa2->csa2_type |= TT_DIRECTION;
+        if (dir) newtype |= TT_DIRECTION;
+        if (side)
+	{
+	    newtype |= TT_SIDE;
+	    loctype = (newtype & TT_RIGHTMASK) >> 14;
+	}
+	else
+	    loctype = newtype & TT_LEFTMASK;
     }
+    else
+#endif
+    loctype = newtype = TiGetTypeExact(tile);
+
+    /* See if the destination cell contains stuff over the whole
+     * current area (on its home plane) that is connected to it.
+     * If so, then there's no need to process the current area,
+     * since any processing that is needed was already done before.
+     */
+
+    pNum = DBPlane(loctype);
+    connectMask = &csa2->csa2_connect[loctype];
+
+    if (DBIsContact(loctype))
+    {
+	TTMaskSetOnlyType(&notConnectMask, loctype);
+	TTMaskCom(&notConnectMask);
+    }
+    else
+    {
+	TTMaskCom2(&notConnectMask, connectMask);
+    }
+
+    def = csa2->csa2_use->cu_def;
+    if (DBSrPaintArea((Tile *) NULL, def->cd_planes[pNum],
+	    &newarea, &notConnectMask, dbcUnconnectFunc,
+	    (ClientData) connectMask) == 0)
+	return 0;
+
+    /* Paint this tile into the destination cell. */
+	
+#ifdef NONMANHATTAN
+    DBNMPaintPlane(def->cd_planes[pNum], newtype, &newarea,
+		DBStdPaintTbl(loctype, pNum), (PaintUndoInfo *) NULL);
+#else
+    DBPaintPlane(def->cd_planes[pNum], &newarea,
+		DBStdPaintTbl(loctype, pNum), (PaintUndoInfo *) NULL);
 #endif
 
-    newCsa2->csa2_next = *pHead;
-    *pHead = newCsa2;
+    /* Stuff unique to the nodename search follows. */
 
-    SimMakePathname( pathName );
+    SimMakePathname(pathName);
 
-    /* extract the node name */
+    /* Extract the node name */
 
     SigDisableInterrupts();
-    strcpy( nodeName, SimGetNodeName( cx->tc_scx, tile, pathName ) ); 
+    strcpy(nodeName, SimGetNodeName(cx->tc_scx, tile, pathName)); 
     SigEnableInterrupts();
 
     /* save the "best" name for this node */
 
-    if( bestName[0] == '\0' || efPreferredName( nodeName, bestName ) )
-	strcpy( bestName, nodeName );
+    if (bestName[0] == '\0' || efPreferredName(nodeName, bestName))
+	strcpy(bestName, nodeName);
 
-    /* abort the name search if the name is global or name is in the
+    /* Since the whole area of this tile hasn't been recorded,
+     * we must process its area to find any other tiles that
+     * connect to it.  Add each of them to the list of things
+     * to process.  We have to expand the search area by 1 unit
+     * on all sides because SimTreeSrTiles only returns things
+     * that overlap the search area, and we want things that
+     * even just touch.
+     */
+
+#ifdef NONMANHATTAN
+    /* Only extend those sides bordering the diagonal tile */
+
+    if (newtype & TT_DIAGONAL)
+    {
+	if (newtype & TT_SIDE)			/* right */
+	    newarea.r_xtop += 1;
+	else					/* left */
+	    newarea.r_xbot -= 1;
+	if (((newtype & TT_SIDE) >> 1)
+		== (newtype & TT_DIRECTION))	/* top */
+	    newarea.r_ytop += 1;
+	else					/* bottom */
+	    newarea.r_ybot -= 1;
+    }
+    else
+    {
+#endif
+    newarea.r_xbot -= 1;
+    newarea.r_ybot -= 1;
+    newarea.r_xtop += 1;
+    newarea.r_ytop += 1;
+#ifdef NONMANHATTAN
+    }
+#endif
+
+    /* Abort the name search if the name is global or name is in the
      * abort name search table 
      */
 
-    i = strlen( nodeName );
-    if( SimSawAbortString  || SigInterruptPending || (nodeName[i - 1] == '!') )
+    i = strlen(nodeName);
+    if (SimSawAbortString  || SigInterruptPending || (nodeName[i - 1] == '!'))
 	return 1;
 
+    /* Register the area and connection mask as needing to be processed */
+
+    if (++csa2->csa2_top == csa2->csa2_size)
+    {
+	/* Reached list size limit---need to enlarge the list	   */
+	/* Double the size of the list every time we hit the limit */
+
+	conSrArea *newlist;
+	int i, lastsize = csa2->csa2_size;
+
+	csa2->csa2_size *= 2;
+
+	newlist = (conSrArea *)mallocMagic(csa2->csa2_size * sizeof(conSrArea));
+	for (i = 0; i < lastsize; i++)
+	{
+	    newlist[i].area = csa2->csa2_list[i].area;
+	    newlist[i].connectMask = csa2->csa2_list[i].connectMask;
+	    
+	}
+	freeMagic((char *)csa2->csa2_list);
+	csa2->csa2_list = newlist;
+    }
+
+    csa2->csa2_list[csa2->csa2_top].area = newarea;
+    csa2->csa2_list[csa2->csa2_top].connectMask = connectMask;
+
     return 0;
 }
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -365,150 +401,37 @@ SimTreeCopyConnect(scx, mask, xMask, connect, area, destUse, Node_Name)
 					 * words in some compilers.
 					 */
 {
-    SearchContext scx2;
-    TileTypeBitMask notConnectMask, *connectMask;
-    TileType loctype;
-    struct conSrArg2 *list, *current;
-    CellDef *def = destUse->cu_def;
-    Cvals cdata;
-    int pNum;
+    TileTypeBitMask *newmask;
+    struct conSrArg2 csa2;
 
-    /* Get one initial tile and put it our list. */
+    csa2.csa2_use = destUse;
+    csa2.csa2_bounds = area;
+    csa2.csa2_connect = connect;
 
-    list = NULL;
-    (void) DBTreeSrTiles(scx, mask, xMask, SimFirstTile, (ClientData) &list );
+    csa2.csa2_size = CSA2_LIST_START_SIZE;
+    csa2.csa2_list = (conSrArea *)mallocMagic(CSA2_LIST_START_SIZE
+			* sizeof(conSrArea));
+    csa2.csa2_top = -1;
 
     SimInitScxStk();
     bestName[0] = '\0';
 
-    /* Enter the main processing loop, pulling areas from the list
-     * and processing them one at a time until the list is empty.
+    (void) SimTreeSrTiles(scx, mask, xMask, SimConnectFunc, (ClientData) &csa2);
+    while (csa2.csa2_top >= 0)
+    {
+	newmask = csa2.csa2_list[csa2.csa2_top].connectMask;
+	scx->scx_area = csa2.csa2_list[csa2.csa2_top].area;
+	csa2.csa2_top--;
+	SimTreeSrTiles(scx, newmask, xMask, SimConnectFunc, (ClientData) &csa2);
+    }
+    freeMagic((char *)csa2.csa2_list);
+
+    /* Recompute the bounding box of the destination and record
+     * its area for redisplay.
      */
     
-    scx2 = *scx;
-    while (list != NULL)
-    {
-	current = list;
-	list = current->csa2_next;
-
-	/* Clip the current area down to something that overlaps the
-	 * area of interest.
-	 */
-	
-	GeoClip(&current->csa2_area, area);
-	if (GEO_RECTNULL(&current->csa2_area)) goto endOfLoop;
-
-	/* See if the destination cell contains stuff over the whole
-	 * current area (on its home plane) that is connected to it.
-	 * If so, then there's no need to process the current area,
-	 * since any processing that is needed was already done before.
-	 */
-	
-#ifdef NONMANHATTAN
-        if (current->csa2_type & TT_DIAGONAL)
-        {
-            loctype = (current->csa2_type & TT_SIDE) ?
-                        ((current->csa2_type & TT_RIGHTMASK) >> 14) :
-                        current->csa2_type & TT_LEFTMASK;
-        }
-        else
-#endif
-	loctype = current->csa2_type;
-
-	connectMask = &connect[loctype];
-        if (TTMaskHasType(&DBContactBits,loctype))
-        {
-             TTMaskSetOnlyType(&notConnectMask, loctype);
-             TTMaskCom(&notConnectMask);
-        }
-        else
-        {
-             TTMaskCom2(&notConnectMask, connectMask);
-        }
-	pNum = DBPlane(loctype);
-
-	cdata.area = &(current->csa2_area);
-	cdata.mask = connectMask;
-
-	if (DBSrPaintArea((Tile *) NULL, def->cd_planes[pNum],
-	    &current->csa2_area, &notConnectMask,
-	    dbcUnconnectFunc, (ClientData) &cdata) != 0)
-	{
-	    /* Since the whole area of this tile hasn't been recorded,
-	     * we must process its area to find any other tiles that
-	     * connect to it.  Add each of them to the list of things
-	     * to process.  We have to expand the search area by 1 unit
-	     * on all sides because SimTreeSrTiles only returns things
-	     * that overlap the search area, and we want things that
-	     * even just touch.
-	     */
-
-	    scx2.scx_area = current->csa2_area;
-
-#ifdef NONMANHATTAN
-            /* Only extend those sides bordering the diagonal tile */
-
-            if (current->csa2_type & TT_DIAGONAL)
-            {
-                if (current->csa2_type & TT_SIDE)               /* right */
-                    scx2.scx_area.r_xtop += 1;
-                else                                            /* left */
-                    scx2.scx_area.r_xbot -= 1;
-                if (((current->csa2_type & TT_SIDE) >> 1)
-                        == (current->csa2_type & TT_DIRECTION)) /* top */
-                    scx2.scx_area.r_ytop += 1;
-                else                                            /* bottom */
-                    scx2.scx_area.r_ybot -= 1;
-            }
-            else
-            {
-#endif
-	    scx2.scx_area.r_xbot -= 1;
-	    scx2.scx_area.r_ybot -= 1;
-	    scx2.scx_area.r_xtop += 1;
-	    scx2.scx_area.r_ytop += 1;
-#ifdef NONMANHATTAN
-            }
-#endif
-
-	    if( SimTreeSrTiles(&scx2, connectMask, xMask, SimConnectFunc,
-	      (ClientData) &list) )
-	    {
-		FREE( (char *) current );
-		break;
-	    }
-	}
-
-	/* Lastly, paint this tile into the destination cell.  This
-	 * marks its area has having been processed.  Then recycle
-	 * the storage for the current list element.
-	 */
-	
-#ifdef NONMANHATTAN
-	DBNMPaintPlane(def->cd_planes[pNum],  current->csa2_type,
-	    &current->csa2_area, DBStdPaintTbl(loctype, pNum),
-	    (PaintUndoInfo *) NULL);
-#else
-	DBPaintPlane(def->cd_planes[pNum], &current->csa2_area,
-	    DBStdPaintTbl(loctype, pNum), (PaintUndoInfo *) NULL);
-#endif
-	endOfLoop: FREE((char *) current);
-    }
-
-	/* Just in case we got aborted, or an abort string was seen */
-    while (list != NULL )
-    {
-	current = list;
-	list = current->csa2_next;
-	FREE( (char *) current );
-    }
-
-    /* Finally, when all done, recompute the bounding box of the
-     * destination and record its area for redisplay.
-     */
-    
-    strcpy( Node_Name, bestName );
-    DBReComputeBbox(def);
+    strcpy(Node_Name, bestName);
+    DBReComputeBbox(destUse->cu_def);
 }
 
 /*
@@ -708,6 +631,7 @@ SimSrConnect(def, startArea, mask, connect, bounds, func, clientData)
 
 
 /*
+ * ----------------------------------------------------------------------------
  * SimInitScxStk
  *
  *	This procedure initializes the doubly-linked stack of search
@@ -720,6 +644,7 @@ SimSrConnect(def, startArea, mask, connect, bounds, func, clientData)
  * Side effects:
  *	Space in the stack is freed if the stack is not empty.
  *	
+ * ----------------------------------------------------------------------------
  */
 
 SimInitScxStk()
@@ -737,6 +662,7 @@ SimInitScxStk()
 }
 
 /*
+ * ----------------------------------------------------------------------------
  * SimPushScx
  *
  *	This procedure pushes a new search context onto the search
@@ -749,11 +675,11 @@ SimInitScxStk()
  * Side effects:
  *	None.
  *
+ * ----------------------------------------------------------------------------
  */
 
 SimPushScx(newscx) 
-
-SearchContext *newscx;
+    SearchContext *newscx;
 {
 
     ScxStackElt *dummy;
@@ -773,6 +699,7 @@ SearchContext *newscx;
 }
 
 /*
+ * ----------------------------------------------------------------------------
  * SimPopScx
  *
  *	This procedure pops the search context on top of the stack.
@@ -783,6 +710,7 @@ SearchContext *newscx;
  * Side effects:
  *	None.
  *
+ * ----------------------------------------------------------------------------
  */
 
 SimPopScx()
@@ -801,6 +729,7 @@ SimPopScx()
 }
 
 /*
+ * ----------------------------------------------------------------------------
  * SimMakePathname
  *
  * 	This procedure creates a pathname for the current hierarchy
@@ -812,12 +741,11 @@ SimPopScx()
  * Side effects:
  *	None.
  *
+ * ----------------------------------------------------------------------------
  */
 
 SimMakePathname(pathname)
-
-char *pathname;
-
+    char *pathname;
 {
     ScxStackElt *dummy;
     char buff[100];
@@ -940,12 +868,10 @@ SimTreeSrTiles(scx, mask, xMask, func, cdarg)
      * in our tile plane.
      */
 
-    if (DBCellSrArea(scx, SimCellTileSrFunc, (ClientData) &filter)) {
+    if (DBCellSrArea(scx, SimCellTileSrFunc, (ClientData) &filter))
 	return 1;
-    }
-    else { 
+    else
 	return 0;
-    }
 }
 
 /*
@@ -1095,6 +1021,7 @@ SimPutLabel(cellDef, rect, align, text, type)
     lab->lab_type = type;
     lab->lab_rect = *rect;
     lab->lab_next = NULL;
+    lab->lab_flags = 0;
     if (cellDef->cd_labels == NULL)
 	cellDef->cd_labels = lab;
     else
@@ -1104,7 +1031,7 @@ SimPutLabel(cellDef, rect, align, text, type)
     }
     cellDef->cd_lastLabel = lab;
 
-    DBUndoPutLabel(cellDef, rect, align, text, type);
+    DBUndoPutLabel(cellDef, rect, align, text, type, 0);
     return align;
 }
 
