@@ -571,7 +571,7 @@ char *nodeName;
  * SimFillBuffer
  *
  *	This procedure reads characters from Rsim via a pipe and
- *	places the charcters into a buffer pointed by pLastChar.
+ *	places the characters into a buffer pointed by pLastChar.
  *	It is assumed the buffer is at least READBUF_SIZE charcters
  *	large, and that charCount contains the number of charcters
  *	which remain unprocessed in the buffer.
@@ -590,6 +590,7 @@ char *nodeName;
  *
  */
 
+int
 SimFillBuffer(buffHead, pLastChar, charCount)
 
     char *buffHead;			/* ptr to start of buffer */
@@ -600,32 +601,51 @@ SimFillBuffer(buffHead, pLastChar, charCount)
     int *charCount;			/* number of chars in the buffer */
 
 {
-    int 	charsRead;
+    int 	charsRead = 0;
     char 	*temp;
-    int		n, nfd, nr, nex;
+    int		n, nfd;
 #if defined(SYSV) || defined(CYGWIN)
     fd_set readfds, writefds, exceptfds;
+#else
+    int		nr, nex;
 #endif  /* SYSV */
 
+    struct timeval timeout;
+
+    /* Set the timeout to 5 seconds so we don't block indefinitely if	*/
+    /* something goes wrong with the pipe.				*/
+
+    timeout.tv_sec = 5;
+    timeout.tv_usec = 0;
+
     /* read reply from Rsim */
 
-  try_again:
-
-    nr = nex = 1 << pipeIn;
+    FD_ZERO(&readfds);		/* ? */
+    FD_ZERO(&exceptfds);	/* ? */
     nfd = pipeIn + 1;
+
+try_again:
+
 #ifdef SYSV
-    FD_SET(nr, &readfds);
+    FD_SET(pipeIn, &readfds);
     FD_ZERO(&writefds);
-    FD_SET(nex, &exceptfds);
-    n = select( nfd, &readfds, &writefds, &exceptfds, (struct timeval *) NULL);
-#else /* SYSV */
-     n = select( nfd, &nr, (int *) NULL, &nex, (struct timeval *) NULL );
-#endif  /* SYSV */
-    if( n < 0 )
+    FD_SET(pipeIn, &exceptfds);
+    n = select(nfd, &readfds, &writefds, &exceptfds, &timeout);
+
+#else /* !SYSV */
+    nr = nex = 1 << pipeIn;
+    n = select(nfd, &nr, (int *) NULL, &nex, &timeout);
+
+#endif
+
+    if (n == 0)
+	return 0;	/* select() timed out */
+
+    else if (n < 0)
     {
-	if( errno == EINTR )
+	if (errno == EINTR)
 	{
-	    if(SigInterruptPending)
+	    if (SigInterruptPending)
 	    {
 		kill(rsim_pid, SIGINT);
 		SigInterruptPending = FALSE;
