@@ -20,7 +20,7 @@
 
 #ifndef lint
 static char sccsid[] = "@(#)SimSelect.c	4.14 MAGIC (Berkeley) 10/3/85";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #ifdef SYSV
