@@ -21,6 +21,7 @@ static char rcsid[] = "$Header: /ufs/repository/magic/tech/tech.c,v 1.3 2001/01/
 #endif  /* not lint */
 
 #include <stdio.h>
+#include <stdarg.h>
 #include <ctype.h>
 #include "misc/magic.h"
 #include "utils/geometry.h"
@@ -28,9 +29,7 @@ static char rcsid[] = "$Header: /ufs/repository/magic/tech/tech.c,v 1.3 2001/01/
 #include "tech/tech.h"
 #include "textio/textio.h"
 #include "windows/windows.h"
-#include "graphics/graphics.h"
 #include "utils/malloc.h"
-#include <varargs.h>
 
 global char *TechDefault = NULL;
 global int  TechFormatVersion;
@@ -511,19 +510,14 @@ TechPrintLine()
 		TechFileName, techLineNumber, section);
 }
 
- /*VARARGS0*/
-
 void
-TechError(va_alist)
-va_dcl
+TechError(char *fmt, ...)
 {
     va_list args;
-    char *fmt;
 
     TechPrintLine();
-    va_start(args);
-    fmt = va_arg(args, char *);
-    (void) GrVfprintf(stderr, fmt, args);
+    va_start(args, fmt);
+    (void) Vfprintf(stderr, fmt, args);
     va_end(args);
 }
 
