@@ -1,5 +1,5 @@
 /*
- * LIBflock.c --
+ * flock.c --
  *
  * File opening with file locks (used by utils/path.c and database/DBio.c
  * Written by Michael D. Godfrey, Stanford University
@@ -490,8 +490,8 @@ flock_test_set(lfilename, f)
 		    fclose(f);
 #ifdef FLOCK_DEBUG
 		else
-		    TxPrintf("flock_test_set called with file <%s> not opened.\n",
-				lfilename);
+		    TxPrintf("flock_test_set called for file <%s>, but file"
+				" is not open.\n", lfilename);
 #endif
 	    }
 	    else
@@ -616,7 +616,7 @@ flock_open(filename, ext, mode)
        MALLOC(char *, recname, strlen(LOCK_DIR)+strlen(filename)+strlen("lock")+4);
        sprintf(recname, "%s/%s.%s", LOCK_DIR, filename, "lock" );
 
-       /* Check for "rb" is the other half of the kludge defined in DBio.c */
+       /* Test for "rb" is the other half of the kludge found in DBio.c */
        if ((flock_list_test(recname) == fnul) && (strcmp(mode, "rb") != 0))
        {
           f = flock_test_set(filename, f);
