@@ -29,7 +29,7 @@
 
 #ifndef	_MAGIC
 int err0 = Need_to_include_misc/magic_header;
-#endif	_MAGIC
+#endif	/* _MAGIC */
 
 /* --------------------- Lists (Lisp like) ---------------------------- */
 
