@@ -1,6 +1,6 @@
 #ifndef lint
 static char rcsid[] = "$Header: /ufs/repository/magic/utils/printstuff.c,v 1.2 2001/01/12 22:13:32 jsolomon Exp $";
-#endif not lint
+#endif /* not lint */
 
 #include "misc/magic.h"
 #include "utils/geometry.h"
