@@ -20,7 +20,7 @@
 
 #ifndef lint
 static char rcsid[] = "$Header: /ufs/repository/magic/utils/runstats.c,v 1.2 2001/01/12 22:13:33 jsolomon Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <sys/types.h>
 #include <sys/times.h>
