@@ -50,7 +50,7 @@ static char rcsid[]="$Header: /ufs/repository/magic/windows/windClient.c,v 1.3 2
 #define WIND_ILG -1
 
 /* our window client ID */
-global WindClient windClientID = NULL;
+global WindClient windClientID = (WindClient) NULL;
 
 extern int windBorderCmd();
 extern int windCaptionCmd(), windCrashCmd(), windCursorCmd();
