@@ -210,7 +210,7 @@ WindSeparateRedisplay(w)
     MagWindow *w;
 {
     windSomeSeparateRedisplay = TRUE;
-    if (w->w_redrawAreas != NULL) return;
+    if (w->w_redrawAreas != (ClientData)NULL) return;
     w->w_redrawAreas = (ClientData) DBNewPlane((ClientData) TT_SPACE);
 }
 
@@ -279,7 +279,7 @@ WindAreaChanged(w, area)
 	} else
 	    windCurRedrawPlane = windRedisplayArea;
     } else {
-	if (w->w_redrawAreas != NULL) 
+	if (w->w_redrawAreas != (ClientData)NULL) 
 	    windCurRedrawPlane = (Plane *) w->w_redrawAreas;
 	else
 	    windCurRedrawPlane = windRedisplayArea;
@@ -770,7 +770,7 @@ WindUpdate()
 
 #ifdef MAGIC_WRAPPER
     GrDisplayStatus = DISPLAY_IN_PROGRESS;
-    SigSetTimer();
+    SigSetTimer(0);
 #endif
 
     TTMaskSetOnlyType(&windTileMask, TT_ERROR_P);
@@ -791,7 +791,7 @@ WindUpdate()
 	if (w->w_flags & WIND_ISICONIC) {
 	    if (w->w_flags & WIND_REDRAWICON) windRedrawIcon(w);
 	} else {
-	    if (w->w_redrawAreas == NULL)
+	    if (w->w_redrawAreas == (ClientData)NULL)
 		windCurRedrawPlane = windRedisplayArea;
 	    else
 		windCurRedrawPlane = (Plane *) w->w_redrawAreas;
@@ -958,8 +958,8 @@ windBackgroundFunc(tile, notUsed)
 
     if (area.r_xtop < GrScreenRect.r_xtop) area.r_xtop -= 1;
     if (area.r_ytop < GrScreenRect.r_ytop) area.r_ytop -= 1;
-    GrLock((MagWindow *)GR_LOCK_SCREEN, FALSE);
+    GrLock(GR_LOCK_SCREEN, FALSE);
     GrClipBox(&area, STYLE_BACKGROUND);
-    GrUnlock((MagWindow *)GR_LOCK_SCREEN);
+    GrUnlock(GR_LOCK_SCREEN);
     return 0;
 }
