@@ -184,12 +184,16 @@ WindSurfaceToScreen(w, surface, screen)
 				 * (in w) of surface.
 				 */
 {
-    register tmp;
+    int tmp;
 
     /* Do the four coordinates one at a time. */
     
-    /* Redundant clipping to screen boundaries disabled (Tim, 3/20/01) */
-/*
+    /* The apparently redundant clipping (also done later by the box	*/
+    /* drawing routine) is necessary for two reasons:  1) it prevents	*/
+    /* having to do a multiply and shift function for any box extending	*/
+    /* outside the viewport, and more critically 2) it prevents integer	*/
+    /* overflow when a tile extends very far out of the viewport.	*/
+
     tmp = surface->r_xbot;
     if (tmp > w->w_surfaceArea.r_xtop)
 	screen->r_xbot = w->w_screenArea.r_xtop + 1;
@@ -198,7 +202,7 @@ WindSurfaceToScreen(w, surface, screen)
 	tmp -= w->w_surfaceArea.r_xbot;
 	if (tmp < 0)
 	    screen->r_xbot = w->w_screenArea.r_xbot - 1;
-        else screen->r_xbot = (w->w_origin.p_x + (tmp*w->w_scale))>>12;
+        else screen->r_xbot = (w->w_origin.p_x + (tmp * w->w_scale)) >> 12;
     }
     
     tmp = surface->r_ybot;
@@ -209,7 +213,7 @@ WindSurfaceToScreen(w, surface, screen)
 	tmp -= w->w_surfaceArea.r_ybot;
 	if (tmp < 0)
 	    screen->r_ybot = w->w_screenArea.r_ybot - 1;
-        else screen->r_ybot = (w->w_origin.p_y + (tmp*w->w_scale))>>12;
+        else screen->r_ybot = (w->w_origin.p_y + (tmp * w->w_scale)) >> 12;
     }
     
     tmp = surface->r_xtop;
@@ -220,7 +224,7 @@ WindSurfaceToScreen(w, surface, screen)
 	tmp -= w->w_surfaceArea.r_xbot;
 	if (tmp < 0)
 	    screen->r_xtop = w->w_screenArea.r_xbot - 1;
-        else screen->r_xtop = (w->w_origin.p_x + (tmp*w->w_scale))>>12;
+        else screen->r_xtop = (w->w_origin.p_x + (tmp * w->w_scale)) >> 12;
     }
     
     tmp = surface->r_ytop;
@@ -231,21 +235,8 @@ WindSurfaceToScreen(w, surface, screen)
 	tmp -= w->w_surfaceArea.r_ybot;
 	if (tmp < 0)
 	    screen->r_ytop = w->w_screenArea.r_ybot - 1;
-        else screen->r_ytop = (w->w_origin.p_y + (tmp*w->w_scale))>>12;
+        else screen->r_ytop = (w->w_origin.p_y + (tmp * w->w_scale)) >> 12;
     }
-*/
-
-    /* The following 8 lines are what remains when the redundant */
-    /* clipping to screen boundaries is removed.		 */
-
-    tmp = surface->r_xbot - w->w_surfaceArea.r_xbot;
-    screen->r_xbot = (w->w_origin.p_x + (tmp*w->w_scale))>>12;
-    tmp = surface->r_ybot - w->w_surfaceArea.r_ybot;
-    screen->r_ybot = (w->w_origin.p_y + (tmp*w->w_scale))>>12;
-    tmp = surface->r_xtop - w->w_surfaceArea.r_xbot;
-    screen->r_xtop = (w->w_origin.p_x + (tmp*w->w_scale))>>12;
-    tmp = surface->r_ytop - w->w_surfaceArea.r_ybot;
-    screen->r_ytop = (w->w_origin.p_y + (tmp*w->w_scale))>>12;
 }
 
 /*
