@@ -537,9 +537,8 @@ WireAddContact(newType, newWidth)
 
     /* Choose a contact type. */
 
-    for (i = 0; i < WireNumContacts; i++)
+    for (contact = WireContacts; contact != NULL; contact = contact->con_next)
     {
-	contact = WireContacts[i];
 	if ((contact->con_layer1 == oldType) &&
 		(contact->con_layer2 == WireType))
 	{
