@@ -31,11 +31,9 @@ static char rcsid[]="$Header: /ufs/repository/magic/wiring/wireTech.c,v 1.2 2001
 #include "wiring/wiring.h"
 #include "utils/malloc.h"
 
-/* Table to store contact information collected by this module: */
+/* Linked list to store contact information collected by this module: */
+Contact *WireContacts;
 
-#define MAXCONTACTS 10
-Contact *WireContacts[MAXCONTACTS];
-int WireNumContacts = 0;
 
 /*
  * ----------------------------------------------------------------------------
@@ -55,18 +53,14 @@ int WireNumContacts = 0;
 void
 WireTechInit()
 {
+    Contact *contact;
     int i;
 
-    for (i = 0; i < MAXCONTACTS; i++)
+    while (WireContacts != NULL)
     {
-	if (WireContacts[i] != 0)
-	{
-	    freeMagic((char *) WireContacts[i]);
-	    WireContacts[i] = NULL;
-	}
+	freeMagic((char *) WireContacts);
+	WireContacts = WireContacts->con_next;
     }
-
-    WireNumContacts = 0;
 }
 
 /*
@@ -104,11 +98,6 @@ WireTechLine(sectionName, argc, argv)
 	TechError("\"contact\" lines must have exactly 7 arguments.\n");
 	return TRUE;
     }
-    if (WireNumContacts >= MAXCONTACTS)
-    {
-	TechError("Sorry, can't have more than %d contacts.\n", MAXCONTACTS);
-	return TRUE;
-    }
 
     new = (Contact *) mallocMagic(sizeof(Contact));
     new->con_type = DBTechNoisyNameType(argv[1]);
@@ -140,8 +129,8 @@ WireTechLine(sectionName, argc, argv)
     }
     else new->con_surround2 = atoi(argv[6]);
 
-    WireContacts[WireNumContacts] = new;
-    WireNumContacts += 1;
+    new->con_next = WireContacts;
+    WireContacts = new;
 
     return TRUE;
 }
@@ -165,14 +154,13 @@ WireTechLine(sectionName, argc, argv)
 void
 WireTechFinal()
 {
-    /* Debugging code to print info about layers:
+    /* Debugging code to print info about layers: */
 
-    int i;
+/*
     register Contact *con;
 
-    for (i = 0; i < WireNumContacts; i += 1)
+    for (con = WireContacts; con != NULL; con = con->con_next)
     {
-	con = WireContacts[i];
 	TxPrintf("Contact type \"%s\", size %d connects\n",
 	    DBTypeLongName(con->con_type), con->con_size);
 	TxPrintf("    \"%s\" (overlap %d) and\n",
@@ -180,7 +168,7 @@ WireTechFinal()
 	TxPrintf("    \"%s\" (overlap %d)\n",
 	    DBTypeLongName(con->con_layer2), con->con_surround2);
     }
-    */
+*/
 }
 
 /*
@@ -197,11 +185,10 @@ Void
 WireTechScale(scalen, scaled)
     int scalen, scaled;
 {
-    int i;
     register Contact *con;
-    for (i = 0; i < WireNumContacts; i++)
+
+    for (con = WireContacts; con != NULL; con = con->con_next)
     {
-	con = WireContacts[i];
 	con->con_size *= scalen;
 	con->con_size /= scaled;
 
